/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.core;

import java.io.File;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.regex.Pattern;

import org.mozilla.javascript.Context;
import org.mozilla.javascript.NativeArray;
import org.mozilla.javascript.NativeJavaObject;
import org.mozilla.javascript.NativeObject;
import org.mozilla.javascript.ScriptRuntime;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.WrapFactory;

import com.unitesk.requality.core.attribute.AppliedEnumValue;
import com.unitesk.requality.core.attribute.Attribute;
import com.unitesk.requality.core.attribute.Attribute.ParameterAvailability;
import com.unitesk.requality.core.attribute.AttributeGenerator;
import com.unitesk.requality.core.attribute.AttributeType;
import com.unitesk.requality.core.attribute.ListValue;
import com.unitesk.requality.core.instancer.TreeInstancer;
import com.unitesk.requality.core.transaction.TransactionStorage;
import com.unitesk.requality.eclipse.core.ResourceStorage;
import com.unitesk.requality.eclipse.handlers.copypaste.CopyPasteHelper;
import com.unitesk.requality.eclipse.handlers.copypaste.PasteIntoHandler;
import com.unitesk.requality.nodetypes.VirtualNode;
import com.unitesk.requality.tools.Index;
import com.unitesk.requality.tools.RequalityCLI;

/**
 * 
 * Class that represents element in requirements catalog. 
 *
 */
public class TreeNode implements Iterable<TreeNode>{
    public static final String ATTR_TYPE = "_type";
    public static final String ATTR_NAME = "_name";
    protected static String NumberRegex = "(-){0,1}\\d*";
    public static final String ATTR_PREDICATE = "_predicate";
    public static final String ATTR_DESCRIPTION = "_description";
    public static final String ATTR_LINKS = "_links";
    public static final String ATTR_GENERATORS = "_generators";
    public static final String NODE_GROUP = "com.unitesk.nodegroup.requality";
    //attribute for common undefined change over node
	public static final String ATTR_UNDEFINED = "_undefined";
	public static final String ATTR_ERROR = "_error";
	public static final String SORT_BY_LOCATIONS="_sort_by_locations";
	public static final String ATTR_REVLINKS = "__revlinks";
	public static final String SORT_SEP_TYPES = "_separate_types_in_sort";
	public static final String SORT_BY_INDEX = "_sort_by_index";
	public static final String ATTR_IGNORE_NAME = "ignoreName";

    private UUID uuid;
    private final TreeDB db;
    protected String id;
    UUID parent;
    protected List<UUID> children = new ArrayList<UUID>();
	protected List<UUID> outchildren = null;
    Map<String, List<Attribute>> attributes = new LinkedHashMap<String, List<Attribute>>();
    private boolean disposed = false;
	private Map<String, Set<String>> dependencies = new HashMap<String,Set<String>>();
	private Map<String, Set<String>> rdependencies = new HashMap<String,Set<String>>();
	
    private Map<String,TreeNode>linkCahce=new HashMap<String,TreeNode>();
    private Map<String,String>linkExpr = new HashMap<String, String>();
    //link type(attr name)
    private Map<TreeNode,Map<String,INodeChangeListener>>revlinks=null;
    protected void saveRevLinks() {
		List k = new ArrayList();
		for(TreeNode node:revlinks.keySet())
			for(String s:revlinks.get(node).keySet())
				k.add((node.isVirtual()?node.getQualifiedId():node.getUUId())+"|"+s);
		putAttribute(new Attribute(this,AttributeType.LIST,ATTR_REVLINKS,new ListValue(AttributeType.STRING,k)));
	}
    private INodeChangeListener formRevChangeListener(final TreeNode revowner, final String key){
    	return new INodeChangeListener() {
			@Override
			public void moved(TreeNode source, TreeNode oldParent, TreeNode newParent) {
				if(!correct(TreeNode.this,key))
					removeLink();
			}
			
			@Override
			public void deleted(TreeNode source) {
				removeLink();
			}
			private void removeLink(){
				revowner.linkCahce.remove(key);
				revowner.linkExpr.remove(key);
				TreeNode.this.removeRevLink(TreeNode.this, key);
				TreeNode.this.removeChangeListener(this);
				TreeNode.this.saveAttributes();
			}
			
			@Override
			public void attributeChange(TreeNode source, String what) {
				if(!correct(source,key))
					removeLink();
			}

			private boolean correct(TreeNode source, String savedName) {
				return db.getLinkResolver().verify(TreeNode.this, revowner.linkExpr.get(savedName));
			}
		};
    }
    protected void addRevLink(final TreeNode node, final String key){
    	revlinks = getRevertLinks();
    	if(!revlinks.containsKey(node))
    		revlinks.put(node, new HashMap<String,INodeChangeListener>());
    	if(!revlinks.get(node).containsKey(key)){
    		revlinks.get(node).put(key,formRevChangeListener(node, key));
    		node.addChangeListener(revlinks.get(node).get(key));
    		saveRevLinks();
    	}
    }
    
    protected void removeRevLink(TreeNode node, String key){
    	revlinks = getRevertLinks();
    	if(revlinks.containsKey(node)&&revlinks.get(node).containsKey(key)){
    		node.removeChangeListener(revlinks.get(node).get(key));
    		revlinks.get(node).remove(key);
    		saveRevLinks();
    	}
    }
    
    public Map<TreeNode, Map<String,INodeChangeListener>> getRevertLinks(){
    	if(revlinks == null){
    		List pairs = getListAttributeValue(ATTR_REVLINKS);
			revlinks = new HashMap<TreeNode, Map<String,INodeChangeListener>>();
    		if(pairs != null)
    			for(Object o : pairs){
    				String s = (String)o;
    				String path = s.substring(0,s.indexOf("|"));
    				TreeNode node = null;
    				try{
    					if(path.startsWith("/"))
    						node = db.getNode(path);
    					else
    						node = db.getNode(UUID.fromString(path));
    				}
    				catch(Exception e){
    					//broken link
    				}
    				String key = s.substring(s.indexOf("|")+1);
    				if(node!=null){
    					TreeNode linked = node.getLinkAttributeValue(key);
    					if(linked!=null && linked.getUUId().equals(getUUId())){
    						if(!revlinks.containsKey(node))
    							revlinks.put(node, new HashMap<String,INodeChangeListener>());
    						revlinks.get(node).put(key,formRevChangeListener(node, key));
    					}
    				}
    			}
    	}
    	return revlinks;
    }
    
    protected boolean containsChildUUID(UUID uid){
    	synchronized (children) {
    		if(outchildren==null)
    			getChildrenUUIds();
    		return children.contains(uid);
    	}
    }
    public String getMultiTypeLabel(){
    	return getTypeLabel()+"s"; 
    }
    public final String getTypeLabel(){
    	return db.getNodeType(getType()).getVisibleClassName();
    }
    protected void addChildUUID(UUID uid){
    	synchronized (children) {
    		children.add(uid);
   			outchildren = null;
   			VirtualNode.updateClonesChildren(this);
    	}
    }
    protected void removeChildUUID(UUID uid){
    	synchronized (children) {
    		children.remove(uid);
    		//not vnode
    		if(outchildren!=null && outchildren.contains(uid))
    			outchildren.remove(uid);
    		else //vnode, todo what to do
    			outchildren = null;
    		VirtualNode.updateClonesChildren(this);
    	}
    }
	public void setChildrenUUIDs(List<UUID> children2) {
		synchronized (children) {
			children = new ArrayList<UUID>(children2);
			outchildren = null;
		}
	}
    public class TreeNodeChildIterator implements Iterator<TreeNode>{
    	private int index;

    	@Override
        public boolean hasNext() {
            return index < sizeChildren();
        }

    	@Override
        public TreeNode next() {
            return getChild( index++ );
        }

    	@Override
        public void remove() {
            removeChild( getChild( index ) );            
        }
    }

    /**
     * Elements can be grouped to support versions of storing formats for groups
     * NODE_GROUP is related to default group. 
     * @return identifier of node group
     */
    public String getGroup(){
    	return NODE_GROUP;
    }
    public boolean isVirtual(){
    	if(db == null)
    		return false;
    	return db.isVirtual();
    }
    
    /**
     * Method to get error messages appears during report generation 
     * @return error messages if any
     */
    public String getError(){
    	if(db.getStorage() instanceof TreeInstancer)
    		db.updateNodeAttributes(this);
    	String s = getStringAttributeValue(ATTR_ERROR);
    	if(s != null)
    		return s;
    	return "";
    }
    public TreeNode(TreeDB db, UUID uuid) {
        if (uuid == null)
            uuid = UUID.randomUUID();
        this.uuid = uuid;
        this.db = db;
        this.disposed = false;
        String indexStr = (String)getAttributeValue( TreeDB.INDEX_ATTRIBUTE );
        Attribute a = getAttribute( TreeDB.ORDER_ATTRIBUTE );
        String id = getId();
        if(indexStr == null && a == null && id != null)
            index = Index.create( id );
    }
    
    /**
     * Method to set predicate that affect on node visibility in reports
     * @param pred predicate
     */
    public void setPredicate(String pred){
    	if(pred.equals(""))
    		removeAttribute(ATTR_PREDICATE);
    	setPredicate(pred, false);
    }

    private void setPredicate(String pred,boolean temp) {
    	boolean changed = true;
    	UUID uid = null;
    	if(isVirtual()){
    		//TODO check correctness
    		uid = getCloneParentUUId();
    		Map<String,Attribute> attrs = VirtualNode.getVirtualStorage(getTreeDB()).getChangedAttributes(uuid);
    		if(attrs!=null)
    			changed = attrs.containsKey(ATTR_PREDICATE);
    		if(uid==null){
    			changed = false;
    			uid = uuid;
    		}
    	}
   		putAttribute(new Attribute(changed?uuid:uid, AttributeType.STRING, ATTR_PREDICATE,pred, pred));
    }
    /**
     * Method to get predicate that affect on node visibility in reports
     * @return predicate
     */
    public String getPredicate() {
    	String pred = (String)getAttributeValue(ATTR_PREDICATE);
        if (pred == null) {
            pred = "";
        }
        String fullpred = pred.toString();
        TreeNode node = getParent(true);
        if(node != null){
        	pred = node.getPredicate();
        	if(!pred.equals(""))
        		if(!fullpred.equals(""))
        			fullpred = "("+pred.toString()+") && ("+fullpred+")";
        		else
        			fullpred = pred.toString();
        }
        if(isVirtual()){
        	TreeDB db = getTreeDB();
        	if(!db.isVirtual())
        		VirtualNode.getVirtualTreeDB(db);
        	if(getCloneParentUUId()!=null){
        		pred = getCloneParent().getPredicate();
        		if(!pred.equals(""))
        			if(!fullpred.equals(""))
        				fullpred = "("+getCloneParent().getPredicate()+") && ("+fullpred+")";
        			else
        				fullpred = pred;
        	}
        }
        return fullpred;
    }

    public TreeNode getCloneParent(){
    	UUID uid = VirtualNode.getVirtualStorage(db).getCloneParent(uuid);
    	if(uid == null)
    		return null;
    	return VirtualNode.getVirtualTreeDB(db).getNode(uid);
    }
    public UUID getCloneParentUUId(){
    	return VirtualNode.getVirtualStorage(db).getCloneParent(uuid);
    }
    /**
     * Method to set type of element in form of string
     * Types names are usually was in constants like Requirement.TYPE_NAME
     * @param type string name of type
     */
    public void setType(String type) {
    	rawPutAttribute(new Attribute(uuid, AttributeType.STRING, ATTR_TYPE, type));
    }
    /**
     * get UUID - unique identificator over element.
     * @link java.util.UUID
     * @return uuid of element
     */
    public final UUID getUUId() {
        return uuid;
    }

    /**
     * Set identifier of element. Id is identical to name of corresponding file on storage.
     * Needs to be unique for siblings. Id also can form indexes that can be used in sorting. 
     * @param id new identifier of element.
     */
    public void setId(String id) {
    	id = id.replace((char)160, ' ');
        db.setId(this, id);
        //TODO update?
        updateTreeNodes(this, "_id");
    }
    /**
     * Get identifier of element. Id is identical to name of corresponding file on storage.
     * Needs to be unique for siblings. Id also can form indexes that can be used in sorting. 
     * @return current identifier of element.
     */
    
    public String getId() {
        return id;
    }

    /**
     * @return String name of node type
     */
    public String getType() {
        return (String) getAttributeValue(ATTR_TYPE);
    }
    /**
     * Set parent element
     * @param parent new parent element
     */
    public void setParent(TreeNode parent){
    	TreeNode par = getParent(true);
    	TreeDB masterDB = db;
    	TreeNode newParent = parent;
    	//if(parent.isVirtual()){
    		//newParent = masterDB.getRootNode();
    	//}
    	if(isVirtual()){
    		masterDB = VirtualNode.getBaseTree(db);
    		CopyPasteHelper cph = CopyPasteHelper.getInstance();
    		List<TreeNode> nodes = new ArrayList<TreeNode>();
			List<NodeDesc> ns = cph.getNodes();
    		nodes.add(this);
    		cph.setNodes(nodes);
    		PasteIntoHandler pih = new PasteIntoHandler();
			String s = (String) pih.pasteNodes(newParent,null, true, true, true, true);
			if(parent.isVirtual())
	    		(parent.getTreeDB().getStorage()).moveNodeTo(uuid, parent.getUUId());
			cph.setNodesDescs(ns);
			//TODO error to show
    		saveAttributes();
			if(s!=null)
				return;
    		
    	}else{
    		for(String s:par.getAttributeKeys())
    			if(par.getAttribute(s).getAvailability()!=ParameterAvailability.LOCAL)
    				removeAttribute(s);
    		db.moveNode(this, parent);
    		for(String s:newParent.getAttributeKeys())
    			if(newParent.getAttribute(s).getAvailability()!=ParameterAvailability.LOCAL)
    				putAttribute(newParent.getAttribute(s).copy());
    		saveAttributes();
    	}
    	fireNodeMoved(par,newParent);
    	fireGUINodeMoved(par,newParent);
    }
    private void fireGUINodeMoved(TreeNode par, TreeNode newParent) {
    	if(guilistenerList!=null)
    		for(INodeChangeListener i:guilistenerList)
    			i.moved(this, par, newParent);
	}

	private void fireNodeMoved(TreeNode par, TreeNode newParent) {
		if(listenerList!=null)
			for(INodeChangeListener i:listenerList)
				i.moved(this, par, newParent);
	}

	/**
     * Get unique identifier of parent element. It does not cause parent element loading.
     * @return unique identifier of parent element
     */
    public UUID getParentUUId() {
    	return getParentUUId(false);
    }
    public UUID getParentUUId(boolean local) {
    	if(local)
    		return parent;
    	TreeNode node = getParent(local);
    	if(node!=null)
    		return node.getUUId();
    	return parent;
    }
    /**
     * Remove child element from current
     * @param node element to remove
     */
    public void removeChild(TreeNode node)  {
        db.removeNode(node);
        synchronized (children) {
        	children.remove(node.getUUId());
        	if(outchildren!=null && outchildren.contains(node.getUUId()))
	        	outchildren.remove(node.getUUId());
    	    else
        		outchildren = null;
        	outchildren = VirtualNode.updateVisablility(db, children);
        	if(!isDisposed())
        		db.fireNodeGUIRemoved(node, true);
	    }
        TreeInstancer ti = VirtualNode.getVirtualStorage(node.getTreeDB());
        for(UUID uid:ti.getClones(node.getUUId()))
        	ti.remove(uid);
    }
    /**
     * Set text description for element. As an example may be description of requirements.
     * @param description new description of element
     */
    public void setDescription(String description) {
    	if(description==null || description.equals(""))
    		removeAttribute(ATTR_DESCRIPTION);
    	else
    		putAttribute(new Attribute(uuid, AttributeType.STRING,
                ATTR_DESCRIPTION, description));
    }
    /**
     * Get current text description for element. As an example may be description of requirements.
     * This method returns description prepared for usage in reports. Picture paths are replaced 
     * to corresponding paths in report folder.
     * @param prefix optional prefix for description
     * @param postfix optional postfix for description
     * @param basePath path to report folder
     * @return description prepared to be used in report
     */
    public String getDescription(String prefix, String postfix, String basePath) {
        String desc = getDescription();
        if(desc == null)
        	return desc;
    	return RequalityCLI.getReportDescription(this,
        		getTreeDB().getStorage().getNodeResourcePath(getUUId()) + File.separator,
                desc, prefix, postfix, basePath);
    }

    /**
     * Get current text description for element. As an example may be description of requirements.
     * This method returns description prepared for usage in reports. Picture paths are replaced 
     * to corresponding paths in report folder.
     * @param basePath path to report folder
     * @return description prepared to be used in report
     */
    public String getDescription(String basePath) {
        String result = getDescription("<div>", "</div>", basePath);
        return result;
    }
    /**
     * Return element description to be shown in uni editor and properties panels.
     * Paths for resources(for ex. images) are not processed
     * @return element description
     */
    public String getDescription() {
        return getDescription(false);
    }

    /**
     * Return element description to be shown in uni editor and properties panels
     * @param process process resource path flag
     * @return element description
     */
    public String getDescription(boolean process) {
        if (getAttribute(ATTR_DESCRIPTION) != null) {
            String result = (String) getAttribute(ATTR_DESCRIPTION)
                    .getValue();
            if (process) {
                result = processResourcesPath(result);
            }
            return result;
        }
        return null;
    }

    /**
     * Process paths in description, replace placeholder ${node.resURL} to actual path to images 
     * @param desc description of element
     * @return description of element with correct path to resources of node 
     */
    public String processResourcesPath(String desc) {
        String resPath = getResourcesPath();
        String result = desc;
        if(resPath!=null)
        	result = desc.replace("${node.resURL}", resPath);
        return result;
    }
    /**
     * Get tree database in which this element exists
     * @return tree database
     */
    public TreeDB getTreeDB() {
        return db;
    }

    /**
     * Set unique identifier of element
     * @link java.util.UUID
     * @param uuid new unique identifier
     */
    void setUUId(UUID uuid) {
        this.uuid = uuid;
    }

    /**
     * Get parent of current element. Causes element loading if it is not in tree database.
     * @return parent element
     */
    public TreeNode getParent() {
    	return getParent(false);
    }
    public TreeNode getParent(boolean local) {
        if (parent == null) {
            // no parent
            assert db.getRootNode().equals(this);
            return null;
        }
        try{
        	TreeNode node = null;
        	if(!local && db.getStorage().canLoad(parent) && VirtualNode.TYPE_NAME.equals(db.getTreeNodeType(parent))){
        		boolean find = false;
        		for(Attribute a:db.getStorage().getAttributes(parent))
        			if(a.getKey().equals(VirtualNode.ATTR_HIDDEN_STATUS)&&a.getValue().equals("true"))
        				find = true;
        		if(find)
        			node = db.getNode(parent).getParent(local);
        		else
         			node = db.getNode(parent);
        	}else
        		if(parent!=null)
        			node = db.getNode(parent);
        		else
        			return null;
        	return node;
        }
        catch(Exception e){
        	e.printStackTrace();
        }
        return null;
    }
    /**
     * Get children count for current element.
     * @return children count
     */
    public int sizeChildren() {
    	synchronized (children) {
    		if(outchildren==null)
				getChildrenUUIds();
    		if(outchildren!=null)
    			return outchildren.size();
			return children.size();
		}
    }
    /**
     * Get child element with index i in children uuid list
     * @param i index of child in uuid list
     * @return child element
     */
    public TreeNode getChild(int i) {
    	synchronized (children) {
    		if(outchildren==null)
				getChildrenUUIds();
    		if(outchildren!=null)
	    		return db.getNode(Collections.unmodifiableList(outchildren).get(i));
    		return db.getNode(Collections.unmodifiableList(children).get(i));
    	}
    }
    /**
     * Get path to element in form of identifier(id) of element and all its ancestors
     * in form of /RootId/.../ElementParentId/ElementId
     * @return path to element
     */
    public String getQualifiedId() {
    	if(isDisposed())
    		return "";
        if (getParent() == null) {
            return "/";
        }
        String parentQId = db.getNode(parent).getQualifiedId();
        if (parentQId.equals("/")) {
            return parentQId + id;
        }
        return parentQId + "/" + id;
    }
    /**
     * Method to locate child with given id. Return null if it is not found. 
     * @param id id of child to find
     * @return child element
     */
    public TreeNode findChild(String id) {
    	id = id.replace((char)160, ' ');
    	List<UUID> uids = getRawChildrenUUIDs();
        for (UUID uid:uids) {
            TreeNode child = db.getNode(uid);
            if (child.getId().equals(id)) {
                return child;
            }
        }
        return null;
    }
    /**
     * Get list of children UUIDs
     * @return list of children UUIDs
     */
    public static String join(String r[],String d)
    {
            if (r.length == 0) return "";
            StringBuilder sb = new StringBuilder();
            int i;
            for(i=0;i<r.length-1;i++)
                sb.append(r[i]+d);
            return sb.toString()+r[i];
    }
    public List<UUID> getChildrenUUIds() {
    	synchronized (children) {
   			List<UUID> ch2 = new ArrayList<UUID>();
   			boolean vchild = false;
   			for(UUID uid:children){
	   			if(!getTreeDB().getStorage().canLoad(uid))
   					continue;
   				else
		   			if(VirtualNode.TYPE_NAME.equals(getTreeDB().getStorage().getNodeType(uid))){
	   					VirtualNode node = (VirtualNode) getTreeDB().getNode(uid);
	   					if(!node.isHidden())
		   					ch2.add(uid);
	   					else{
		   					vchild = true;
	   						ch2.addAll(node.getChildrenUUIds());
	   					}
	   				}else
		   				ch2.add(uid);
   			}
   			if((isVirtual()||vchild)&&ch2.size()>0){
	   			ch2 = VirtualNode.updateVisablility(getTreeDB(),ch2);
   			}
   			outchildren = ch2;
   			VirtualNode.updateClonesChildren(this);
   			return outchildren;
   		}
    }

    @Override
    public String toString() {
    	if(this.getClass() == TreeNode.class)
    		return id + "(" + getType() + ")";
    	return getNameOrId();
    }
    
    /**
     * Create child element with given id and type "Node"
     * @param id id of created element
     * @return new child element
     */
    public TreeNode createChild(String id) {
        if (findChild(id) != null)
            throw new IllegalArgumentException("element with id '" + id
                    + "' already exists");
        id = id.replace((char)160, ' ');
		synchronized (children) {
        	outchildren = null;
        	if (getType().equals("DBRoot")) {
            	TreeNode node = db.createChildNode(uuid, id, "Node");
            	return node;
        	}
        	TreeNode node = db.createChildNode(uuid, id, getType());
        	return node;
        }
    }
    public TreeNode createChild(String id, String type) {
    	return createChild(id,type,TreeDB.EMPTY_TEMPLATE);
    }
    /**
     * Create child element with given id and type type
     * @param id id of created element
     * @param templ 
     * @type string type of element
     * @return new child element
     */
    public TreeNode createChild(String id, String type, String templ) {
    	synchronized (children) {
        	outchildren = null;
        	id = id.replace((char)160, ' ');
        	TreeNode node = db.createChildNode(uuid, id, type, templ);
        	return node;
        }
    }
    /**
     * Delete this element
     */
    public void delete(){
        getParent(true).removeChild(this);
    }
    public Attribute rawPutAttribute(Attribute a){
    	Attribute result = getAttribute(a.getKey());    	
    	putAttributeInList(a.getKey(),a);
    	return result;
    }
    private void putAttributeInList(String key, Attribute a) {
    	List<Attribute> al = attributes.get(key);
    	if(al==null)
    		al = new LinkedList<Attribute>();
    	int p = 0;
    	Iterator<Attribute> it = al.iterator();
    	Attribute ia = null;
    	boolean stop = false;
    	while(it.hasNext()){
    		ia = it.next();
    		if(!ia.isMoreVisible(a,getUUId()))
    			stop = true;
    		if(ia.getOrigin().equals(a.getOrigin())){
    			it.remove();
    			if(!stop)
    				p--;
    		}
    		if(!stop)
    			p++;
    	}
    	al.add(p, a);
    	attributes.put(key, al);
	}

	/**
     * Put attribute to this element. If name of attribute is equal to one stored in element then stored attribute will be 
     * replaced by new attribute. To save all changes over attributes, saveAttributes() method needs to be called
     * @param a attribute to add or replace
     * @return value of old attribute with same name
     */
    public Attribute putAttribute(Attribute a) {
    	return putAttribute(a, false, true);
    }
    private boolean correctNested(Attribute a){
    	if(!a.getOrigin().equals(getUUId()))
    	{
//    		List<Attribute> atrs = getParent(true).attributes.get(a.getKey());
//    		if(atrs == null || atrs.isEmpty())
//    			return false;
//    		for(Attribute ca : atrs)
//    			if(ca.getKey().equals(a.getKey())&& ca.isAvaiableOn(this))
//    				return true;
    		return a.isAvaiableOn(this);
    	}
    	return true;
    }
    public List<TreeNode> propogateAttributeChange(Attribute a){
    	List<TreeNode> nd = new ArrayList<TreeNode>();
    	if(a!=null && a.getAvailability()!=ParameterAvailability.LOCAL)
			for(UUID child:getRawChildrenUUIDs())
				if(db.isLoaded(child)){
					TreeNode node = db.getNode(child);
					if(a.isAvaiableOn(node)){
					
						Attribute a2 = a.copy();
						//	a2.setAvailability(a.getAvailability().equals(ParameterAvailability.DIRECT_CHILDREN)?ParameterAvailability.LOCAL:ParameterAvailability.SUBTREE);
						Attribute b = node.putAttribute(a2,false,true);
						nd.add(node);
					}
				}
    	return nd;
    }
    public Attribute putAttribute(Attribute a, boolean notchange, boolean propogate) {
    	if(!correctNested(a))
    		return null;
    	AttributeGenerator ag = a.getGenerator();
    	if(ag!=null){
    		Object value = ag.getValue(this);
    		ParameterAvailability pa = a.getAvailability();
    		a = new Attribute(a.getOrigin(),a.getRawType(),a.getKey(),null,value,ag.copy(false));
    		a.setAvailability(pa);
    	}
    	//last attribute value
        Attribute old = getAttribute(a.getKey(), a.getOrigin());
        if(a.equals(old)&&!a.isValueRaw())
         	return null;
        Attribute vis = getAttribute(a.getKey());
        if(old!=null){
        	removeFromAttributeList(a.getOrigin(), a.getKey());
        	//reactOnAttrRemove(old.getKey(),old.getOrigin(),notchange,propogate);
        	if(old.getOrigin().equals(vis.getOrigin()))
        		removeDependencies(a.getKey());
        }
        
        boolean chvis = (vis==null || vis.getOrigin().equals(a.getOrigin())&&!vis.equals(a)||a.isMoreVisible(vis,getUUId()));
        if(isVirtual()&&chvis){
        	buildDependencies(a);
        	if(dependencies.containsKey(a.getKey()))
        		a = getUpdatedAttribute(a.getOrigin()!=uuid?Attribute.deepCopy(a):a);
        }
        Map<String, Attribute> diff = new HashMap<String,Attribute>();
        if(chvis)
        	diff.put(a.getKey(), a);
        rawPutAttribute(a);
        if(chvis)// || !a.equals(result))){
        	diff.putAll(updateDependencies(a,notchange));
        //if(VirtualNode.TYPE_NAME.equals(this.getType()))
        updateDirectCloneAttributes();
        //	VirtualNode.updateClonesAttributes(this, diff);
        return old;
    }
	
	public Attribute getAttribute(String key, UUID origin) {
		List<Attribute> l = attributes.get(key);
		if(l == null || l.isEmpty())
			return null;
		Iterator<Attribute> atrs = l.iterator();
		while(atrs.hasNext()){
			Attribute a = atrs.next();
			if(a.getOrigin().equals(origin))
				return a;
		}
		return null;
	}

	private List<TreeNode> reactOnAttrRemove(String key, UUID origin, boolean notchange, boolean propogate) {
		List<TreeNode>changed = new ArrayList<TreeNode>();
		for(UUID child:getRawChildrenUUIDs())
    		if(db.isLoaded(child)){
    			TreeNode node = db.getNode(child);
    			Attribute a2 = node.getAttribute(key,origin);
    			if(a2!=null){
    				Attribute k = node.removeAttribute(origin, key, notchange, propogate);
    				changed.add(node);
    			}
    		}
		return changed;
	}

	public void rawSaveAttributes(Set<String> set) {
		db.saveAttributes(uuid, set, getAttributes().values());
    	//updateDirectCloneAttributes();
	}
	private Collection<Attribute> getFullAttributesCollection() {
		Collection<Attribute>al = new ArrayList<Attribute>();
		for(List<Attribute> list:attributes.values())
			al.addAll(list);
		return al;
	}
	private void updateDirectCloneAttributes() {
        if(uuid!=null && db!=null && db.getStorage()!=null && uuid!=db.getStorage().getRootNode()&&!isVirtual()
        		&&!VirtualNode.getVirtualStorage(getTreeDB()).hasCloneParent(uuid))
        			VirtualNode.getVirtualTreeDB(getTreeDB()).getNode(uuid).attributes = attributes;
	}
	
	/**
     * Remove attribute with given attribute name. To save all changes over attributes, saveAttributes() method needs to be called
     * @param key name of attribute to delete
     * @return removed attribute
     */
    public Attribute removeAttribute(String key) {
    	return removeAttribute(getUUId(),key,false,true);
    }
    private Map<String, Attribute> updateDependencies(Attribute a,boolean notchange){
    	Map<String,Attribute> changes = new HashMap<String,Attribute>();
    	Set<String> set = rdependencies.get(a.getKey());
    	if(set!=null)
    		set = new HashSet<String>(set);
    	else
    		set = new HashSet<String>();
    	for(String s:set)
    	{
    		Attribute dep = getAttribute(s);
    		if(dep != null){
    			if(dep.isGenerated()){
    				AttributeGenerator ag2 = dep.getGenerator();
    				if(ag2!=null){
    					ag2.updateValue(this);
    				}else
        				removeAttribute(getUUId(),s,notchange, true);
    			}
    			Attribute up = getUpdatedAttribute(dep.getOrigin().equals(uuid)?dep:Attribute.deepCopy(dep));
    			if(!up.getValue().equals(dep.getValue()))
    				putAttribute(up,notchange,true);
    		}
    		Attribute ndep = getAttribute(s);
    		if(ndep==null)
    			changes.put(s, null);
    		else
    			if(!ndep.equals(dep))
    				changes.put(s, ndep);
    			
    	}
		return changes;
    }
    public Attribute removeAttribute(UUID origin, String key,boolean notchange, boolean propogate) {
    	Attribute back = getAttribute(key);
    	Attribute a = removeFromAttributeList(origin, key);
    	AttributeGenerator ag = null;
    	//if(a!=null)
    		//reactOnAttrRemove(key,a.getOrigin(),notchange,propogate);
    	Attribute n = getAttribute(key);
    	if(back!=null && (n==null || !n.equals(back))){
    		removeDependencies(key);
    		if(n!=null)
    			//TODO fix
    			putAttribute(n, notchange, propogate);
    			
    	}
        return a;
    }
    private Attribute removeFromAttributeList(UUID origin, String key) {
    	List<Attribute> list = attributes.get(key);
		if(list == null || list.isEmpty())
			return null;
		Iterator<Attribute> it = list.iterator();
		while(it.hasNext()){
			Attribute a = it.next();
			if(a.getOrigin().equals(origin)){
				it.remove();
				if(list.isEmpty())
					attributes.remove(key);
				else
					attributes.put(key, list);
				return a;
			}
		}
		return null;
	}

	/**
     * Get attribute with given attribute name.
     * @param key name of attribute 
     * @return attribute with given name 
     */
    public Attribute getAttribute(String key) {
    	List<Attribute> atrs = attributes.get(key);
    	if(atrs == null || atrs.isEmpty())
    		return null;
        return atrs.iterator().next();
    }
    /**
     * Get value of attribute with given name.
     * @param key name of attribute 
     * @return value of attribute with given name
     */
    public Object getAttributeValue(String key) {
        return getAttributeValue(key, null);
    }
    /**
     * Get value of attribute with given key. If attribute with given key is not exists then return value of "def"
     * @param key name of attribute
     * @param def default return value if attribute is not found
     * @return value of attribute or "def" if it is not found
     */
    public Object getAttributeValue(String key, Object def) {
        Attribute value = getAttribute(key);
        return value != null ? value.getValue() : def;
    }
    /**
     * Get value of attribute with given key casted to TreeNode type. If attribute with given key is not exists then return value of "def"
     * @param key name of attribute
     * @param def default return value if attribute is not found
     * @return value of attribute or "def" if it is not found
     */

    public TreeNode resolveLink(String expr, TreeNode def, final String savedName){
    	if(!linkCahce.containsKey(savedName)||(linkCahce.get(savedName)==null)||!expr.equals(linkExpr.get(savedName))||!getTreeDB().getLinkResolver().verify(linkCahce.get(savedName), expr))
    	{
        	linkExpr.put(savedName, expr);
    		linkCahce.put(savedName, expr != null ? getTreeDB().getLinkResolver().resolveSingle(this,expr) : def);
    		TreeNode node = linkCahce.get(savedName);
    		final TreeNode linksource = this;
    		if(node!=null&&(!node.getRevertLinks().containsKey(this)||!node.getRevertLinks().get(this).containsKey(savedName))){
    			node.addRevLink(this, savedName);
    			//TODO why?
    			if(!node.getUUId().equals(getUUId()))
    				node.saveAttributes();
    		}
    	}
    	return linkCahce.get(savedName);
    }
    public TreeNode getLinkAttributeValue(String key, TreeNode def){
    	String val = null;
    	if(key.contains("[")){
    		String l = key.substring(key.indexOf("[")+1); 
    		Attribute a = getAttribute(key.substring(0,key.indexOf("[")));
    		if(a == null || a.getResultType()!=AttributeType.LIST)
    			return null;
    		ListValue lv = a.getGRawValue();
    		//Object[]o = (Object[]) a.getRawValue();
    		int ind = Integer.parseInt(l.substring(0,l.indexOf("]")));
    		if(ind == -1 || ind>=lv.getValues().size())
    			return null;
    		Object i = lv.getValues().get(ind);
    		l = l.substring(l.indexOf("]")+1);
    		while(lv.getType() == AttributeType.LIST){
    			lv = (ListValue)i;
    			if(l.startsWith("["))
    				l = l.substring(1);
    			i = lv.getValues().get(Integer.parseInt(l.substring(0,l.indexOf("]"))));
        		l = l.substring(l.indexOf("["));
    		}
    		val = i.toString();
    	}else{
    		Attribute a = getAttribute(key);
    		if(a!=null && a.getResultType() == AttributeType.REFERENCE)
    			val = a.getRawValue().toString();
    	}
    	if(val == null)
    		return null;
    	return resolveLink(val, def, key);
    }
    public TreeNode getLinkAttributeValue(String key){
    	return getLinkAttributeValue(key, null);
    }
    /**
     * Get value of attribute with given key casted to int type. If attribute with given key is not exists then return value of "def"
     * @param key name of attribute
     * @param def default return value if attribute is not found
     * @return value of attribute or "def" if it is not found
     */
    public int getIntAttributeValue(String key, int def) {
        Attribute value = getAttribute(key);
        return value != null ? (Integer) value.getValue() : def;
    }
    /**
     * Get value of attribute with given key casted to int type. 
     * @param key name of attribute 
     * @return value of attribute with given name
     */

    public int getIntAttributeValue(String key) {
        return getIntAttributeValue(key, 0);
    }
    /**
     * Get value of attribute with given key casted to float type. If attribute with given key is not exists then return value of "def"
     * @param key name of attribute
     * @param def default return value if attribute is not found
     * @return value of attribute or "def" if it is not found
     */
    public float getFloatAttributeValue(String key, float def) {
        Attribute value = getAttribute(key);
        return value != null ? (Float) value.getValue() : def;
    }

    /**
     * Get value of attribute with given key casted to float type. 
     * @param key name of attribute 
     * @return value of attribute with given name
     */
    public float getFloatAttributeValue(String key) {
        return getFloatAttributeValue(key, 0);
    }
    /**
     * Get value of attribute with given key casted to boolean type. If attribute with given key is not exists then return value of "def"
     * @param key name of attribute
     * @param def default return value if attribute is not found
     * @return value of attribute or "def" if it is not found
     */
    public boolean getBooleanAttributeValue(String key, boolean def) {
        Attribute value = getAttribute(key);
        if(value!=null)
        {
        	if(value.getValue() instanceof Boolean)
        		return (Boolean) value.getValue();
        	return Boolean.parseBoolean(value.getValue().toString());
        }
        return  def;
        //return value != null ? (Boolean) value.getValue() : def;
    }

    /**
     * Get value of attribute with given key casted to boolean type. 
     * @param key name of attribute 
     * @return value of attribute with given name
     */
    public boolean getBooleanAttributeValue(String key) {
        return getBooleanAttributeValue(key, false);
    }

    /**
     * Get value of attribute with given key casted to string type. If attribute with given key is not exists then return value of "def"
     * @param key name of attribute
     * @param def default return value if attribute is not found
     * @return value of attribute or "def" if it is not found
     */
    public String getStringAttributeValue(String key, String def) {
        Attribute value = getAttribute(key);
        return value != null ? (String) value.getValue() : def;
    }
    /**
     * Get value of attribute with given key casted to string type. 
     * @param key name of attribute 
     * @return value of attribute with given name
     */
    public String getStringAttributeValue(String key) {
        return getStringAttributeValue(key, null);
    }

    /**
     * Get value of attribute with given key casted to list of string. 
     * @param key name of attribute
     * @return value of attribute or "def" if it is not found
     */
    @SuppressWarnings("unchecked")
    public List<Object> getListAttributeValue(String key) {
        return (List<Object>) getAttributeValue(key, null);
    }
    /**
     * Get set of names of available attributes
     * @return set of names of attributes
     */
    public Set<String> getAttributeKeys() {
        return Collections.unmodifiableSet(getAttributes().keySet());
    }
    /**
     * Get map of available attributes. Key is attribute name, value - corresponding attribute
     * @return map of attributes
     */
    public Map<String, Attribute> getAttributes() {
    	Map<String,Attribute> retMap = new HashMap<String,Attribute>();
    	for(String s:attributes.keySet())
    		if(attributes.get(s)!=null && !attributes.get(s).isEmpty())
    			retMap.put(s, attributes.get(s).iterator().next());
        return Collections.unmodifiableMap(retMap);
    }
    
    /**
     * Save all attribute changes in storage. If it is not called after changes then all
     * attribute changes can be lost.
     */
    public void saveAttributes()  {
    	Collection<Attribute>oldAttrs = db.getStorage().getAttributes(uuid);
    	Map<String,Attribute> diff = new HashMap<String, Attribute>(),
    						  adiff = new HashMap<String, Attribute>();
    	Set<String> newAttributesK = new HashSet<String>();
    	newAttributesK.addAll(this.attributes.keySet());
    	if(oldAttrs!=null)
	    	for(Attribute a:oldAttrs){
	    		if(a.getKey().startsWith("__"))
	    			continue;
	    		newAttributesK.remove(a.getKey());
	    		if(!this.attributes.containsKey(a.getKey()) || 
	    				a!=null && 
	    					!a.equals(getAttribute(a.getKey()))){
	    			diff.put(a.getKey(),a);
	    		}
	    	}
    	for(String s:newAttributesK)
    	{
    		Attribute a = getAttribute(s);
    		if(a != null)
    			diff.put(s,a);
    		else
    			diff.put(s, null);
    	}
    	//nothing changes - nothing to worry
    	if(diff.isEmpty())
    		return;
		fireNodeAttributeChange(diff.keySet());
    	rawSaveAttributes(diff.keySet());
    	for(String s:diff.keySet()){
    		Attribute o = diff.get(s),n =attributes.containsKey(s)? getAttribute(s):null;
    		Set<TreeNode> changed = new HashSet<TreeNode>();
    		if(o!=null && o.getAvailability()!=ParameterAvailability.LOCAL)
    			changed.addAll(reactOnAttrRemove(o.getKey(), o.getOrigin(), false, true));
    		changed.addAll(propogateAttributeChange(n));
    		for(TreeNode node:changed)
    			node.saveAttributes();
    		if(attributes.containsKey(s))
    			adiff.put(s, getAttribute(s));
    		else 
    			adiff.put(s, null);

    		updateLink(s,o,n);
    	}
    	fireGUINodeAttributeChange(adiff.keySet());
   		VirtualNode.updateClonesAttributes(this, adiff, false);
    }
	public void reactBeforeAttributeChange(String s, Attribute attribute) {
	}
	public void reactAfterAttributeChange(String s, final Attribute oldAttribute, final Attribute attribute) {
    	//attributes.put(s, attribute);
		//TODO activities on attribute change
    	if(s.equals(ATTR_PREDICATE)){
       		if(isVirtual()){
       			try{
       				TreeNode par = getParent();
       				TreeInstancer ti = VirtualNode.getVirtualStorage(db);
       				if(!ti.hasCloneParent(par.getUUId())){
       		    		TreeDB db2 = VirtualNode.getBaseTree(db);
       		    		if(db2!=null)
       		    			db2.updateNodeChildren(par);
       				}else
       					db.updateNodeChildren(par);
       			}
       			catch(RuntimeException re){
       				//parent not found
       			}
       		}
    	}
    	if(s.equals(ATTR_DESCRIPTION))
    		if(attribute==null || (attribute.getValue()==null || attribute.getValue().equals("")))
        		removeAttribute(ATTR_DESCRIPTION);

    	if(s.equals(TreeDB.INDEX_ATTRIBUTE))
    		if(attribute!=null){
    			Index ind = null;
    			if(attribute!=null && attribute.getValue()!=null)
    				setIndex(Index.create(attribute.getValue().toString()));
    			else
    				setIndex(Index.create(Integer.toString(Integer.MAX_VALUE)));
    		}
    	
    	//Links core
    	updateLink(s, oldAttribute,attribute);
    	if(s.equals(ATTR_REVLINKS)){
    		revlinks = null;
    		getRevertLinks();
    	}
    	//TODO move to db
        if(getUUId().equals(db.getOrCreateTemplatesFolder().getUUId())){
	       	if(s.equals(TreeDB.ACTIVE_TEMPLATES_ATTRIBUTE))
        		db.loadActiveTemplates();
        }
	}
	private void updateLink(String s, Attribute oldAttribute, Attribute attribute){

		boolean revlink = false;
		TreeNode targ = null,newtarg = null;
		if(oldAttribute!=null && oldAttribute.getRawType().equals(AttributeType.REFERENCE)){
			revlink = true;
			targ = linkCahce.get(s);
		}
   		if(attribute!=null && attribute.getRawType().equals(AttributeType.REFERENCE)){
    		revlink = false;
    		if(targ==null || !db.getLinkResolver().verify(targ, attribute.getValue().toString())){
    			revlink = true;
    			newtarg = resolveLink(attribute.getValue().toString(), null, s);
    		}
    	}
    	if(revlink){
    		if(targ!=null){
    			targ.removeRevLink(this, s);
    			linkCahce.remove(s);
    			linkExpr.remove(s);
    		}
    	}
    	if(newtarg!=null){
    		newtarg.addRevLink(this, s);
    		if(!newtarg.uuid.equals(uuid))
    			newtarg.saveAttributes();
    	}
	}
    @Override
    public boolean equals(Object obj) {
        if (obj == null)
            return false;
        if (super.equals(obj))
            return true;
        if (!(obj instanceof TreeNode))
            return false;
        TreeNode node = (TreeNode) obj;
        if (!node.getUUId().equals(uuid))
            return false;
        if (!getQualifiedId().equals(node.getQualifiedId()))
            return false;
        if (!getAttributeKeys().equals(node.getAttributeKeys()))
            return false;
        for (String key : getAttributeKeys()) {
            if (!getAttributeValue(key).equals(node.getAttributeValue(key)))
                return false;
        }
        return true;
    }
    /**
     * Get children elements in array sorted by default sort mechanism
     * @return array of children elements
     */
    public TreeNode[] getSortedChildren() {
        return getSortedChildren(null, null);
    }
    /**
     * Get children elements of selected type in array sorted by default sort mechanism
     * @param type required type of children 
     * @return array of children elements of selected type
     */
    public TreeNode[] getSortedChildren(String type) {
        return getSortedChildren(null, type);
    }
    /**
     * Get children elements of selected type in array sorted by order defined in given comparator
     * @param c comparator that defines order of return elements
     * @param type required type of children 
     * @return array of children elements of selected type
     */
    public TreeNode[] getSortedChildren(Comparator<TreeNode> c, String type) {
        ArrayList<TreeNode> result = new ArrayList<TreeNode>();
        int num = sizeChildren();
        for (int i = 0; i < num; i++) {
            TreeNode ch = getChild(i);
            if (type == null || ch.getType().equals(type)) {
                result.add(ch);
            }
        }
        TreeNode[] r = (TreeNode[]) result.toArray(new TreeNode[0]);
        Arrays.sort(r, TreeNodeComparator.getComparator(db));
        return r;
    }

    public Object getListAttributeValue(String attrLocations,
			AttributeType tp) {
    	List<Object> ol = getListAttributeValue(attrLocations);
    	if(ol==null)
    		return null;
    	switch (tp) {
			case BOOL:
			{
				List<Boolean> b = new ArrayList<Boolean>();
				for(Object o:ol)
					b.add((Boolean)o);
				return b;
			}
			case STRING:{
				List<String> b = new ArrayList<String>();
				for(Object o:ol)
					b.add(o.toString());
				return b;
			}
			case INT:{
				List<Integer> b = new ArrayList<Integer>();
				for(Object o:ol)
					b.add((Integer)o);
				return b;
			}
			case FLOAT:{
				List<Float> b = new ArrayList<Float>();
				for(Object o:ol)
					b.add((Float)o);
				return b;
			}

		default:
			break;
		}
		return null;
    }

    /**
     * Get path to resources folder related to current element
     * @return path to resources folder
     */
    public String getResourcesPath() {
        try {
        	TreeDB db = getTreeDB();
        	UUID uid = uuid;
        	while(VirtualNode.getBaseTree(db)!=null){
        		if(db.getStorage() instanceof TreeInstancer)
        		{
        			TreeInstancer ti = (TreeInstancer)db.getStorage();
        			while(ti.getCloneParent(uid)!=null)
        				uid = ti.getCloneParent(uid);
        		}
        		db = VirtualNode.getBaseTree(db);
        	}
        	if(db.getStorage() instanceof ResourceStorage){
        		//TODO get real UUID
        		String path = new File(db.getStorage().getNodeResourcePath(uid)).toURI()
            			.toURL().toString();
            	if (path.length() > 1 && path.endsWith("/")) {
            		path = path.substring(0, path.length() - 1);
            	}
            	return path;
        	}else
        		if(db.getStorage() instanceof TransactionStorage)
        		{
        			String path = new File(db.getStorage().getNodeResourcePath(uid)).toURI()
                			.toURL().toString();
                	if (path.length() > 1 && path.endsWith("/")) {
                		path = path.substring(0, path.length() - 1);
                	}
                	return path;
        		}
        } catch (MalformedURLException e) {
            e.printStackTrace();
        }
        return db.getStorage().getNodeResourcePath(uuid);
    }
    /**
     * Get intermediate representation of element for copying process.
     * @return NodeDesc with information about current element
     */
    public NodeDesc toNodeDesc() {
        return toNodeDesc(true);
    }
    /**
     * Get intermediate representation of element for copying process.
     * @param deep make intermediate representation of children
     * @return NodeDesc with information about current element(if deep is true then with all descendants)
     */
    public NodeDesc toNodeDesc(Boolean deep) {
        return NodeDesc.fromNode(this, deep);
    }
    /**
     * Get content of resource with given name corresponding to this element
     * @param name name of resource to get
     * @return content of resource in form of stream to read
     */
    public InputStream getResourceContent(String name) {
        return db.getStorage().getResourceContents(uuid, name);
    }
    /**
     * List all available resources for current element
     * @return list of resources names
     */
    public String[] listResources() {
        return db.getStorage().listResources(uuid);
    }
    /**
     * Set content of resource with given name on current node
     * @param name name of resource
     * @param in content of resource in form of stream
     */
    public void setResourceContent(String name, InputStream in) {
        db.setResourceContent(this, name, in, null);
    }

    /**
     * Set content of resource with given name on current node
     * @param name name of resource
     * @param in content of resource in form of stream
     * @param data deprecated
     */
    public void setResourceContent(String name, InputStream in, Object data) {
        db.setResourceContent(this, name, in, data);
    }
    /**
     * Delete resource with given name from current element
     * @param name name of resource to delete
     */
    public void deleteResource(String name) {
        db.deleteResource(this, name);
    }
	
	/**
	 * Check if resource with given name exists on current element
	 * @param name name of resource to check
	 * @return true if resource exists, false otherwise 
	 */
    public boolean isResourceExists(String name) {
        return db.isResourceExists(this, name);
    }
    
    protected Index index;
    /**
	 * Get index of element. Index is specific field that is used to define custom order of elements in sorted list
     * @return index of element
     */
    public Index getIndex()
    {
        if( index == null)
        {
            String indexStr = (String)getAttributeValue( TreeDB.INDEX_ATTRIBUTE );
            Attribute a = getAttribute( TreeDB.ORDER_ATTRIBUTE );
            if(a != null)
                if(a.getValue() instanceof String){
                    if( indexStr == null ) indexStr = (String)a.getValue(); // old behaviour
                }else{
                    if( indexStr == null ) indexStr = ((Integer)a.getValue()).toString();
                }
            if(indexStr != null){
                index = Index.create( indexStr );
            }else{
                index = Index.create( ((Integer)Integer.MAX_VALUE).toString() );
            }
        }
        return index;
    }
	/**
	 * Set index of element. Index is specific field that is used to define custom order of elements in sorted list
     * @param index index of element
     */
    public void setIndex( Index index )
    {
        Attribute indexAttr = new Attribute(uuid, AttributeType.STRING, TreeDB.INDEX_ATTRIBUTE, index.toString() );
        putAttribute( indexAttr );
        this.index = index;
        //TODO update needed?
        updateTreeNodes(this, TreeDB.INDEX_ATTRIBUTE);
    }
	/**
	 * Get disposed flag for element. It is true if element is deleted.
	 * @return true if element is disposed, false otherwise 
	 */
    public boolean isDisposed() {
        return disposed;
    }
	/**
	 * Set disposed flag for element to true. It is usually do after element deletion.
	 */
    public void setDisposed() {
        disposed = true;
    }

    /**
     * If element have 'name' attribute then return it, else return  ParentUserFriendlyName/Id
     * Tends to be unique in project by agreement
     * @param inner deprecated?
     * @return path to element
     */
    public String getUserFriendlyName(boolean inner) {
    	StringBuilder sb = new StringBuilder();
    	 getUserFriendlyName(inner, true,sb);
    	return sb.toString(); 
    }
    private void getUserFriendlyName(boolean inner,boolean fc,StringBuilder sb) {
    	String name = getName();
    	if(name!=null){
    		Boolean ignore = getBooleanAttributeValue(ATTR_IGNORE_NAME);
    		if(!ignore){
    			sb.append(name);
    			return;
    		}
    	}
    	TreeNode parent = (TreeNode) getParent(inner);
        String id = getId();
   		if(parent.getType().equals("DBRoot")){
   			if(inner||!id.equals("Requirements"))
   				sb.append("/"+id);
   			return;
   		}else{
   			int k = sb.length();
   			parent.getUserFriendlyName(false,false,sb);
   			if(sb.length()!=k)
   				sb.append("/");
       	}
        //userFriendlyName = userFriendlyName.substring(c);
   		sb.append(id);
    }
    /**
     * If element have 'name' attribute then return it, else return  ParentUserFriendlyName/Id
     * Tends to be unique in project by agreement
     * @return path to element
     */
    
    public String getUserFriendlyName() {
        return getUserFriendlyName(true);
    }
    /**
     * Method to update Requirement id, name and corresponding Location nodes
     * @param req requirement to update
     * @param what name of attribute to be updated(or "_id" for when identifier is updating)
     */
    public void updateTreeNodes(TreeNode req, String what) {
    	updateTreeNodes(req, what, false);
    }
    /**
     * Update Requirement id, name and corresponding locations
     * @param req requirement to update
     * @param what name of attribute to be updated(or "_id" for when identifier is updating)
     * @param internal true if call source is internal
     */
    private void updateTreeNodes(TreeNode req, String what, boolean internal) {
        TreeNode[] reqs = req.getSortedChildren();
        //TODO update needed?
//        for (TreeNode node : reqs) {
//            updateTreeNodes(node, what, true);
//        }
        //if(!internal)
        	//db.fireNodeChange(req, true);
//        req.fireNodeAttributeChange(what);
  //      db.fireNodeGUIChange(req, true);
    //    req.fireGUINodeAttributeChange(what);
    }
    
    private List<INodeChangeListener> listenerList;
    private List<INodeChangeListener> guilistenerList;
    /**
     * Add Change Listener to be informed on changes of current node  
     * @param l change listener
     */
    public void addChangeListener(INodeChangeListener l) {
        if(listenerList == null)
            listenerList = new LinkedList<INodeChangeListener>();
        synchronized (listenerList) {
            if(!listenerList.contains(l))
                listenerList.add(l);
        }
    }
    /**
     * Remove change listener from current node 
     * @param l change listener to be removed 
     */
    public void removeChangeListener(INodeChangeListener l) {
        if(listenerList == null)
            return;
        synchronized (listenerList) {
            if(listenerList.contains(l))
                listenerList.remove(l);
        }
    }

    /**
     * Add change listener to be notified after set of actual changes  
     * @param l change listener to add
     */
    public void addGuiChangeListener(INodeChangeListener l) {
        if(guilistenerList == null)
        	guilistenerList = new LinkedList<INodeChangeListener>();
        synchronized (guilistenerList) {
            if(!guilistenerList.contains(l))
            	guilistenerList.add(l);
        }
    }
    /**
     * Remove change listener to be notified after set of actual changes
     * @param l change listener to remove
     */
    public void removeGuiChangeListener(INodeChangeListener l) {
        if(guilistenerList == null)
            return;
        synchronized (guilistenerList) {
            if(guilistenerList.contains(l))
            	guilistenerList.remove(l);
        }
    }
    /**
     * Fire event when node is deleted 
     */    
    protected void fireNodeDelete() {
        if(listenerList == null)
            return;
        INodeChangeListener[] listeners = listenerList.toArray(new INodeChangeListener[0]);
        int[]del = new int[listeners.length];
        int j = 0;
        for(int i = 0;i<listeners.length; i++){
            INodeChangeListener k = listeners[i];
            try
            {
                k.deleted(this);
            }
            catch(Exception e){
                e.printStackTrace();
                del[j++] = i;
            }
        }
        synchronized (listenerList) {
            for(int i = 0; i < j; i++)
                listenerList.remove(listeners[del[i]]);
        }
    }
    /**
     * Fire node change event when some attribute is changed
     * @param changed attribute name
     */
    protected void fireNodeAttributeChange(Set<String> what) {
        if(listenerList == null)
            return;
        INodeChangeListener[] listeners = listenerList.toArray(new INodeChangeListener[0]);
        int[]del = new int[listeners.length];
        int j = 0;
        for(String wh:what)
	        for(int i = 0;i<listeners.length; i++){
	            INodeChangeListener k = listeners[i];
	            try
	            {
	                k.attributeChange(this, wh);
	            }
	            catch(Exception e){
	                //in the case of exception in
	                //listener, delete it
	                e.printStackTrace();
	                del[j++] = i;
	            }
	        }
        synchronized (listenerList) {
            for(int i = 0; i < j; i++)
                listenerList.remove(listeners[del[i]]);
        }
    }
    /**
     * Returns change listener that represents reaction on other nodes(used in linkto mechanics) 
     */
    protected INodeChangeListener getChangeListener() {
        return null;
    }
    public Map<String, Object> getAttributesMap(boolean handlePredicate) {
    	Map<String, Object> result = new HashMap<String, Object>();
    	for(Attribute a:getAttributes().values())
    		if(!result.containsKey(a.getKey())||!a.isGenerated())
    			result.put(a.getKey(), a.getValue());
        return result;
    }
    private Object[] listToObject(List l){
		Object[] o = new Object[l.size()];
		for(int i = 0;i<l.size();i++){
			o[i]=l.get(i);
			if(o[i] instanceof List)
				o[i] = listToObject((List)o[i]);
		}
		return o;
    }
    private Scriptable createScope(Context context, Map<String, Object> attributes) {
        Scriptable topScope = ScriptRuntime.initStandardObjects(context, null, true);
        Scriptable scope = new NativeObject();
        scope.setParentScope( null );
        scope.setPrototype( topScope );
        for(String attrName: attributes.keySet() ) {
        	Object value = attributes.get(attrName);
        	Scriptable sc = scope;
        	if(value instanceof List){
        		value = listToObject((List)value);
        	}
        	else if(!value.getClass().isArray())
        		value = value.toString();
        	else if(value.getClass().isArray()){
        		sc = new NativeArray((Object[])value);
        		sc.setParentScope( scope );
                sc.setPrototype( scope );
        	}
        	Object obj = context.getWrapFactory().wrap( context, sc, value, value.getClass() );
        	scope.put( attrName, scope, obj );
        }

        return scope;
    }
    
    public String evaluatePredicateString(TreeNode node, String predicate) {
    	if(predicate == null || predicate.length()==0) return "";
    	String contextResult = "";
        try {
            Context context = Context.enter();
            WrapFactory wrapFactory = new WrapFactory();
            context.setWrapFactory(wrapFactory);
            Map<String,Object> attrMap;
            if(node == this)
            	attrMap = getAttributesMap(false);
            else
            	attrMap = node.getAttributesMap(false);
            Scriptable scope = createScope(context, attrMap);
            //invalid return
            Object res = context.evaluateString( scope, predicate, "", 1, null );
            if(res instanceof NativeJavaObject)
            	contextResult = ((NativeJavaObject)res).unwrap().toString();
            else
            	contextResult = res.toString();
        }
        catch(Exception e){
        	String s = e.toString(),s2="";
        	return s;
        } finally {
            Context.exit();
        }
        return contextResult;
    }

    public Attribute getUpdatedAttribute(Attribute a) {
    	return getUpdatedAttribute(a,Integer.parseInt(getStringAttributeValue("__i","0")));
    }
    public Attribute getUpdatedAttribute(Attribute a, int num) {
    	String ret= null,t = null, val = null,valret = null;
    	
    	if(a.getRawType().equals(AttributeType.ENUM)&&
    			a.getRawValue() instanceof AppliedEnumValue)
    	{
    		AppliedEnumValue aev = a.getGRawValue();
    		if(aev!=null){
    			AttributeType lt = aev.getType();
    			Object rt = aev.getValue();
    			//if(lt!=AttributeType.LIST){
    			a.setNewValue(rt);
    			//}
    		}
    		return a;
    		//Attribute enumdef = getTreeDB().getRootNode().getAttribute(defKey);
//    		if(enumdef==null){
//        		Object[]v2 = (Object[]) enumdef.getRawValue();
//        		if(v2!=null&&v2.length>0){
//        			AttributeType lt = (AttributeType) v2[0];
//
//    			//Enum def removed - create common attribute by default
//        		//generator will be ignored if it was exists 
//        			return new Attribute(a.getOrigin(), lt, a.getKey(), v2[(Integer) v2[1]].toString());
//        		}
//        		//all bed. nothing saved...
//        		return new Attribute(a.getOrigin(),AttributeType.STRING,a.getKey(),AttributeType.getDefaultValue(AttributeType.STRING));
//    		}
//    		Object[]v = (Object[]) enumdef.getRawValue();
//    		if(v!=null){
//    			AttributeType lt = (AttributeType) v[0];
//    			List<Object> vl = (List<Object>) v[1];
//    			if(lt!=AttributeType.LIST){
//    				Object[]rv = (Object[])a.getRawValue();
//    				Integer ind = (Integer)rv[1];
//    				if(ind<0||ind>=vl.size())
//    					ind = 0;
//    				ret = valret = "";
//    				if(vl.size()>0)
//    					ret = valret = vl.get(ind).toString();
//    				a.setValue(new Object[]{lt, rv[2], ret});
//    				a.setNewValue(ret);
//    			}else
//    				return a;
//    		}else
//    			return a;
    	}
    	if(a.getResultType().equals(AttributeType.LIST)||a.getResultType().equals(AttributeType.ENUM_DEFINITION)||a.getRawValue()==null)
    		return a;
    	if(ret == null){
        	ret = a.getRawValue().toString();
    		valret = a.getRawValue().toString();
    	}
		t = ret;
		val = valret;
		Set<String> cdeps = dependencies.get(a.getKey());
		Set<String> iter = null;
		if(cdeps == null)
			iter = new HashSet<String>();
		else
			iter = new HashSet<String>(cdeps); 
    	for(String s : iter){
    		Attribute b = getAttribute(s);
			if(b == null)
				continue;
			String replace = b.getValue().toString();
			if(b.getResultType() == AttributeType.LIST)
			{
			//	int size = ((List)b.getValue()).size();
				//if(size != 0)
					//replace = ((List)b.getValue()).get(num % size).toString();
				//TODO handle lists
			}else
				valret = AttributeGenerator.replaceRegexpVar(valret, b.getKey(),replace.replace("$", "\\$"));
			if(!t.equals(valret)){
				cdeps.add(b.getKey());
				t = valret;
			}
		}
		if(!val.equals(valret)){
			if(!a.isGenerated())
				setDependencies(a.getKey(),cdeps);
			else{
				if(!dependencies.containsKey(a.getKey()))
					setDependencies(a.getKey(),new HashSet<String>());
				addDependencies(a.getKey(),cdeps);
			}
			a.setNewValue(valret);
			return a;
		}
		return a;
	}
	private void setDependencies(String key, Set<String> cdeps) {
		Set<String>old = dependencies.get(key);
		if(old!=null)
			for(String s:old)	
				if(!cdeps.contains(s))
					removeDependency(key, s);
		for(String s:cdeps)
			if(old==null || !old.contains(s))
				addDependency(key, s);
		if(!dependencies.containsKey(key))
			dependencies.put(key, new HashSet<String>());
		dependencies.remove(ATTR_PREDICATE);
	}
	private boolean addDependencies(String key, Set<String> cdeps){
		boolean t = true;
		for(String s:cdeps)
			t = t && addDependency(key, s);
		return t;
	}
	private Set<String> collectPreDeps(String key){
		Set<String> res = new HashSet<String>();
		LinkedList<String> queue = new LinkedList<String>();
		while(!queue.isEmpty()){
			String s = queue.peekFirst();
			res.add(s);
			if(rdependencies.containsKey(s))
				for(String s2: rdependencies.get(s))
					if(!res.contains(s2))
						queue.add(s2);
		}
		return res;
	}

	public Set<String> getDependencies(String key){
		if(!dependencies.containsKey(key))
			return Collections.EMPTY_SET;
		return dependencies.get(key);
	}
	public Set<String> getRevertDependencies(String key){
		if(!rdependencies.containsKey(key))
			return Collections.EMPTY_SET;
		return rdependencies.get(key);
	}
    private boolean addDependency(String key, String key2) {
    	if(collectPreDeps(key).contains(key2))
    		return false;
    	if(key2.equals(key))
    		return false;
    	if(!dependencies.containsKey(key))
    		dependencies.put(key, new HashSet<String>());
    	dependencies.get(key).add(key2);
    	if(!rdependencies.containsKey(key2))
    		rdependencies.put(key2, new HashSet<String>());
    	rdependencies.get(key2).add(key);
    	return true;
	}
    private void removeDependency(String key, String key2){
    	if(dependencies.containsKey(key))
    		dependencies.get(key).remove(key2);
    	if(rdependencies.containsKey(key2))
    		rdependencies.get(key2).remove(key);
    }
    private void removeDependencies(String key){
    	Set<String>torem = null;
    	if(rdependencies.containsKey(key))
    		for(String s : rdependencies.get(key))
    			if(dependencies.containsKey(s))
    				dependencies.get(s).remove(key);
    	if(dependencies.containsKey(key)){
    		torem = new HashSet<String>(dependencies.get(key));
    		for(String s:torem)
    			removeDependency(key, s);
    	}
    	if(rdependencies.containsKey(key))
    		rdependencies.remove(key);
    }
    
    public boolean isNameDefined() {
        return getAttributeValue(ATTR_NAME) != null;
    }

    public String getNameOrId(){
    	return getNameOrId(true);
    }
    private String getNameOrId(boolean inner) {
    	if(!inner&&id.equals("Requirements"))
    		return "";
        if (getAttributeValue(ATTR_NAME) == null)
            return getId();
        return (String) getAttributeValue(ATTR_NAME);
    }
    public void setName(String name) {
        if (name == null || name.equals("")) {
            removeAttribute(ATTR_NAME);
        } else {
        	name = name.replace((char)160, ' ');
            putAttribute(new Attribute(this, AttributeType.STRING, ATTR_NAME, name));
        }
        //update name in cache
        getTreeDB().updateRequirementName(this);
        //fire related to name update actions
        //TODO update needed?
        updateTreeNodes(this, "_name");
    }

    public String getName() {
        return (String) getAttributeValue(ATTR_NAME);
    }

    /**
     * Fire event when node is deleted
     */
    public void fireGUINodeDelete() {
	      if(guilistenerList == null)
	            return;
	        INodeChangeListener[] listeners = guilistenerList.toArray(new INodeChangeListener[0]);
	        int[]del = new int[listeners.length];
	        int j = 0;
	        for(int i = 0;i<listeners.length; i++){
	            INodeChangeListener k = listeners[i];
	            try
	            {
	                k.deleted(this);
	            }
	            catch(Exception e){
	                e.printStackTrace();
	                del[j++] = i;
	            }
	        }
	        synchronized (guilistenerList) {
	            for(int i = 0; i < j; i++)
	            	guilistenerList.remove(listeners[del[i]]);
	        }
	}
    /**
     * File event when attribute was changed
     * @param what name of changed attribute
     */
	protected void fireGUINodeAttributeChange(Set<String> what) {
	      if(guilistenerList == null)
	            return;
	        INodeChangeListener[] listeners = guilistenerList.toArray(new INodeChangeListener[0]);
	        int[]del = new int[listeners.length];
	        int j = 0;
	        for(String wh : what)
		        for(int i = 0;i<listeners.length; i++){
		            INodeChangeListener k = listeners[i];
		            try
		            {
		                k.attributeChange(this, wh);
		            }
		            catch(Exception e){
		                //in the case of exception in
		                //listener, delete it
		                e.printStackTrace();
		                del[j++] = i;
		            }
		        }
	        synchronized (guilistenerList) {
	            for(int i = 0; i < j; i++)
	            	guilistenerList.remove(listeners[del[i]]);
	        }
	}
	/**
	 * Get iterator for children
	 */
	@Override
	public Iterator<TreeNode> iterator() {
		if(outchildren==null)
			getChildrenUUIds();
		return new TreeNodeChildIterator();
	}
	public void fireNodeAttributeChange(Set<String> attributeName,
			ParameterAvailability availability) {
		fireNodeAttributeChange(attributeName);
		if(availability != ParameterAvailability.LOCAL)
		{
			for(TreeNode child:getSortedChildren())
				child.fireNodeAttributeChange(attributeName, 
					availability == ParameterAvailability.DIRECT_CHILDREN ? ParameterAvailability.LOCAL : availability);
			
		}
	}
	public void fireGUINodeAttributeChange(Set<String> attributeName,
			ParameterAvailability availability) {
		fireGUINodeAttributeChange(attributeName);
		if(availability != ParameterAvailability.LOCAL)
		{
			for(TreeNode child:getSortedChildren())
				child.fireGUINodeAttributeChange(attributeName, 
					availability == ParameterAvailability.DIRECT_CHILDREN ? ParameterAvailability.LOCAL : availability);
			
		}
	}
	/**
	 * Get list of nodes linked to current by linkto: relation
	 * @return list of linked nodes
	 */
	public List<TreeNode> getLinkedNodes(){
		List<TreeNode> nodes = new ArrayList<TreeNode>();
		Attribute links = getAttribute(TreeNode.ATTR_LINKS);
        if(links != null){
            ArrayList<String> linksarray = (ArrayList<String>)links.getValue();
            for(String link : linksarray){
                if(link.startsWith("*"))
                    link = link.substring(1);
                String uid = link.substring(0,link.indexOf('_'));
                TreeNode node = db.getNode(UUID.fromString(uid));
                if(node == null)
                	node = db.getNode(link.substring(uid.length()+1));
                nodes.add(node);
            }
        }
		return nodes;
	}
	public List<UUID> getRawChildrenUUIDs() {
		synchronized (children) {
			return Collections.unmodifiableList(new CopyOnWriteArrayList<UUID>(children));
		}
	}
	public int sizeRawChildren() {
		return getRawChildrenUUIDs().size();
	}
	public void virtualReplaceNames() {
		if(!isVirtual())
			return;
    	List<Attribute>change = new LinkedList<Attribute>();
    	if(db!=null && db.isVirtual())
	    	for(Attribute a:getAttributes().values()){
	    		if(a!=null && a.getResultType()!=AttributeType.LIST){
	    			Attribute newa = getUpdatedAttribute(a, Integer.parseInt(getStringAttributeValue("__i","0")));
	    			if(newa.getValue()!=null && !newa.getValue().equals(a.getValue()))
	    				change.add(newa);
	    		}
	    	}
	}
	public TreeNode getRawChild(int i) {
		synchronized (children) {
			UUID uid = Collections.unmodifiableList(children).get(i);
			if(uid == null)
				return null;
			return db.getNode(uid);
		}
	}
	public void buildDependencies(Attribute a) {
		if(this.getUUId().equals(db.getStorage().getRootNode()))
			return;
		if(a.isGenerated()){
			long k = new Date().getTime();
			AttributeGenerator ag = a.getGenerator();
			if(ag==null)
			//TODO check
				attributes.remove(a.getKey());
			else
				setDependencies(a.getKey(), ag.getDependencies(this));
		}
		Set<String> set = new HashSet<String>(getAttributes().keySet());
		for(String key:set)
    	{
    		Attribute aa = getAttribute(key);
    		if(a.getRawValue()!=null && (" "+a.getRawValue().toString()+" ").indexOf(aa.getKey())>-1&& Pattern.compile(Attribute.getRegExp(aa.getKey()))
    				.matcher(" " + a.getRawValue().toString() + " ").find()){
    				if(!dependencies.containsKey(a.getKey())||!dependencies.get(a.getKey()).contains(aa.getKey()))
    					addDependency(a.getKey(),aa.getKey());
    		}
    		if(aa.getRawValue()!=null && (" "+aa.getRawValue().toString()+" ").indexOf(a.getKey())>-1&& Pattern.compile(Attribute.getRegExp(a.getKey()))
    				.matcher(" " + aa.getRawValue().toString() + " ").find()){
    				if(!dependencies.containsKey(aa.getKey())||!dependencies.get(aa.getKey()).contains(aa.getKey()))
    					addDependency(aa.getKey(),a.getKey());
    		}
    	}
	}

	public String getLinkLabel(String key) {
		TreeNode node = getLinkAttributeValue(key);
		return node==null?"Missing reference: '"+getStringAttributeValue(key)+"'":node.getUserFriendlyName();
	}
	public static String getLinkLabel(TreeNode targ, String expression, String label) {
		TreeNode node = targ.resolveLink(expression, null, label);
		return node==null?"Missing reference: '"+expression+"'":node.getUserFriendlyName();
	}

	public boolean isRestricted() {
		return false;
	}
	public boolean hasUserAttributes(){
		return false;
	}

	public boolean isRoot() {
		if(isVirtual()&&VirtualNode.getBaseTree(db)!=null && VirtualNode.getBaseTree(db).getRootNode().getUUId().equals(uuid))
			return true;
		if(db.getRootNode().equals(this))
			return true;
		return false;
	}
}

