package com.unitesk.requality.core;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;

import com.unitesk.requality.core.attribute.Attribute;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class TreeNodeComparator {
    private static Map<Integer, Map<Boolean,Comparator<TreeNode>>> comparators = 
    		new HashMap<Integer, Map<Boolean,Comparator<TreeNode>>>();
    public static Comparator<TreeNode> getComparator(TreeDB db){
    	Attribute a = db.getRootNode().getAttribute(TreeNode.SORT_BY_LOCATIONS),
  			  b = db.getRootNode().getAttribute(TreeNode.SORT_SEP_TYPES),
  			  c = db.getRootNode().getAttribute(TreeNode.SORT_BY_INDEX);
    	boolean sepSortingMix = b==null || b.getValue().equals("true") || b.getValue().equals(true);
        boolean projectSoringFlag = a==null || a.getValue().equals("true") || a.getValue().equals(true);
        boolean projectSoringFlag2 = c!=null && (c.getValue().equals("true") || c.getValue().equals(true));
		return getComparator(projectSoringFlag2?2:projectSoringFlag?1:0,sepSortingMix);
    }
    public static Comparator<TreeNode> getComparator(final int method, final boolean sepSortingMix){
    	if(!comparators.containsKey(method)||!comparators.get(method).containsKey(sepSortingMix)){
    		if(!comparators.containsKey(method))
    			comparators.put(method,new HashMap<Boolean,Comparator<TreeNode>>());
            comparators.get(method).put(sepSortingMix, new Comparator<TreeNode>() {
                @Override
                public int compare( TreeNode n1, TreeNode n2 )
                {
                	if(n1.getUUId().equals(n2.getUUId()))
                		return 0;
                    if (sepSortingMix && (n1.getType().compareTo(n2.getType()) != 0)) {
                        return n1.getType().compareTo(n2.getType());
                    }
                    if(    n1 instanceof Location 
                        && n2 instanceof Location 
                      ) 
                    {
                        int li = ((Location)n1).compareIndexTo((Location)n2);
                        if(li != 0)
                        	return li;
                    }
                    if(method==1 && n1.getType().equals(Requirement.TYPE_NAME) && n2.getType().equals(Requirement.TYPE_NAME))
                    {
                    	int ci = ((Requirement)n1).compareLocationIndexes((Requirement)n2);
                    	if(ci!=0) return ci;
                    }
                    int k = 0;
                    if( method==1 && n1.getTreeDB().getTreeLogic().usesUndexByDefault(n1.getType())
                    		||method==2)
                    	k = n1.getIndex().compareTo( n2.getIndex() );
                    if(k == 0){
                        String s1 = n1.getNameOrId(), s2=n2.getNameOrId();
                        return compareStrings(s1,s2);
                    }
                    return k;
                }
            });
    	}
        return comparators.get(method).get(sepSortingMix);
    }
    private static int compareNumbers(String in1, String in2) {
		while(in1.length()>in2.length())
			in2 = "0"+in2;
		while(in1.length()<in2.length())
			in1 = "0"+in1;
		int i = 0;
		while(in1.charAt(i)==in2.charAt(i) && i<(in1.length()-1))
			i++;
		char c1=in1.charAt(i),c2 = in2.charAt(i);
		if(c1>c2)
			return 1;
		if(c1<c2)
			return -1;
		return 0;
	}
	public static int compareStrings(String s1, String s2) {
		int k = 0;
		StringBuffer l1 = new StringBuffer(),l2 = new StringBuffer(),nu1 = new StringBuffer(),nu2 = new StringBuffer();
        int i1=0,i2=0;
        for(;i1<s1.length()&&i2<s2.length();){
        	while(i1<s1.length()&&Character.isDigit(s1.charAt(i1))){
        		nu1.append(s1.charAt(i1));
        		i1++;
        	}
        	while(i2<s2.length()&&Character.isDigit(s2.charAt(i2))){
        		nu2.append(s2.charAt(i2));
        		i2++;
        	}
        	String in1="0",in2 = "0";
        	if(nu1.length()>0)
        		in1 = nu1.toString();
        	if(nu2.length()>0)
        		in2 = nu2.toString();
        	if(nu1.length()>0)
        		if(nu2.length()>0)
        			k = compareNumbers(in1,in2);
        		else
        			k = -1;
        	else
        		if(nu2.length()>0)
            		k = 1;
        	if(k!=0)
        		return k;
        	if(i1<s1.length()&&i2<s2.length()){
        		while(i1<s1.length()&&!Character.isDigit(s1.charAt(i1))){
            		l1.append(s1.charAt(i1));
            		i1++;
            	}
            	while(i2<s2.length()&&!Character.isDigit(s2.charAt(i2))){
            		l2.append(s2.charAt(i2));
            		i2++;
            	}
            	k = l1.toString().compareToIgnoreCase(l2.toString());
            	if(k!=0)
            		return k;
        	}
        }
        return s1.substring(i1).compareToIgnoreCase(s2.substring(i2));	
     }
}
