/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.core.attribute;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;
import java.util.UUID;
import java.util.regex.Pattern;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.nodeiterators.AttributeValueIteratorType;
import com.unitesk.requality.core.nodeiterators.IAttributeGenerator;
import com.unitesk.requality.core.nodeiterators.RequalityIterator;
import com.unitesk.requality.core.nodeiterators.RequalityIteratorType;

public class AttributeGenerator extends RequalityIterator implements
		IAttributeGenerator, java.lang.Comparable {
	private UUID uid;
	public static final String 	SETT_ITERTYPE="_iteratorType",
								SETT_AVAILABILITY="_availability",
								SETT_FORMULA="_formula",
								SETT_ITERATION="_iter",
								SETT_MIN="_min",
								SETT_MAX="_max",
								RAND_VALUES="_randValues";
	private Map<String,Object> settings;
	private static Map<String, IAttributeGenerator> genMap = new HashMap<String, IAttributeGenerator>();
	public static void initGenMap(Map<String, IAttributeGenerator> p){
		genMap = p;
	}
	static {
		/*Activator a = Activator.getDefault();
		if (a != null) {
			genMap.putAll(a.loadAttributeGeneratorsDescriptions());
		}*/
		for (String s : genMap.keySet()) {
			IAttributeGenerator gen = genMap.get(s);
			AttributeValueIteratorType.registerAVIType(s, gen.getLabel());
		}
	}
	private Object getValue(String key,Object def){
		if(!settings.containsKey(key))
			return def;
		return settings.get(key);
	}
	public static boolean iteratorTypeIsHidden(AttributeValueIteratorType nitt) {
		return false;
	}

	public static String replaceRegexpVar(String valret, String key,
			String string) {
		try {
			String ret = ((" " + valret + " ").replaceAll(
					Attribute.getRegExp(key), string));
							//TODO make more properly
			ret = ret.substring(1, ret.length() - 1);
			return ret;
		} catch (java.lang.IllegalArgumentException e) {
			e.printStackTrace();
		}
		return valret;
	}


	private AttributeValueIteratorType ittype = AttributeValueIteratorType
			.fromId("CYCLE");

	// for Cycle id iterator and random
//	private int min, max, iter;

	private Attribute baseAttribute;
	private Random rand = new Random();

	// for Set type and for current set of Random Vars
	//private List<String> setValues;
	public Attribute getBaseAttribute(){
		return baseAttribute;
	}
	public AttributeGenerator(Attribute relatedAttr,UUID guid, Map<String, Object> settings2) {
		super(guid,relatedAttr==null?null:relatedAttr.getOrigin(), RequalityIteratorType.ATTRIBUTE);
		this.baseAttribute = relatedAttr;
		loadSettings(settings2);
	}
	@SuppressWarnings("unchecked")
	@Override
	public boolean equals(Object obj) {
		if (super.equals(obj))
			return true;
		if (!(obj instanceof AttributeGenerator)) {
			return false;
		}
		//AttributeGenerator it = (AttributeGenerator) obj;
		return super.equals(obj);
	}
	private List generateSet(int count)  {
		List setValues = new LinkedList();
		for (int i = 0; i < count; i++)
			setValues.add(Integer.toString((rand.nextInt(Math.abs(getMax() - getMin()) + 1) + getMin())));
		return setValues;
		//updateMaster();
	}

	public AttributeType getAttributeListType() {
		if (ittype.getId().equals(AttributeValueIteratorType.CYCLE_ID))
			return AttributeType.STRING;
		else if (ittype.getId().equals(AttributeValueIteratorType.RANDOM_ID))
			return AttributeType.STRING;
		else if (ittype.getId().equals(AttributeValueIteratorType.FORMULA_ID))
			return null;
		else if (genMap.containsKey(ittype.getId()))
			return genMap.get(ittype.getId()).getAttributeListType();
		return AttributeType.STRING;
		
	}
	public AttributeType getAttributeType() {
		if (ittype.getId().equals(AttributeValueIteratorType.CYCLE_ID))
			return AttributeType.LIST;
		else if (ittype.getId().equals(AttributeValueIteratorType.RANDOM_ID))
			return AttributeType.LIST;
		else if (ittype.getId().equals(AttributeValueIteratorType.FORMULA_ID))
			return AttributeType.STRING;
		else if (genMap.containsKey(ittype.getId()))
			return genMap.get(ittype.getId()).getAttributeType();
		return AttributeType.STRING;
		// return AttributeType.STRING;
	}

	public Set<String> getDependencies(TreeNode treeNode) {
		updateDependencies(treeNode);
		if (dependencies == null || !dependencies.containsKey(treeNode))
			return new HashSet<String>();
		return dependencies.get(treeNode);
	}

	public void setGeneratorType(AttributeValueIteratorType type){
		ittype = type;
	}
	public AttributeValueIteratorType getGeneratorType() {
		return ittype;
	}

	public int getItemsCount() {
		if (ittype == null || ittype.getId() == null)
			return 0;
		if (ittype.getId().equals(AttributeValueIteratorType.CYCLE_ID)) {
			return getIteration() != 0 ? ((getMax() - getMin()) / getIteration()) + 1 : 0;
		} else if (ittype.getId().equals(AttributeValueIteratorType.RANDOM_ID)) {
			if (baseAttribute.getRawValue() == null)
				return 0;
			else
				return getIteration();
		} else if (ittype.getId().equals(
				AttributeValueIteratorType.FORMULA_ID)) {
			return 1;
		} else if (genMap.containsKey(ittype.getId()))
			return genMap.get(ittype.getId()).getItemsCount();
		return 0;
	}

	public int getIteration() {
		return (Integer) getValue(SETT_ITERATION, 1);
	}

	@Override
	public String getLabel() {
		return ittype.getLabel();
	}

	public int getMax() {
		return (Integer) getValue(SETT_MAX, 0);
	}

	public int getMin() {
		return (Integer) getValue(SETT_MIN, 0);
	}

	public String getPredicate() {
		return (String) getValue(SETT_FORMULA, "");
	}

	public Object getSingleValue(TreeNode targ, int i2) {
		// In a case of new variable it will be inactive
		Object ret = "";
		List listRet = null;
		if (ittype.getId().equals(AttributeValueIteratorType.CYCLE_ID)) {
			ret =  i2 * getIteration() + getMin();
		} else if (ittype.getId().equals(AttributeValueIteratorType.RANDOM_ID)){
			return (getRandValues(false).getValues()).get(i2);
		} else if (ittype.getId().equals(
				AttributeValueIteratorType.FORMULA_ID)) {
			try {
				ret = targ.evaluatePredicateString(targ, getPredicate());
			} catch (Exception e) {
				String s = e.getMessage();
				String tor = "";
				for (int k = 0; k < s.length(); k++)
					if (Character.isJavaLetterOrDigit(s.charAt(k))
							|| Character.isWhitespace(k))
						tor += (s.charAt(k));
				ret = tor;
			}
		} else if (genMap.containsKey(ittype.getId()))
			ret = genMap.get(ittype.getId()).getSingleValue(targ,i2);
		if (listRet != null)
			return listRet;
		if (ret == null)
			return AttributeType.getDefaultValue(getAttributeType());
		return ret;
	}

	public ListValue getRandValues(boolean renew) {
		ListValue l = (ListValue) settings.get(RAND_VALUES);
		if(l == null || renew || (l.getValues()).size()!=getItemsCount()){
			l = new ListValue(getAttributeListType(),generateSet(getItemsCount()));
			setRandValues(l);
		}
		return l;
	}
	public void setRandValues(ListValue v) {
		settings.put(RAND_VALUES, v);
	}
	public Object getValue(TreeNode targ) {
		if(getAttributeType().equals(AttributeType.LIST)){
			List list = new ArrayList();
			int numchilds = getItemsCount();
			for (int i = 0; i < numchilds; i++) {
				String ret = getSingleValue(targ, i).toString();
				if (ret != null)
					list.add(ret);
			}
			return new ListValue(getAttributeListType(), list);
		}
		return getSingleValue(targ, -1);
	}

	/*public void loadSettingsFromStrings(String[] s) {
		if (ittype == null || ittype.getId() == null)
			return;
		// cycle
		if (ittype.getId().equals(AttributeValueIteratorType.CYCLE_ID)) {
			setAttribute(new Attribute(getMaster(),AttributeType.INT, "0"));
			min = Integer.parseInt(safeGet(s, 0, "0"));
			max = Integer.parseInt(safeGet(s, 1, "0"));
			iter = Integer.parseInt(safeGet(s, 2, "0"));
		}
		// random
		if (ittype.getId().equals(AttributeValueIteratorType.RANDOM_ID)) {
			min = Integer.parseInt(safeGet(s, 0, "0"));
			max = Integer.parseInt(safeGet(s, 1, "0"));
			setValues = new LinkedList<String>();
			int count = Integer.parseInt(safeGet(s, 2, "0")) + 3;
			for (int i = 3; i < count; i++) {
				String val = safeGet(s, i, "\"\"");
				val = val.substring(1, val.length() - 1);
				setValues.add(val);
			}
		}
		// predicate
		if (ittype.getId().equals(AttributeValueIteratorType.FORMULA_ID)) {
			predicate = safeGet(s, 0, "\"\"");
			predicate = predicate.substring(1, predicate.length() - 1);
		}
		// valuefromset
		if (ittype.getId().equals(AttributeValueIteratorType.VALUE_FROM_SET_ID)) {
			iter = Integer.parseInt(safeGet(s, 0, "0"));
		}
	}*/

	private String safeGet(String[] del, int s) {
		return safeGet(del, s, "");
	}

	// prefer to use it. for some cases del will be empty
	private String safeGet(String[] del, int s, String def) {
		if (s >= del.length || del[s].equals(""))
			return def;
		return del[s];
	}

	/*public void setAttributeName(String val)  {
		if (val.trim().equals(""))
			return;
		/*Attribute a = getMaster().getAttribute(attributeName);
		if (a != null && a.isGenerated()
				&& a.getOrigin().equals(getMaster().getUUId()))
			getMaster().removeAttribute(attributeName);*/

		/*getMaster().removeIterator(this);
		setAttribute(new Attribute(getMaster(), AttributeType.STRING, "_name", value));
		getMaster().addExistingAttributeIterator(this);
		updateMaster();
	}*/

	public void setIdType(AttributeValueIteratorType tp)
			 {
		AttributeValueIteratorType old = ittype;
		ittype = tp;
		if(!ittype.equals(tp)){
			//getMaster().removeIterator(this);
			//getMaster().addExistingAttributeIterator(this);
			//updateMaster();
		}
	}

	public void setIteration(int it)  {
		//int old = iter;
		//th/is.iter = it;
		//if(old!=it)
		setValue(SETT_ITERATION,it);
		//updateMaster();
	}

	public void setMax(int max)  {
		/*int old = this.max,oldm=min;
		if (max > this.min) {
			this.max = max;
		} else {
			this.max = min;
			min = max;
		}
		if(old!=max || oldm!=min)*/
		setValue(SETT_MAX,max);
		//updateMaster();
	}

	private void setValue(String key, Object value) {
		settings.put(key,value);
	}
	public void setMin(int min)  {
		/*
		int old = this.min,oldmx=this.max;
		if (max > min) {
			this.min = min;
		} else {
			this.min = max;
			max = min;
		}
		if(oldmx!=max || old!=min)*/
		setValue(SETT_MIN,min);
	//	updateMaster();
	}

	public void setFormula(String value, boolean butch)
	{
	//	if (!predicate.equals(value)) {
		//	predicate = value;
//			if (!butch)
		setValue(SETT_FORMULA,value);
		//if (!butch)
			//updateMaster();
	//	}
	}
/*
	public void setSet(List<String> set,boolean propogate)  {
		setValues = set;
		if(propogate)
			updateMaster();
	}
*/
	/*public String settingsToString() {
		if (ittype.getId().equals(AttributeValueIteratorType.CYCLE_ID)) {
			return min + "|" + max + "|" + iter;
		} else if (ittype.getId().equals(AttributeValueIteratorType.RANDOM_ID)) {
			String s = min + "|" + max + "|";
			if (setValues != null) {
				s += setValues.size();
				Iterator<String> iter = setValues.iterator();
				while (iter.hasNext())
					s += "|\"" + iter.next() + "\"";
			} else
				s += "0";
			return s;
		} else if (ittype.getId().equals(
				AttributeValueIteratorType.FORMULA_ID)) {
			return "\"" + predicate + "\"";
		} else if (ittype.getId().equals(
				AttributeValueIteratorType.VALUE_FROM_SET_ID)) {
			return Integer.toString(iter);
		} else if (genMap.containsKey(ittype.getId()))
			return genMap.get(ittype.getId()).settingsToString();
		return "";
	}*/

	/*public String toAttributeString() {
		String s = super.toAttributeString() + "|" + availability.ordinal()
				+ "|";
		s += attributeName + "|";
		s += ittype.getId();
		s += "|" + settingsToString();
		return s;
	}*/


	protected Set<String> updateDependencies(final TreeNode node) {
		if(!getGeneratorType().getId().equals(AttributeValueIteratorType.FORMULA_ID))
			return null;
		Map<String, Attribute> attrs = node.getAttributes();
		if(dependencies.containsKey(node))
			dependencies.remove(node);
		dependencies.put(node, new HashSet<String>());
		Set<String> names = new HashSet<String>(attrs.keySet());
		for (String rag : attrs.keySet())
			names.add(rag);
		String predicate = getPredicate();
		if (predicate != null)
			for (String s : names)
				if (predicate.indexOf(s)>-1 && Pattern.compile(Attribute.getRegExp(s))
						.matcher(" " + predicate + " ").find())
					dependencies.get(node).add(s);
		Set<String> todo = new HashSet<String>();
		for (final String s : dependencies.get(node)) {
			if (!attrs.containsKey(s)) {
				todo.add(s);
				continue;
			}
		}
		return todo;
	}
	
	@Override
	public Map<String, Object> getSettings() {
		return settings;
	}

	@Override
	public void loadSettings(Map<String, Object> attributes) {
		this.settings = attributes;
		if(settings.containsKey(SETT_ITERTYPE)&&settings.get(SETT_ITERTYPE)instanceof String)
			settings.put(SETT_ITERTYPE, AttributeValueIteratorType.fromId(settings.get(SETT_ITERTYPE).toString()));
		ittype = (AttributeValueIteratorType) getValue(SETT_ITERTYPE, AttributeValueIteratorType.fromId(AttributeValueIteratorType.FORMULA_ID));
		
	}
	public String getAttributeName(){
		return baseAttribute.getKey();
	}
	public void updateValue(TreeNode targ) {
		baseAttribute.setValue(getValue(targ));
	}
	public void setAttribute(Attribute attribute) {
		baseAttribute = attribute;
		//TODO value generation
	}
	public AttributeGenerator copy(){
		return copy(true);
	}
	public AttributeGenerator copy(boolean newId) {
		AttributeGenerator ag = new AttributeGenerator(baseAttribute, newId?UUID.randomUUID():genuid,new HashMap<String, Object>(settings));
		ag.ittype = ittype;
		return ag;
	}
	public UUID getUUID() {
		return genuid;
	}
	@Override
	public int compareTo(Object arg0) {
		if(!(arg0 instanceof AttributeGenerator))
			return -1;
		// 	TODO Auto-generated method stub
		return getUUID().compareTo(((AttributeGenerator)arg0).getUUID());
	}
}
