/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.core.attribute;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.UUID;

public enum AttributeType {
    INT,
    FLOAT,
    BOOL,
    STRING,
    LIST,
    RESOURCE,
    REFERENCE,
    GENERATED,
    ENUM,
    ENUM_DEFINITION;
    private static AttributeType[] visvalues;
    static{
    	visvalues = new AttributeType[values().length-3];
    	int i = 0;
    	for(AttributeType at:values())
    		if(at!=RESOURCE&&at!=GENERATED&&at!=ENUM)
    			visvalues[i++]=at;
    }
    public static AttributeType[]visibleValues(){
    	return visvalues;
    }
    public Object fromString(String string) {
        if(this == INT) {
            return Integer.valueOf(string);
        }
        if(this == FLOAT) {
            return Float.valueOf(string);
        }
        if(this == BOOL) {
            return Boolean.valueOf(string);
        }
        // TODO: LIST and Resource
        return string;
    }

	public static Object getDefaultValue(AttributeType attributeType) {
		if(attributeType==INT)
			return "0";
		if(attributeType==FLOAT)
			return "0.0";
		if(attributeType==BOOL)
			return "false";
		if(attributeType==STRING)
			return "";
		if(attributeType == REFERENCE)
			return ".";
		if(attributeType == ENUM)
			return new AppliedEnumValue("", null, AttributeType.STRING, "");
		if(attributeType == AttributeType.LIST)
			return new ListValue(AttributeType.STRING, Arrays.asList(new Object[]{""}));
		if(attributeType == ENUM_DEFINITION)
		{
			return new EnumDefinitionValue(AttributeType.STRING, new ArrayList<EnumValue>());
		}
		if(attributeType == AttributeType.GENERATED)
			return new AttributeGenerator(null, UUID.randomUUID(), new HashMap<String, Object>());
	    return null;
	}
}