package com.unitesk.requality.core.checkers;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.attribute.Attribute;

public class NodeAttribChecker implements IChecker {

    public NodeAttribChecker() {
    }

    @Override
    public CheckerProblem[] check(TreeNode node) {
        ArrayList<CheckerProblem> result = new ArrayList<CheckerProblem>();
        Set<String> attrs = new HashSet<String> (node.getAttributeKeys());
        for (String key : attrs) {
            Attribute attr = node.getAttribute(key);
            if(attr == null){
            	result.add( new CheckerProblem( node, "Attribute \"" + key + "\" is null" ) );
            	continue;
            }
            if( attr.getValue() == null )
            {
                result.add( new CheckerProblem( node, "Attribute \"" + key + "\" has null value" ) );
                continue;
            }
            String value = attr.getValue().toString();
            switch (attr.getResultType()) {
            case INT:
                try {
                    Integer.parseInt(value);
                } catch (NumberFormatException e) {
                    result.add(new CheckerProblem(node, "Attribute \"" + key + "\" of type INT has wrong value: \"" + value + "\""));
                }
                break;
            case FLOAT:
                try {
                    Float.parseFloat(value);
                } catch (NumberFormatException e) {
                    result.add(new CheckerProblem(node, "Attribute \"" + key + "\" of type FLOAT has wrong value: \"" + value + "\""));
                }
                break;
            case BOOL:
                if (!"true".equalsIgnoreCase(value) && !"false".equalsIgnoreCase(value)) {
                    result.add(new CheckerProblem(node, "Attribute \"" + key + "\" of type BOOL has wrong value: \"" + value + "\""));
                }
                break;
            case STRING:
                if(!Attribute.checkNameIsValid(key, true))
                    result.add(new CheckerProblem(node, "Attribute \"" + key + "\" of type STRING has wrong value: \"" + value + "\""));
            default:
                break;
            }
        }
        return result.toArray(new CheckerProblem[0]);
    }

}
