package com.unitesk.requality.core.nodeiterators;

import java.util.Iterator;
import java.util.UUID;

import com.unitesk.requality.core.INodeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.VirtualNode;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public abstract class RequalityChildIterator extends RequalityIterator {
	protected RequalityChildIteratorType chitype;
	public enum RequalityChildIteratorType {REUSE_CARTESIAN, BASEREQ};
	public String toAttributeString(){
		return super.toAttributeString()+"|"+Integer.toString(chitype.ordinal());
	}
	public static RequalityChildIterator fromAttributeString(UUID uid, TreeNode it, String s){
    	RequalityIteratorType nit = RequalityIteratorType.ATTRIBUTE;
     	if(s != null && !s.equals("")){
     		String p = "";
     		if(s.indexOf("|")>-1){
     			p = s.substring(0, s.indexOf("|"));
     			s = s.substring(s.indexOf("|")+1);
     			if(!s.isEmpty()&&s.startsWith("UUID:")){
   					String ts = s.substring(5,s.length());
   					int t = ts.indexOf("|");
   					int end = t!=-1?t:s.length();
   					if(end>0)
   						uid = UUID.fromString(ts.substring(0,end-1));
   					if(t>-1)
   						s = ts.substring(end+1);
   				}
     		}else 
     			p = s;
     		if(p.equals(""))
     			p = "0";
     		nit = RequalityIteratorType.values()[Integer.parseInt(p)];
     	}
    	try {
    		return getOne(uid, it,nit,s);
		} catch (Exception e) {
		}
    	return null;
	}
	public RequalityChildIterator(UUID guid, UUID mast, RequalityIteratorType tp,
			String s){
		super(guid, mast, tp);
		String tp2 = "0";
		if(s.indexOf("|")>0){
			tp2 = s.substring(0,s.indexOf("|"));
			chitype = RequalityChildIteratorType.values()[Integer.parseInt(tp2)];
		}else
		{
			tp2 = s;
			if(tp2.equals(""))
				tp2 = "0";
		}
	}
    
    public VirtualNode getInstancerMaster(TreeDB db){
    	TreeNode mast = getMaster(db);
    	if(mast instanceof VirtualNode)
    		return (VirtualNode)mast;
    	return null;
    }
	public RequalityChildIteratorType getChildIteratorType(){
		if(chitype == null)
			return RequalityChildIteratorType.REUSE_CARTESIAN;
		return chitype;
	}
	public int getItemsCount(TreeDB db){
		if(getInstancerMaster(db).getTarget()==null)
			return 0;
		return getCount(db);
	}
	public abstract int getCount(TreeDB db);
	public abstract void updateChildren(TreeDB db);
	public static RequalityChildIterator getOne(UUID guid, TreeNode it, RequalityIteratorType nit, String s){
		String cl = null;
		if(s.indexOf("|")>-1)
			cl = s.substring(0, s.indexOf("|"));
		else 
			cl = s;
		if(cl == null || cl.equals(""))
			cl = "0";
		Integer i = Integer.parseInt(cl);
		if(s.indexOf("|")>-1)
			s = s.substring(s.indexOf("|")+1);
		else
			s = "";
		switch(RequalityChildIteratorType.values()[i]){
			 case BASEREQ:{
				 return new BaseReqChildIterator(guid, it.getUUId(), s);
			 }
			 case REUSE_CARTESIAN:{
				 return new ReuseRequalityChildIterator(guid, it.getUUId(),s);
			 }
		}
		return null;
	}
	protected abstract String checkPossibleTarget(TreeNode newtarget);
	public String checkTarget(TreeNode newtarget){
		if(newtarget == null)
			return "Target is not set";
		TreeNode mnode = newtarget.getTreeDB().getNode(getMasterUUId());
		if((mnode.getQualifiedId()+"/").startsWith(newtarget.getQualifiedId()+"/"))
			return "Target on '"+newtarget.getUserFriendlyName()+"' causes recursion";
		if( (newtarget.getQualifiedId()+"/").startsWith(mnode.getQualifiedId()+"/"))
			return "Cannot select elements being already reused by altering virtual node";
		String s = checkPossibleTarget(newtarget);
		if(s!=null)
			return s;
		DeepFirstTreeWalker dftw = new DeepFirstTreeWalker(newtarget);
		Iterator<TreeNode> it = dftw.iterator();
		while(it.hasNext()){
			TreeNode nod = it.next();
			if(nod.getType().equals(VirtualNode.TYPE_NAME)){
				TreeNode targ = ((VirtualNode)nod).getTarget();
				if(targ!=null && mnode.equals(targ))
					return "Target on '"+newtarget.getUserFriendlyName()+"' causes recursion";
			}
		}
		return null;
	}
	public INodeChangeListener getMasterListener() {
		return null;
	}
	public void updateDependencies(TreeDB db){
		
	}
}
