package com.unitesk.requality.eclipse.core;

import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.resources.WorkspaceJob;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class EclipseChecker extends IncrementalProjectBuilder {
    public final static String BUILDER_ID = "com.unitesk.requality.checkers";

    private IResourceDeltaVisitor visitor = new IResourceDeltaVisitor() {
        
        @Override
        public boolean visit(IResourceDelta delta) throws CoreException {
            if(delta.getKind() == IResourceDelta.REMOVED) {
                return false;
            }
            if(delta.getResource().getType() != IResource.FILE) {
                return true;
            }
            TreeNode node = TreesTracker.getNodeByResource((IFile) delta.getResource());
            if(node!=null) {
                RequalityPlugin.getCheckers().check(node,false);
            }
            return false;
        }
    };

    public EclipseChecker() {
    }

    @Override
    protected IProject[] build(int kind, Map<String, String> args,
            IProgressMonitor monitor) throws CoreException {
        if (kind == FULL_BUILD) {
        	IProject proj = getProject();
        	if(proj == null)
        		return null;
        	TreeDB db = TreesTracker.getTracker().getTree(proj);
        	ResourceStorage rs = null;
        	if(db.getStorage() instanceof ResourceStorage){
        		rs = (ResourceStorage)db.getStorage();
        		while(!rs.lockProject())
					try {
						Thread.currentThread().join(400);
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
        	}
        	
        	if(db == null)
        		return null;
            final TreeNode root = db.getRootNode();
            if(root != null) {
                final DeepFirstTreeWalker tree = new DeepFirstTreeWalker(root);
                WorkspaceJob checkNode = new WorkspaceJob("Check Project " + proj.getName()) {
        			@Override
                	public IStatus runInWorkspace(IProgressMonitor monitor) {
        				Set<TreeNode> nodes = new HashSet<TreeNode>();
        				for (TreeNode node : tree){
        					if(monitor.isCanceled())
        						return Status.CANCEL_STATUS;
                    		nodes.add(node);
        				}
        				for (TreeNode node : nodes){
        					if(monitor.isCanceled())
        						return Status.CANCEL_STATUS;
        					if(!node.isDisposed())
        						RequalityPlugin.getCheckers().check(node,true);
        				}
						return Status.OK_STATUS;
        			}
        		};
        		//checkNode.setRule(getProject());
        		checkNode.setPriority(Job.LONG);
        		checkNode.schedule();
            }
            if(rs!=null)
            	rs.unLockProject();
        } else {
            if (getDelta(getProject()) != null) {
                getDelta(getProject()).accept(visitor, false);
            }
        }
        return null;
    }
}
