package com.unitesk.requality.eclipse.handlers;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.TreeNodeChangeEvent;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.Requirement;

public class SelectReferencedNode extends AbstractHandler implements ITreeChangeListener{
	private TreeDB oldDb=null;
	private Map<TreeDB,TreeNode>refMap = new HashMap<TreeDB,TreeNode>(),
			selMap = new HashMap<TreeDB, TreeNode>();
	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		final TreeNode node = RequalityCNF.getSelectedNodes().get(0),
				old = node.getTreeDB().getRefTarget();
		selMap.put(node.getTreeDB(), node);
		node.getTreeDB().setRefTarget(node);
		
		updateButtonLabel();
		
		Display.getCurrent().syncExec(new Runnable() {
			
			@Override
			public void run() {
				RequalityCNF view = (RequalityCNF) PlatformUI.getWorkbench()
		                .getActiveWorkbenchWindow().getActivePage()
		                .findView(RequalityCNF.ID);
		        	if (view != null) {
		        		view.refresh(old);
		        		view.refresh(node);
						view.getNavigatorActionService().updateActionBars();
		        	}
			}
		});
		return null;
	}

	@Override
	public boolean isEnabled() {
		List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
		if(nodes.size()!=1)
			return false;
		TreeNode node = nodes.get(0);
		if(!node.getType().equals(Requirement.TYPE_NAME))
			return false;
		if(oldDb == null || node!=null && (!oldDb.equals(node.getTreeDB())||
				oldDb.getRefTarget()!=null&&!oldDb.getRefTarget().equals(refMap.get(oldDb)))){
			if(oldDb!=null && selMap.containsKey(oldDb)){
				Display.getCurrent().syncExec(new Runnable() {
					
					@Override
					public void run() {
						RequalityCNF view = (RequalityCNF) PlatformUI.getWorkbench()
				                .getActiveWorkbenchWindow().getActivePage()
				                .findView(RequalityCNF.ID);
				        	if (view != null) {
				        		view.refresh(selMap.get(oldDb));
								view.getNavigatorActionService().updateActionBars();
				        	}
					}
				});
			}
			oldDb=node.getTreeDB();
			boolean listen = refMap.containsKey(oldDb);
			refMap.put(oldDb, oldDb.getRefTarget());
			updateButtonLabel();
			if(listen)
				node.getTreeDB().addGUIListener(this);
		}
		return true;
	}

	@Override
	public void changed(TreeNode node,Set<String> params, boolean internal) {
		//TODO filter by changes
		if(node.equals(selMap.get(node.getTreeDB())))
			updateButtonLabel();
	}

	private void updateButtonLabel() {
		Display d = Display.getCurrent();
		if(d!=null)
			d.syncExec(new Runnable() {
			@Override
			public void run() {
				RequalityCNF view = (RequalityCNF) PlatformUI.getWorkbench()
		                .getActiveWorkbenchWindow().getActivePage()
		                .findView(RequalityCNF.ID);
		        	if (view != null) {
		        		ToolBarManager manager=(ToolBarManager) view.getViewSite().getActionBars().getToolBarManager();
		        		ToolItem ti = null;//((ToolItem)ci.getWidget());
		        		for(ToolItem iti:manager.getControl().getItems()){
		        			Object o = iti.getData();
		        			if(o!=null && o instanceof ContributionItem){
		        				if("com.unitesk.requality.commands.RequilityCNFRefSel".equals(((ContributionItem)o).getId()))
		        					ti = iti;
		        			}
		        		}
		        		if(ti!=null){
		        			int ref = 0;
		        			if(oldDb!=null)
		        				ref = oldDb.getGlobalRefTargetState();
		        			switch(ref){
		        				case 0:{
		        					ti.setImage((Image)Activator.getImageDescriptor("/icons/zerolink.png").createResource(Display.getCurrent()));
		        				}break;
		        				case 1:{
		        					ti.setImage((Image)Activator.getImageDescriptor("/icons/onelink.png").createResource(Display.getCurrent()));
		        				}break;
		        				case 2:{
		        					ti.setImage((Image)Activator.getImageDescriptor("/icons/multilink.png").createResource(Display.getCurrent()));
		        				}break;
		        			}
		        		}
		        		//((HandledContributionItem)ci).widget.update();
						//view.getNavigatorActionService().updateActionBars();
		        	}
			}
		});
		
	}

	@Override
	public void created(TreeNode node, boolean internal) {
	}

	@Override
	public void deleted(NodeDesc nodeDesc, boolean internal) {
	}

	@Override
	public void moved(TreeNode node, TreeNode oldParent, boolean internal) {
	}

	@Override
	public void changed(List<TreeNodeChangeEvent> changes) {
		//TODO remade
		for(TreeNodeChangeEvent ce:changes)
		{
			switch (ce.getType()) {
				case DELETE:
					deleted(ce.getNodeDescTarget(), (Boolean)ce.getArguments()[0]);
				break;
				case CREATE:
					created(ce.getTreeNodeTarget(),(Boolean)ce.getArguments()[0]);
				break;
				case MOVE:
					moved(ce.getTreeNodeTarget(),(TreeNode)ce.getArguments()[1],(Boolean)ce.getArguments()[0]);
				break;
				case CHANGE:
					changed(ce.getTreeNodeTarget(),(Set<String>)ce.getArguments()[1],(Boolean)ce.getArguments()[0]);
				break;
			default:
				break;
			}
		}
	}
}
