package com.unitesk.requality.eclipse.handlers.location;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;

public class CommonRemoveLocation {

    private static void doRemoveLocation(Location loc, Requirement req) {
        req.removeLocation(loc.getQualifiedId());
        req.saveAttributes();
    }

    //parameterized removing locations:
    //if req != null, then from requirement req, else from everywhere;
    //if reqMarker != null, then from Markup Editor reqMarker, else search any Markup Editor
    //if batch, then asks, else TRANSACTION MUST BE STARTED!
    public static void RemoveLocation(Location loc, Requirement req, ReqMarker reqMarker, boolean batch) {
        if (loc == null) {
            return;
        }
        if (!batch) {
            if (!MessageDialog.openConfirm(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(), "Delete Location",
                    "Delete selected Location?")) {
                return;
            }
            if(!loc.getTreeDB().startTransaction("Delete Location",true)){
            	MessageDialog.openError(PlatformUI.getWorkbench()
                    .getActiveWorkbenchWindow().getShell(),"Delete Location Error",
        				"Some actions are already performing");
            	return;
            }
        }
        if (req != null)
            doRemoveLocation(loc, req); else
        for (Requirement _req : loc.getRequirements())
            doRemoveLocation(loc, _req);
        String jsData = null;
        if (req == null || loc.getRequirements().length == 0) {
            Document doc = (Document) loc.getParent();
            if (doc != null)
            {
                if (reqMarker == null) {
                    IEditorReference eRef = null;
                    try {
                        eRef = RequalityPlugin.findMarkEditorForDocument(
                                doc.getQualifiedId());
                    } catch (PartInitException e) {
                        e.printStackTrace();
                    }
                    if (eRef != null && eRef.getEditor(true) != null
                            && (eRef.getEditor(true) instanceof ReqMarker)) {
                        reqMarker = (ReqMarker) eRef.getEditor(true);
                    }
                }
                if (reqMarker != null) {
                    try {
                        jsData = reqMarker.getBrowser().deleteLocation(loc.getId());
                        reqMarker.saveDocument(jsData);
                    } catch (CoreException e) {
                        e.printStackTrace();
                    }
                } else {
                	try {
            			doc.getDOM();
            		} catch (Exception e) {
            			e.printStackTrace();
            		}
                    doc.removeLocation(loc.getId());
                    doc.saveDOM();
                }
                loc.delete();
            }
        }
        if (!batch) {
            loc.getTreeDB().commit();
        }
    }

}
