/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.ui.cnf;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.UUID;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.navigator.CommonDropAdapter;
import org.eclipse.ui.navigator.CommonDropAdapterAssistant;

import com.unitesk.requality.core.ITreeLogic.IdContainsWrongChars;
import com.unitesk.requality.core.ITreeLogic.IdEmpty;
import com.unitesk.requality.core.ITreeLogic.PastingIntoLeafException;
import com.unitesk.requality.core.ITreeLogic.SameIdExistsInChildren;
import com.unitesk.requality.core.ITreeLogic.WrongProjectException;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeDB.SortingMethod;
import com.unitesk.requality.core.TreeLogic;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.transaction.TransactionStorage;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.nodetypes.Comment;
import com.unitesk.requality.nodetypes.TestPurpose;
import com.unitesk.requality.nodetypes.VirtualNode;
import com.unitesk.requality.tools.DeepFirstTreeWalker;
import com.unitesk.requality.tools.Index;
import com.unitesk.requality.tools.RequalityCLI;

public class CNFDropAssistant extends CommonDropAdapterAssistant{
    private List<TreeNode>srcNodes;

    public CNFDropAssistant() {
        // TODO Auto-generated constructor stub
    }

    @Override
    public IStatus validateDrop(Object target, int operation,
            TransferData transferType) {
        if (!isSupportedType(transferType)) {
            return Status.CANCEL_STATUS;
        }
        if (target instanceof TreeNode) {
            LocalSelectionTransfer transfer = LocalSelectionTransfer
                    .getTransfer();
            ITreeSelection selection = (ITreeSelection) transfer.getSelection();
            if (selection == null) {
                return Status.CANCEL_STATUS;
            }
            srcNodes = new ArrayList<TreeNode>();
            Object[] select = selection.toArray();
            for (Object sel : select) {
                if (sel instanceof TreeNode) {
                    srcNodes.add((TreeNode) sel);
                } else {
                    return Status.CANCEL_STATUS;
                }
            }
            if(select.length == 1 && select[0] == target)
            	return Status.CANCEL_STATUS;
            return Status.OK_STATUS;
        }
        return Status.CANCEL_STATUS;
    }
    @Override
    public IStatus handleDrop(CommonDropAdapter aDropAdapter,
            DropTargetEvent aDropTargetEvent, Object aTarget) {
        if (srcNodes == null) {
            return Status.CANCEL_STATUS;
        }
        if (!(aTarget instanceof TreeNode)) {
            return Status.CANCEL_STATUS;
        }
        List<TreeNode>workList = new ArrayList<TreeNode>();
        SortedMap<UUID, String> quids 
            = new TreeMap<UUID, String>();
        TreeDB db = srcNodes.get(0).getTreeDB();
        for(TreeNode node: srcNodes){
            quids.put(node.getUUId(), node.getQualifiedId());
            //it is not possible to move nodes over different treedbs
            if(node.getTreeDB()!=db &&(node.getTreeDB()!=VirtualNode.getBaseTree(db)))
                return Status.CANCEL_STATUS; 
        }
        TreeNode basetarget = (TreeNode) aTarget,target = basetarget;
        TreeNode lastNode = null;
        SortedMap<String, Integer> cnt = new TreeMap<String, Integer>();
        for(int ii = 0; ii < srcNodes.size(); ii++){
        	TreeNode node = srcNodes.get(ii);
            boolean add = true;
            for(int j = 0; j < ii; j++){
            	TreeNode n2 = srcNodes.get(j);
                if(n2 != node && (quids.get(node.getUUId())+"/").contains(
                        (quids.get(n2.getUUId()))+"/")){
                    add = false;
                    break;
                }
            }
            if(add){
                workList.add(node);
            	if(!cnt.containsKey(node.getType()))
            		cnt.put(node.getType(), 0);
            	cnt.put(node.getType(), cnt.get(node.getType())+1);
            }
            TreeLogic l = node.getTreeDB().getTreeLogic();
            if((!l.canBeChildOf(node.getType(), basetarget.getType()) &&
                    l.canBeChildOf(node.getType(), basetarget.getParent(true)))
                    || RequalityCNF.shiftPressed){
                lastNode = basetarget;
                target = basetarget.getParent(true);
            }
        }   
        if(lastNode == null && target.sizeChildren() > 0)
            lastNode = target.getSortedChildren()[target.sizeChildren()-1];
        if(workList.size() == 0)
            return Status.CANCEL_STATUS;
        TreeLogic l = workList.get(0).getTreeDB().getTreeLogic();
        String generalssting = workList.size()>1?"s":"";
        for(TreeNode srcNode:workList){
        	boolean adds = cnt.get(srcNode.getType())>1;
            try {
                if(!l.canBeCopiedToExc(srcNode, target, true)){
                    MessageDialog.openError(PlatformUI.getWorkbench()
                            .getActiveWorkbenchWindow().getShell(), "Move Element"+generalssting,
                            (adds?srcNode.getMultiTypeLabel():srcNode.getTypeLabel()) +
                            " cannot be moved to '"+target.getUserFriendlyName()+"'");
                    return Status.CANCEL_STATUS;
                }
            } catch (PastingIntoLeafException e) {
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Move Element"+generalssting,
                        "Can't move " + (adds?srcNode.getMultiTypeLabel():srcNode.getTypeLabel()) 
                        +" to "+target.getTypeLabel()+" with " + (srcNode.getType().equals(TestPurpose.TYPE_NAME)?"non-":"")
                        + "Test Purpose child");
                return Status.CANCEL_STATUS;
            } catch (WrongProjectException e) {
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Move Element"+generalssting,
                        "Moving elements between projects is prohibited");
                return Status.CANCEL_STATUS;
            } catch (SameIdExistsInChildren e) {
                //not possible here
            } catch (IdEmpty e) {
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Move Element"+generalssting,
                        "Element id is empty");
                return Status.CANCEL_STATUS;
            } catch (IdContainsWrongChars e) {
                MessageDialog.openError(PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getShell(), "Move Element"+generalssting,
                        "Element id contains wrong characters :" + e.getWrongCharacter());
                return Status.CANCEL_STATUS;
            }
            for (TreeNode node : new DeepFirstTreeWalker(srcNode)) {
                if (node.getQualifiedId().equals(target.getQualifiedId())) {
                    return Status.CANCEL_STATUS;
                }
            }
        }
        String fromNames = "";
        List<Integer>renIndex = new ArrayList<Integer>();
        int ccc = 0;
        for(int i = 0;i<workList.size();i++)
            if(!l.isChildIdValid(target, workList.get(i), workList.get(i).getId())){
                renIndex.add(i);
                if(!workList.get(i).getTreeDB().getTreeLogic().isAutoRenamable(workList.get(i).getType())){
                    fromNames += workList.get(i).getId() + ", ";
                    ccc++;
                }
            }
        String purposes = "",adds = ccc>1?"s":"";
        byte purpcnt = 0;
        if(!fromNames.equals("")){
        	purposes+="\n- Element"+adds+" with id"+adds+" " + fromNames.substring(0, fromNames.length()-2) 
                    + " already exist"+(adds.equals("")?"s":"")+" in "
                    + target.getUserFriendlyName() + ".";
        	purpcnt++;
        }
        boolean askrename = true;
        for(int i = 0;i<workList.size() && askrename;i++){
        	if(l.isAutoRenamable(workList.get(i).getType()))
        		continue;
            for(int j = i+1;j<workList.size();j++)
                if(workList.get(i).getId().equals(workList.get(j).getId()))
                {
                    if(l.isAutoRenamable(workList.get(i).getType()))
                        continue;
                    purposes += "\n- Selection contains elements with same ids.";
                    purpcnt++;
                    ccc+=2;
                    askrename = false;
                    break;
                }
        }
        Shell shell = Activator.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell();
        if(!purposes.equals(""))
        	if(!MessageDialog
                    .openConfirm(shell, "Move Element"+generalssting,
                            "Moved element"+(generalssting.equals("")?" has":generalssting+" have")+" id conflict"+(purpcnt>1?"s":"")+":\n" + purposes + "\n" 
                                    + "\nConflicting element"+adds+" will be renamed. If you still want to proceed click OK, otherwise click Cancel."))
                    {
                        return Status.CANCEL_STATUS;
                    }
        adds = ccc>1?"s":"";
        if(workList.get(0).getTreeDB().startTransaction("Move Element"+generalssting,true)){
        	boolean flaglast = false;
        	if (target.sizeChildren() > 0 && lastNode == target.getSortedChildren()[target.sizeChildren()-1])
        		flaglast = true;
        	if(target!=lastNode){
        		TreeNode[]sorted = target.getSortedChildren();
        		for(int j = 0;j<sorted.length;j++){
        			Index lastInd;
        			if(j>0)
        				lastInd = Index.create(Integer.toString(j+1));
        			else	
        				lastInd = Index.createOne();
        			sorted[j].setIndex(lastInd);
        			sorted[j].saveAttributes();
        		}
			}
        	for(int i = 0; i < workList.size(); i++){
        		String possibleId = "";
        		boolean isIdValid = target.getTreeDB().getTreeLogic().isChildIdValid(target, 
        				workList.get(i), workList.get(i).getId());
        		if(!isIdValid){
        			Iterable<TreeNode> it = null;
        			String tp = "";
        			if(workList.get(i).getType().equals(Comment.TYPE_NAME))
        			{
        				tp = Comment.TYPE_NAME + " ";
        				it = target;
        			}
        			possibleId = RequalityCLI.getNextId(it, target, workList.get(i).getType(), tp);
        			workList.get(i).setId(RequalityCLI.getNextId(workList.get(i).getParent(),
        					target, workList.get(i).getType(), ""));
        		}
        		if(!target.getTreeDB().getTreeLogic().canBeChildOf(workList.get(i), target))
        		{
        			target.getTreeDB().rollback();
        			MessageDialog
                        	.openError(getShell(), "Move Element"+generalssting,
                        			"Selected elements cannot be moved together. ");
        			return Status.CANCEL_STATUS;
        		}
        		if(workList.get(i).getParent() != target)
        			workList.get(i).setParent(target);
        		if(target!=lastNode){
        			workList.get(i).setIndex(Index.getNextIndex(workList.get(i).getType(), target, lastNode, workList.get(i).getType().equals("TestStep")));
    				workList.get(i).saveAttributes();
        		}else
        			if(target.getTreeDB().getTreeLogic().usesUndexByDefault(workList.get(i).getType()) || target.getTreeDB().getSortingMehod() == SortingMethod.BY_INDEX){
        				workList.get(i).setIndex(Index.getNextIndex(workList.get(i).getType(), target, lastNode, workList.get(i).getType().equals("TestStep")));
        				workList.get(i).saveAttributes();
        			}
        		if(flaglast && target.sizeChildren() > 0){
        			lastNode = target.getSortedChildren()[target.sizeChildren()-1];
        		}else{
        			lastNode = workList.get(i);
        		}
        		if(!isIdValid)
        			workList.get(i).setId(possibleId);
        	}
        	workList.get(0).getTreeDB().commit();
        	return Status.OK_STATUS;
        }
        if(workList.get(0).getTreeDB().getStorage() instanceof TransactionStorage)
        	workList.get(0).getTreeDB().rollback();
        return Status.CANCEL_STATUS;
    }
}
