/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.ui.cnf;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPreferenceConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.navigator.CommonNavigator;
import org.eclipse.ui.navigator.CommonViewer;
import org.eclipse.ui.progress.UIJob;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.ITreeNodeFilter;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.TreeNodeChangeEvent;
import com.unitesk.requality.core.attribute.Attribute;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.core.IUIStateChangeListener;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.ui.cnf.filters.AbstractNodeFilter;
import com.unitesk.requality.eclipse.views.RequalityContentProvider;
import com.unitesk.requality.eclipse.views.RequalityPreferences;

public class RequalityCNF extends CommonNavigator implements
        ITabbedPropertySheetPageContributor {
	public static boolean shiftPressed = false;
    public final static String ID = "com.unitesk.requality.views.RequalityExplorer";
    public static RequalityCNF activeNavigator;
    private boolean recAdd(TreeNode node, List<TreeNode> list){
    	if(list.contains(node))
    		return false;
    	list.add(node);
    	int size = node.sizeChildren();
    	for(int i = 0;i<size;i++)
    		if(!node.getChild(i).isDisposed())
    			recAdd(node.getChild(i), list);
    	return true;
    }
    private List<TreeNode> localToRefresh = new ArrayList<TreeNode>();
    private void refresh(TreeNode node, boolean intern){
    	if(intern)
    		localToRefresh.add(node);
    	RequalityCNF.this.refresh(node);
    	if(intern)
    		localToRefresh.remove(node);
    }
    private List<StructuredSelection> localToSelect = new ArrayList<StructuredSelection>();
    private void setSelection(StructuredSelection sel, boolean intern){
    	if(intern)
    		localToSelect.add(sel);
    	RequalityCNF.this.selectReveal(sel);
    	if(intern)
    		localToSelect.remove(sel);
    }
    ITreeChangeListener guiTreeChangeListener = new TreeChangeListener() {
        @Override
        public void moved(final TreeNode node, final TreeNode oldParent) {
        	System.out.println("moved "+node.getUUId().toString());
        	if (dirtyNodes.contains(node)) {
                return;
            }
            dirtyNodes.add(node);
        	UIJob job = new UIJob("update requality cnf") {
                @Override
                public IStatus runInUIThread(IProgressMonitor monitor) {
                	dirtyNodes.remove(node);
                	RequalityCNF.this.updateTitle();
                	refresh(node.getParent(), true);
                	if(oldParent!=node.getParent())
                		refresh(oldParent, true);
                	//refresh(node, true);
                    return Status.OK_STATUS;
                }
            };
            job.schedule();
        }

        @Override
        public void deleted(final NodeDesc nodeDesc) { //TODO: sync? Job?
        	System.out.println("deleted "+nodeDesc.getUUId().toString());
            UIJob job = new UIJob("update requality cnf") {
                @Override
                public IStatus runInUIThread(IProgressMonitor monitor) {
                	try{
                		refresh(db.getNode(nodeDesc.getParentUUId()), true);
                	}catch(RuntimeException re){
                		//node not fount
                	}
                    return Status.OK_STATUS;
                }
            };
            job.schedule();
        }

        private List<TreeNode> dirtyNodes = new ArrayList<TreeNode>();

        @Override
        public void changed(final TreeNode node, Set<String> what) {
        	System.out.println("changed "+node.getUUId().toString());
        	boolean affect = false,affectParent = false;
        	for(String s:what)
        		if(Attribute.affectsId(node, s)||Attribute.affectsStatus(node, s))
        			affect = true;
        	for(String s:what)
        		if(Attribute.affectsPosition(node, s))
        			affectParent = true;
        	final boolean a=affect,ap=affectParent;
        	if(a || ap)
	        	(new UIJob("update requality cnf") {
	                @Override
	                public IStatus runInUIThread(IProgressMonitor monitor) {
	                	if(ap)
	                		refresh(node.getParent(),true);
	                	if(a&&!ap)
	                		refresh(node);
	                	return Status.OK_STATUS;
	                }
	            }).schedule();
        }

        @Override
        public void created(final TreeNode node) {
        	UIJob job = new UIJob("update requality cnf") {
                @Override
                public IStatus runInUIThread(IProgressMonitor monitor) {
                	refresh(node.getParent(),true);
                	return Status.OK_STATUS;
                }
            };
            job.schedule();
        }

		@Override
		public void changed(List<TreeNodeChangeEvent> changes) {
			Set<TreeNode> toup = new HashSet<TreeNode>();
			for(TreeNodeChangeEvent ce:changes){
				switch (ce.getType()) {
				case CREATE:
					toup.add(ce.getTreeNodeTarget().getParent());
					break;
				case DELETE:{
					TreeNode node = null;
					try{
						node = db.getNode(ce.getNodeDescTarget().getParentUUId());
					}
					catch(Exception e){
					}
					if(node!=null)
						toup.add(node);
				}break;
				case CHANGE:{
					Set<String> what = (Set<String>)ce.getArguments()[1];
					TreeNode node = ce.getTreeNodeTarget();
					boolean affect = false,affectParent = false;
		        	for(String s:what)
		        		if(Attribute.affectsId(node, s)||Attribute.affectsStatus(node, s))
		        			affect = true;
		        	for(String s:what)
		        		if(Attribute.affectsPosition(node, s))
		        			affectParent = true;
		        	final boolean a=affect,ap=affectParent;
		        	if(ap)
		        		toup.add(node.getParent());
		        	if(a&&!ap)
		        		toup.add(node);
				}break;
				case MOVE:{
					TreeNode node = ce.getTreeNodeTarget();
					toup.add(node.getParent());
					TreeNode old = ((TreeNode)ce.getArguments()[1]);
					if(!node.getParent().getUUId().equals(old.getUUId()))
						toup.add(old);
				}
				default:
					break;
				}
			}
			final Set<TreeNode> ftoup = toup; 
			if(toup.size()>0)
				(new UIJob("update requality cnf") {
	                @Override
	                public IStatus runInUIThread(IProgressMonitor monitor) {
	                	for(TreeNode node:ftoup)
	                		refresh(node);
	                	return Status.OK_STATUS;
	                }
	            }).schedule();
		}
    };
    private TreeDB db = null;
    final private IUIStateChangeListener uiListener = new IUIStateChangeListener() {
        @Override
        public void newActiveNodes(final List<TreeNode> nodes) {
            UIJob job = new UIJob("update selection") {
                @Override
                public IStatus runInUIThread(IProgressMonitor monitor) {
                	if (nodes == null) {
                		setSelection(StructuredSelection.EMPTY, true);
                		return Status.OK_STATUS;
                	}
                	else{
                		Object[] objs = ((IStructuredSelection) getCommonViewer()
                				.getSelection()).toArray();
                		if(objs!=null && objs.length == nodes.size()){
                			boolean stop = false;
                			for(int i = 0; i<objs.length && !stop; i++)
                				stop = nodes.contains(objs[i]);
                			if(stop)
                				return Status.CANCEL_STATUS;
                		}
                		if(nodes == null || nodes.size() == 0)
                			return Status.CANCEL_STATUS;
                		if(db != null)
                			db.removeGUIListener(guiTreeChangeListener);
                		db = nodes.get(0).getTreeDB();
                		db.addGUIListener(guiTreeChangeListener);
                		List<TreeNode>sel = new LinkedList<TreeNode>();
                		List<TreeNode>updated = new ArrayList<TreeNode>();
                		for(TreeNode node:nodes){
                			if(node.getTreeDB() != db)
                				continue;
                			TreeNode parentElement = node.getParent(true);
                			if (parentElement == null || !recAdd(parentElement, updated))
                				continue;
                			//refresh(node.getParent(true), true);
                			getCommonViewer().expandToLevel(parentElement, 1);
                			sel.add(node);
                		}
                		StructuredSelection elements = new StructuredSelection(sel);
                		getCommonViewer().removePostSelectionChangedListener(treeSelection);
                		setSelection(elements,false);
                		getCommonViewer().addPostSelectionChangedListener(treeSelection);
                	}
					return Status.OK_STATUS;
                }
            };
            job.schedule();
        }
    };
   
    final private ISelectionChangedListener treeSelection = new ISelectionChangedListener() {
        @Override
        public void selectionChanged(SelectionChangedEvent event) {
            if (event.getSelection() instanceof IStructuredSelection) {
                Iterator<Object> it = ((IStructuredSelection) event.getSelection()).iterator();
                TreeDB db2 = null;
                SortedMap<String, List<TreeNode>> nodes = new TreeMap<String, List<TreeNode>>();
                while(it.hasNext()){
                    Object obj = it.next();
                    //not sure that it is possible
                    if (obj == null)
                    	continue;
                    if (obj instanceof TreeNode) {
                        TreeNode node = (TreeNode) obj;
                        if(db2 == null)
                            db2 = node.getTreeDB();
                        //possible missunderstood here
                        if(db2 != node.getTreeDB())
                            continue;
                        if(db != null)
                            db.removeGUIListener(guiTreeChangeListener);
                        if(node!=null)
                            db = node.getTreeDB(); 
                        db.addGUIListener(guiTreeChangeListener);
                        if(!nodes.containsKey(node.getType()))
                            nodes.put(node.getType(), new LinkedList<TreeNode>());
                        nodes.get(node.getType()).add(node);
                    }
                }
                if(!localToRefresh.contains(event.getSelection())){
                	Map<String,List<TreeNode>> oldNodes = Activator.getDefault().getActiveNodes();
                	Set<String> keys = new HashSet<String>(nodes.keySet());
                	boolean equal = true;
                	for(String s:oldNodes.keySet())
                		if(keys.contains(s)){
                			for(TreeNode node:nodes.get(s))
                				if(!oldNodes.get(s).contains(node))
                					equal = false;
                			for(TreeNode node:oldNodes.get(s))
                				if(!nodes.get(s).contains(node))
                					equal = false;
                			keys.remove(s);
                		}else 
                			equal = false;
                	if(keys.size()>0)
                		equal = false;
                	if(!equal)
                		Activator.getDefault().setActiveNodes(nodes);
                }
            }
        }
    };
    private static boolean shown = false;
    public RequalityCNF() {
        super();
    	activeNavigator = this;
        //check engines
		if(!shown && Activator.getDefault().getPreferenceStore().getBoolean(RequalityPreferences.OPEN_NO_ENGINES_DIALOGS)){
			shown = true;
			//TODO fix window
			if(RequalityPlugin.getBrowserType() == SWT.NONE){
				MessageDialog.openInformation(Activator.getDefault().getWorkbench().getActiveWorkbenchWindow().getShell(), "Browser engines are not available", "There are no available browser engines. Please install XulRunner or WebKit.\nFollow instructions from http://requality.org/en/general.en.html#browser_integration.");
			}
		}
    }

    @Override
    public void dispose() {
        super.dispose();
        getCommonViewer().removePostSelectionChangedListener(treeSelection);
        Activator.getDefault().removeChangeListener(uiListener);
        if(db != null)
            db.removeGUIListener(guiTreeChangeListener);
    }

    @Override
    public void createPartControl(Composite aParent) {
        super.createPartControl(aParent);
        Activator.getDefault().addChangeListener(uiListener);
        getCommonViewer().addPostSelectionChangedListener(treeSelection);
        getCommonViewer().setSorter(new RequalitySorter());
        updateStaticFilter(getCommonViewer());
        getCommonViewer().getControl().addKeyListener(new KeyListener() {
			
			@Override
			public void keyReleased(KeyEvent e) {
				// TODO Auto-generated method stub
				shiftPressed = false;
			}
			
			@Override
			public void keyPressed(KeyEvent e) {
				if((e.keyCode&SWT.SHIFT)!=0)
					shiftPressed = true;
			}
		});
        if(!PlatformUI.getPreferenceStore().getBoolean(IWorkbenchPreferenceConstants.LINK_NAVIGATOR_TO_EDITOR))
        	this.setLinkingEnabled(true);
    }
    

    private void updateStaticFilter(CommonViewer cv) {
        final SortedSet<String> tps = new TreeSet<String>();
        for(ViewerFilter o :cv.getFilters())
			if(o instanceof AbstractNodeFilter){
				tps.addAll(Arrays.asList(((AbstractNodeFilter)o).getFilterTypes()));
			}
        RequalityContentProvider.setStaticFilter(new ITreeNodeFilter() {
			
			@Override
			public boolean select(TreeNode node) {
				if(node == null)
					return false;
				if(node.getType() == null)
					return false;
				return !(tps.contains(node.getType()));
			}
		});
	}

	// This is danger hack
    @Override
    protected CommonViewer createCommonViewerObject(Composite aParent) {
        CommonViewer cv = new CommonViewer(getViewSite().getId(), aParent,
                SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI){
        	@Override
        	public void refresh(){
        		updateStaticFilter(this);
        		super.refresh();
        	}
        };
        return cv;
    }

    @Override
    public String getContributorId() {
        return "com.unitesk.requality.views.RequalityPropertyContributor";
    }

    @Override
    public Object getAdapter(@SuppressWarnings("rawtypes") Class adapter) {
        if (adapter == IPropertySheetPage.class) {
        	//TODO null pointer cause??
            return new TabbedPropertySheetPage(this);
        }
        return super.getAdapter(adapter);
    }
    public static synchronized List<TreeNode> getSelectedNodes() {
        IWorkbenchWindow activeWorkbenchWindow = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow();
        if (activeWorkbenchWindow == null)
            return null;
        ISelection selection = activeWorkbenchWindow.getSelectionService()
                .getSelection();

        if (!(selection instanceof IStructuredSelection)) {
            return null;
        }
        IStructuredSelection selected = (IStructuredSelection) selection;

        if (selected.isEmpty()) {
            return null;
        }
        List<TreeNode> nodes = new ArrayList<TreeNode>();
        List<Object> objList = (List<Object>)selected.toList();
        for(Object obj : objList){
            if (obj instanceof TreeNode) {
                TreeNode node = (TreeNode) obj;
                nodes.add(node);
            } else if (obj instanceof IFile) {
                TreeNode node = TreesTracker.getNodeByResource((IFile) obj);
                if(node!=null)
                	nodes.add(node);
            }
        }
        if(nodes.size() > 0)
            return nodes;
        return null;
    }
    public static synchronized List<Object> getSelectedObj() {
        IWorkbenchWindow activeWorkbenchWindow = PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow();
        if (activeWorkbenchWindow == null)
            return null;
        ISelection selection = activeWorkbenchWindow.getSelectionService()
                .getSelection();

        if (!(selection instanceof IStructuredSelection)) {
            return null;
        }
        IStructuredSelection selected = (IStructuredSelection) selection;

        if (selected.isEmpty()) {
            return null;
        }
        return (List<Object>)selected.toList();
    }
    public void refresh(TreeNode tn){
        if(tn == null)
            return;
        if(this.getCommonViewer() != null){
            try{
                this.getCommonViewer().refresh(tn);
            }
            catch(Exception e){
                //TODO fix unnecessary refresh
            }
        }
    }
    @Override
    protected Object getInitialInput() {
        if(super.getInitialInput()==null) {
            return ResourcesPlugin.getWorkspace().getRoot();
        }
        return super.getInitialInput();
    }
}
