/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;

import com.unitesk.requality.core.INodeChangeListener;
import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.TreeNodeChangeEvent;
import com.unitesk.requality.core.attribute.Attribute;
import com.unitesk.requality.core.attribute.AttributeType;
import com.unitesk.requality.core.attribute.ListValue;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.core.ITreeStateListener;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.ui.RequalityLCNF;
import com.unitesk.requality.nodetypes.VirtualNode;
import com.unitesk.requality.report.TraceabilityDataSource;

public class RequalityContentLinksProvider implements ITreeContentProvider {
    private TreeViewer viewer;
    private ArrayList<TreeDB> dbs = new ArrayList<TreeDB>();
    public RequalityContentLinksProvider() {
    }

    private ITreeChangeListener treeChangeListener = new TreeChangeListener() {
        private List<TreeNode> dirtyNodes = new ArrayList<TreeNode>();
        
        private void guiRefresh(final TreeNode node) {
            if (dirtyNodes.contains(node) || viewer.getControl().isDisposed()) {
                return;
            }
            dirtyNodes.add(node);
            viewer.getControl().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run() {
                    dirtyNodes.remove(node);
                    if (viewer.getControl().isDisposed()) {
                        return;
                    }
                    Object o = viewer.getInput();
                    //System.out.println(node.toString());
                    if(o == null || !(o instanceof TreeNode) ||  node == null ||((TreeNode)o).getUUId().equals(node.getUUId()))
                    	viewer.refresh(true);
                    if(node!=null && !node.isDisposed()&&!viewer.isBusy())
                    	viewer.refresh(node,true);
                }
            });
        }

        @Override
        public void moved(TreeNode node, TreeNode oldParent) {
            guiRefresh(oldParent);
            if(!oldParent.equals(node.getParent()))
            	guiRefresh(node.getParent());
            guiRefresh(node);
        }

        @Override
        public void deleted(NodeDesc node) {
            guiRefresh(null);
        }

        @Override
        public void changed(TreeNode node, Set<String> setts) {
         	boolean changeIt=false,changeParent=false;
        	for(String w:setts){
        		if(Attribute.affectsId(node, w)||Attribute.affectsStatus(node, w))
        			changeIt = true;
        		if(Attribute.affectsPosition(node, w))
        			changeParent = true;
        		if(!changeIt){
        			Attribute a = node.getAttribute(w);
        			if((a!=null)&&(a.getRawType().equals(AttributeType.REFERENCE) ||
        				w.equals(TreeNode.ATTR_REVLINKS)))
        				changeIt = true;
        		}
        		if(changeParent)
        			break;
        	}
        	if(changeParent)
        		guiRefresh(node.getParent());
        	else
        		if(changeIt)
        			guiRefresh(node);
        }

        @Override
        public void created(TreeNode node) {
            guiRefresh(node);
        }

		@Override
		public void changed(List<TreeNodeChangeEvent> changes) {
			Set<TreeNode> toch = new HashSet<TreeNode>();
			for(TreeNodeChangeEvent ch:changes){
				switch (ch.getType()) {
					case MOVE:{
						TreeNode node = ch.getTreeNodeTarget();
						toch.add(node);
						TreeNode old = (TreeNode)ch.getArguments()[1];
						if(old!=null)
							toch.add(old);
			            if(!old.equals(node.getParent()))
			            	toch.add(node.getParent());

						//moved(ch.getTreeNodeTarget(),(TreeNode)ch.getArguments()[1],(Boolean)ch.getArguments()[0]);
					}break;
					case CREATE:
						toch.add(ch.getTreeNodeTarget());
						//created(ch.getTreeNodeTarget(), (Boolean)ch.getArguments()[0]);
					break;
					case DELETE:
						toch.add(null);
						//deleted(ch.getNodeDescTarget(), (Boolean)ch.getArguments()[0]);
					break;
					case CHANGE:{
						TreeNode node = ch.getTreeNodeTarget();
						boolean changeIt=false,changeParent=false;
						Set<String> what = (Set<String>)ch.getArguments()[1];
			        	for(String w:what){
			        		if(Attribute.affectsId(node, w)||Attribute.affectsStatus(node, w))
			        			changeIt = true;
			        		if(Attribute.affectsPosition(node, w))
			        			changeParent = true;
			        		if(!changeIt){
			        			Attribute a = node.getAttribute(w);
			        			if(a!=null && a.getRawType().equals(AttributeType.REFERENCE) ||
			        				w.equals(TreeNode.ATTR_REVLINKS))
			        				changeIt = true;
			        		}
			        		if(changeParent)
			        			break;
			        	}
			        	if(changeParent)
			        		toch.add(node.getParent());
			        	else
			        		if(changeIt)
			        			toch.add(node);
					}break;
				default:
					break;
				}
			}
			final Set<TreeNode> ftoch = toch;
			if(viewer.getControl().isDisposed())
				return;
			 viewer.getControl().getDisplay().asyncExec(new Runnable() {
	                @Override
	                public void run() {
	                    if (viewer.getControl().isDisposed()) {
	                        return;
	                    }
	                    //TODO null update? mb bug
	                    Object o = viewer.getInput();
	                    //System.out.println(node.toString());
	                    if(o == null || !(o instanceof TreeNode) ||  ftoch.contains(null))
	                    	viewer.refresh(true);
	                    else
	                    	if(ftoch.size()!=0)
	                    		viewer.refresh(true);
	                }
	            });
		}
    };

    private ITreeStateListener stateListener = new ITreeStateListener() {
        @Override
        public void open(TreeDB db) {
            dbs.add(db);
            db.addGUIListener(treeChangeListener);
        }

        @Override
        public void delete(IProject proj) {
        	dbs.remove(TreesTracker.getTracker().getTree(proj));
        }

        @Override
        public void close(TreeDB db) {
            dbs.remove(db);
        }
    };

    @Override
    public void dispose() {
        RequalityPlugin.getTreesTracker().removeListener(stateListener);
        
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
        RequalityPlugin.getTreesTracker().removeListener(stateListener);
        if (!(viewer instanceof TreeViewer)) {
            throw new IllegalArgumentException();
        }
        this.viewer = (TreeViewer) viewer;
        for (TreeDB db : dbs) {
            db.removeGUIListener(treeChangeListener);
        }
        dbs.clear();

        RequalityPlugin.getTreesTracker().addListener(stateListener);
        if (newInput == null) {
            return;
        }
        if (newInput instanceof TreeNode) {
            dbs.add(((TreeNode) newInput).getTreeDB());
        } else if (newInput instanceof IWorkspaceRoot) {
            dbs.addAll(Arrays.asList(RequalityPlugin.getTreesTracker()
                    .getTrees()));
        } else if (newInput instanceof Object[]) {
            dbs.add(((TreeNode) ((Object[]) newInput)[0]).getTreeDB());
        } else if (newInput instanceof TreeDB) {
            dbs.add((TreeDB) newInput);
        } else {
            throw new IllegalArgumentException();
        }
        for (TreeDB db : dbs) {
            db.addGUIListener(treeChangeListener);
        }
        viewer.getControl().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                //TODO if control is disposing, old code will throw exception
                if((RequalityContentLinksProvider.this.viewer.getControl() != null) && 
                        !RequalityContentLinksProvider.this.viewer.getControl().isDisposed())
                    RequalityContentLinksProvider.this.viewer.refresh();
            }
        });
    }
    public static class TestNodeLinks{
    	private String name;
    	private Object[] nodes;
    	private TreeNode target;
    	private boolean isup;
    	public TestNodeLinks(boolean up, TreeNode node, String name, Object[] linked){
    		nodes = linked;
    		this.name = name;
    		target = node;
    		isup = up;
    	}
    	public boolean isUp(){
    		return isup;
    	}
    	public Object[]getChildren(){
    		return nodes;
    	}
    	@Override
    	public String toString() {
    		if(isup)
    			return TraceabilityDataSource.getRevertName(VirtualNode.getRootTree(target.getTreeDB()),name);
    		return name;
    	}
		public TreeNode getTarget() {
			return target;
		}
    }

    @Override
    public Object[] getElements(Object inputElement) {
        return getChildren(inputElement);
    }
    
    private TreeNode getTreeNode(Object parentElement){
    	TreeNode parent = null;
    	if (parentElement instanceof IProject) {
            IProject project = (IProject) parentElement;
            if (!project.isOpen()) {
                return null;
            }
            if (TreesTracker.getTracker().getTree(project) != null
                    && TreesTracker.getTracker().getTree(project).getRootNode() != null // Root
                                                                                        // node
                                                                                        // can
                                                                                        // be
                                                                                        // not
                                                                                        // created
                                                                                        // yet
            ) {
                parent = TreesTracker.getTracker().getTree(project)
                        .getRootNode();
            }
        }
        if (parentElement instanceof TreeNode) {
            parent = (TreeNode) parentElement;
        }
        if (parentElement instanceof TreeDB) {
            parent = ((TreeDB) parentElement).getRootNode();
        }
        return parent;
    }

    private void addListReferences(TreeNode parent,String key, Map<String,Object> links, ListValue lv){
    	if(lv.getType() == AttributeType.LIST){
    		for(int i=0;i<lv.getValues().size();i++)
    			addListReferences(parent,key+"["+i+"]", links,(ListValue)lv.getValues().get(i));
    	}else if(lv.getType() == AttributeType.REFERENCE){
    		for(int i=0;i<lv.getValues().size();i++){
    			TreeNode node = parent.resolveLink((String)lv.getValues().get(i), null, key+"["+i+"]");
    			if(node!=null)
    				links.put(key+"["+i+"]",node);
    		}
    	}
    }
    @Override
    public Object[] getChildren(Object parentElement) {
    	if (parentElement instanceof IProject) {
            IProject project = (IProject) parentElement;
            if (!project.isOpen()) {
                return new Object[0];
            }
    	}
    	if(parentElement instanceof TreeNode && parentElement==getTreeNode(viewer.getInput())){
    		TreeNode parent = getTreeNode(parentElement);
    		Map<String,Object> links = new HashMap<String,Object>(),rlinks = new HashMap<String, Object>();
    		List<Attribute> attrs = new ArrayList<Attribute>(parent.getAttributes().values());
    		for(Attribute a:attrs)
    			if(a!=null){
    				if(a.getResultType().equals(AttributeType.REFERENCE)){
    					Object node = parent.getLinkAttributeValue(a.getKey());
    					if(node == null)
    						node = parent.getLinkLabel(a.getKey());
    					links.put(a.getKey(),node);
    				}
    				if(a.getResultType().equals(AttributeType.LIST))
    					addListReferences(parent,a.getKey(),links,(ListValue)a.getRawValue());
    			}
    		Map<TreeNode, Map<String, INodeChangeListener>>revlinks = parent.getRevertLinks();
    		for(TreeNode nd:revlinks.keySet())
    			for(String s:revlinks.get(nd).keySet()){
    				if(!rlinks.containsKey(s))
    					rlinks.put(s, new ArrayList());
    				((ArrayList) rlinks.get(s)).add(nd);
    			}
    		List<Object>tnl = new ArrayList<Object>();
    		Set<String> fl = RequalityLCNF.getFilters(parent.getTreeDB());
    		if((RequalityLCNF.getReferenceDirection()&1)==1)
    			for(String s:links.keySet())
    				if(fl==null || fl.contains(s)||fl.isEmpty())
    					tnl.add(new TestNodeLinks(false, parent, s, new Object[]{links.get(s)}));
    		if((RequalityLCNF.getReferenceDirection()&2)==2)
    			for(String s:rlinks.keySet())
    				if(fl==null || fl.contains(s)||fl.isEmpty())
    					tnl.add(new TestNodeLinks(true, parent, s, ((List)rlinks.get(s)).toArray()));
        	return tnl.toArray();
    	}else if (parentElement instanceof TestNodeLinks)
    		return ((TestNodeLinks)parentElement).getChildren();
    	return new Object[0];
    }

    @Override
    public Object getParent(Object element) {
    	if(element instanceof TestNodeLinks){
    		return ((TestNodeLinks)element).getTarget();
    	}
        if (element instanceof TreeNode) {
            TreeNode node = (TreeNode) element;
            if (node.isDisposed()) {
                return null;
            }
            if (node.getTreeDB().getRootNode().equals(node.getParent())) {
                return TreesTracker.getTracker().getProject(node.getTreeDB());
            }
            return node.getParent();
        }
        return null;
    }

    @Override
    public boolean hasChildren(Object element) {
    	return getChildren(element).length>0;
    }
}
