/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.eclipse.views.properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Requirement;

public class AdvancedPropertySection extends TreeNodePropertiesTab {
    private Text textPredicate;
    private Requirement req;

    public AdvancedPropertySection() {
    }

    @Override
    public void createControls(Composite parent,
            TabbedPropertySheetPage aTabbedPropertySheetPage) {
        super.createControls(parent, aTabbedPropertySheetPage);
        TabbedPropertySheetWidgetFactory wf = aTabbedPropertySheetPage
                .getWidgetFactory();
        
        parent.setLayout(new GridLayout());
        parent.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        wf.createLabel(parent, "Predicate:");
        
        textPredicate = wf.createText(parent, "", SWT.BORDER | SWT.MULTI
                | SWT.H_SCROLL | SWT.V_SCROLL);
        textPredicate.setLayoutData(new GridData(GridData.FILL_BOTH));
        textPredicate.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent e) {
            	if(updating)
            		return;
                //if new attribute text is "", then remove this attribute rather than set it to "",
                //because otherwise we get modified .json-files with empty attribute added,
                //when simply navigating nodes in Requality Explorer and Properties View,
                //and this is affecting .svn
                if(req == null) {
                    return;
                }
                String oldDescr = req.getPredicate();
                if (oldDescr == null) {
                    oldDescr = "";
                }
                String newDescr = textPredicate.getText();
                if (oldDescr.equals(newDescr)) {
                    return;
                }
                if(req.getTreeDB().startTransaction("Update Requirement",true)){
                	req.setPredicate(newDescr);
                	req.saveAttributes();
                	req.getTreeDB().commit();
            	}else
            		textPredicate.setText(oldDescr);
            }
        });
    }

    @Override
    protected void setTreeNode(TreeNode node) {
        req = toRequirement(node);

        if (req == null || textPredicate == null || textPredicate.isDisposed()) {
            return;
        }
        textPredicate.setText(req.getPredicate());
    }
}
