package com.unitesk.requality.eclipse.views.properties;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.UUID;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IFontProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.model.WorkbenchLabelProvider;

import com.unitesk.requality.core.CoreUtils;
import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.ITreeNodeFilter;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.TreeNodeChangeEvent;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.ui.cnf.RequalitySorter;
import com.unitesk.requality.eclipse.ui.dialogs.EditElementDialog;
import com.unitesk.requality.eclipse.ui.dialogs.SelectNodeFromRequirementsTree;
import com.unitesk.requality.nodetypes.NodeType;
import com.unitesk.requality.nodetypes.Requirement;
public class TemplatesPanel implements ITreeChangeListener{
	private class DecorationProvider extends LabelProvider implements /*IColorProvider,*/ IFontProvider{
		@Override
        public Image getImage(Object element) {
        	if(element == null)
        		return null;
            if(element instanceof Requirement) {
                Requirement req = (Requirement) element;
                if(req.getQualifiedId().equals("/Requirements")) {
                    return WorkbenchLabelProvider.getDecoratingWorkbenchLabelProvider().getImage(TreesTracker.getResourceByNode(req).getProject());
                }
            }
            return WorkbenchLabelProvider.getDecoratingWorkbenchLabelProvider().getImage(element);
        }
        @Override
        public String getText(Object element) {
        	if(element instanceof String)
        		return "Empty(Default)";
            if(element instanceof TreeNode) {
                TreeNode nd = (TreeNode) element;
                return nd.getId();
            }
            return WorkbenchLabelProvider.getDecoratingWorkbenchLabelProvider().getText(element);
        }

//		@Override
//		public Color getForeground(Object element) {
//			// TODO Auto-generated method stub
//			return null;
//		}
//
//		@Override
//		public Color getBackground(Object element) {
//			String id = null,templ=null;
//			StructuredSelection ss = (StructuredSelection) nodeTree.getSelection();
//			if(ss!=null && !ss.isEmpty()){
//				Object o = ss.getFirstElement();
//				if(o instanceof TreeNode)
//					templ = ((TreeNode)o).getId();
//			}
//			if(element instanceof TreeNode)
//				id = ((TreeNode) element).getId();
//			//if(id == null && templ == null || id!=null && id.equals(templ))
//				//return new Color(nodeTree.getControl().getDisplay(), new RGB(120,250,120));
//			return null;
//		}
		@Override
		public Font getFont(Object element) {
			String id = null,templ=null;
			TreeNode node = db.getActiveTemplate(filteredType);
			if(node!=null)
				templ = node.getId();
			if(element instanceof TreeNode)
				id = ((TreeNode) element).getId();
			if(id == null && templ == null || id!=null && id.equals(templ))
			{
				Font f = nodeTree.getControl().getFont();
				FontData fd = new FontData(f.getFontData()[0].toString());
				fd.setStyle(fd.getStyle() | SWT.BOLD);
				return new Font(f.getDevice(),fd);
			}
			return null;
		}
		
	}
	protected TreeViewer nodeTree = null;	

    private static int TREE_WIDTH = 250;
    private static int TREE_HEIGHT = 300;

    private java.util.List<String> types = null;
    //private ElementViewPanel viewp;
    private Composite newPar;
	private Button setsel,createt,editt;
	private Combo templlist;
	private Button removeB;
	private TreeDB db;
	private DecorationProvider lprovider = new DecorationProvider();
	private String filteredType = Requirement.TYPE_NAME;
    //protected void setTemplate(TreeNode node){
//    	viewp.setTreeNode(node);
  //  }
    protected ITreeContentProvider getContentProvider() {
    	return new ITreeContentProvider() {
    		private TreeDB db;
			@Override
			public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
				if(newInput == null)
					return;
				if(newInput instanceof TreeDB)
					this.db = (TreeDB)newInput;
				else
					this.db = ((TreeNode)newInput).getTreeDB();
			}
			
			@Override
			public void dispose() {
				//Remove listener on panel dispose
				db.removeListener(TemplatesPanel.this);
			}
			
			@Override
			public boolean hasChildren(Object element) {
				return getElements(element).length>0;
			}
			
			@Override
			public Object getParent(Object element) {
				return db;
			}
			
			@Override
			public Object[] getElements(Object inputElement) {
				if(db!=null && (inputElement instanceof TreeDB||db.getRootNode().equals(inputElement))){
					List<Object> filteredTemplates = new ArrayList<Object>();
					for(TreeNode node:db.getTemplates())
						if(filteredType.equals(node.getType()))
							filteredTemplates.add(node);
					filteredTemplates.add("Empty(Default)");
					return filteredTemplates.toArray();
				}
				return new Object[0];
			}
			
			@Override
			public Object[] getChildren(Object parentElement) {
				return getElements(parentElement);
			}
		};
    }
    
    public TemplatesPanel(final Composite parent) {
    	//parent.setLayout(new GridLayout(1,true));
    	if(parent.getParent().getLayout() instanceof GridLayout)
    		parent.setLayoutData(new GridData(GridData.FILL_BOTH));
    	
    	newPar = new Composite(parent,SWT.NONE);

    	GridLayout gl = new GridLayout(1,false);
    	gl.marginHeight=gl.marginWidth=0;
    	newPar.setLayout(gl);
    	if(parent.getLayout() instanceof GridLayout)
    		newPar.setLayoutData(new GridData(GridData.FILL_BOTH));
    	
    	Composite header = new Composite(newPar,SWT.NONE);
    	header.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	header.setLayout(new GridLayout(2,false));
    	
    	(new Label(header,SWT.NONE)).setText("Select Node Type");
    	
    	templlist = new Combo(header, SWT.READ_ONLY);
    	templlist.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	//GridData gd = new GridData(GridData.FILL_BOTH);
    	//gd.horizontalSpan = 3;
    	//templlist.setLayoutData(gd);
    	templlist.addSelectionListener(new SelectionListener() {
			
			@Override
			public void widgetSelected(SelectionEvent e) {
				int sel = templlist.getSelectionIndex();
				if(sel>-1){
					String s = types.get(sel);
					filteredType = s;
					nodeTree.refresh();
				}
				correctButtonsEnabledState();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				// TODO Auto-generated method stub
				
			}
		});
    	
    	Group mainG = new Group(newPar, SWT.NONE);
    	mainG.setLayoutData(new GridData(GridData.FILL_BOTH));
    	gl = new GridLayout(1,false);
    	gl.marginHeight=gl.marginWidth=0;
    	mainG.setLayout(gl);
    	
    	
    	Composite tbuttons = new Composite(mainG, SWT.NONE);
    	GridLayout l = new GridLayout(2,false);
    	l.marginWidth=l.marginHeight=0;
    	tbuttons.setLayout(l);
    	tbuttons.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
    	
    	createt = new Button(tbuttons,SWT.NONE);
    	createt.setText("Create template");
    	createt.setEnabled(true);
    	createt.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				int sel = templlist.getSelectionIndex();
				String type = types.get(sel);
				//hack to init templates folder before transaction
				db.getOrCreateTemplatesFolder();
				if(db.startTransaction("Create template of '"+CoreUtils.splitWords(type)+"' type."))
				{
					//TreeNode act = db.getActiveTemplate(type);
					TreeNode node = db.createTemplate(type);
					EditElementDialog eed = new EditElementDialog(parent.getShell(), node, 
							"Create "+db.getNodeType(type).getVisibleClassName()+" template", true);
					if(eed.open()==eed.OK){
						node.saveAttributes();
						db.commit();
					}else
						db.rollback();
					//db.setActiveTemplate(type, act==null?null:act.getId());
				}
				nodeTree.refresh();
				
			}
    		
    	});
    	
    	removeB = new Button(tbuttons,SWT.NONE);
    	removeB.setText("Extract template");
    	removeB.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				SelectNodeFromRequirementsTree sn = new SelectNodeFromRequirementsTree(parent.getShell(),db) {
				};
				sn.setContentFilter(new ITreeNodeFilter() {
					private Map<TreeNode,Boolean> enTree = new HashMap<TreeNode,Boolean>();
					@Override
					public boolean select(TreeNode node) {
						if(node.isRestricted())
							return false;
						if(enTree.containsKey(node))
							return enTree.get(node);
						if(node.getType().equals(filteredType)){
							enTree.put(node, true);
							return true;
						}
						for(TreeNode node2:node.getSortedChildren())
							if(select(node2)){
								enTree.put(node, true);
								return true;
							}
						enTree.put(node, false);
						return false;
					}
				});
				sn.setFilter(new ITreeNodeFilter() {
					@Override
					public boolean select(TreeNode node) {
						return(node.getType().equals(filteredType)||
								node.getTreeDB().getTreeLogic().canBeChildOf(filteredType,node.getType()));
						//return node.getTreeDB().getTreeLogic().canBeTemplate(node.getType());
					}
				});
				sn.setEnablementFilter(new ITreeNodeFilter() {
					@Override
					public boolean select(TreeNode node) {
						return(node.getType().equals(filteredType));
						//return node.getTreeDB().getTreeLogic().canBeTemplate(node.getType());
					}
				});
				int res = sn.open();
				if(res == sn.OK){
					TreeNode node = sn.getResult();
					//hack to init templates not in transaction
					node.getTreeDB().getOrCreateTemplatesFolder();
					if(node.getTreeDB().startTransaction("Extract Template '"+node.getNameOrId()+"'")){
						if(node!=null)
							node.getTreeDB().extractTemplate(node, node.getNameOrId());
						node.getTreeDB().commit();
					}
				}
				nodeTree.refresh(true);
			}
		});
    	removeB.setEnabled(true);
    	//newPar.setLayoutData(new GridData(GridData.FILL_BOTH));
    	
    	new Label(mainG,SWT.NONE).setText("List of templates:");
    	
    	Composite list = new Composite(mainG,SWT.NONE);
    	list.setLayoutData(new GridData(GridData.FILL_BOTH));
    	l = new GridLayout(2,false);
    	l.marginWidth=l.marginHeight=0;
    	list.setLayout(l);
    	
    	nodeTree = new TreeViewer(list);
    	GridData gd = new GridData(GridData.FILL_BOTH);
    	gd.widthHint = TREE_WIDTH;
    	gd.horizontalSpan = 2;
    	nodeTree.getControl().setLayoutData(gd);
        nodeTree.setLabelProvider(getLabelProvider());
        nodeTree.setAutoExpandLevel(2);
        nodeTree.setComparator(new RequalitySorter(){
        	@Override
        	public int compare(Viewer viewer, Object e1, Object e2) {
        		if(e1 instanceof String)
        			return -1;
        		if(e2 instanceof String)
        			return -1;
        		return super.compare(viewer, e1, e2);
        	}
        });
        if(db!=null)
        	nodeTree.setInput(db);
        
        nodeTree.setContentProvider(getContentProvider());
        nodeTree.getControl().setLayoutData(new GridData(GridData.FILL_BOTH));
    	
        nodeTree.addSelectionChangedListener(new  ISelectionChangedListener() {
			
			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				updateTemplListSel();
				correctButtonsEnabledState();
			}
		});
        
    	nodeTree.addDoubleClickListener(new IDoubleClickListener() {
			
			@Override
			public void doubleClick(DoubleClickEvent event) {
				callEditTemplate();
			}
		});
        
        Composite buttons2 = new Composite(list, SWT.NONE);
        l = new GridLayout(1,false);
    	l.marginLeft=l.marginHeight=0;
        buttons2.setLayout(l); 
        buttons2.setLayoutData(new GridData(SWT.RIGHT,SWT.TOP,false,true));
        
        
        gd = new GridData(SWT.LEFT,SWT.CENTER,true,false);
        gd.widthHint = 100;
        
        editt = new Button(buttons2,SWT.NONE);
    	editt.setText("Edit");
    	editt.setEnabled(false);
    	editt.setLayoutData(gd);
    	editt.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				callEditTemplate();
			}
		});
    	
    	gd = new GridData(SWT.LEFT,SWT.CENTER,true,false);
        gd.widthHint = 100;
        
        removeB = new Button(buttons2,SWT.NONE);
    	removeB.setText("Remove");
    	removeB.setLayoutData(gd);
    	removeB.addListener(SWT.Selection, new Listener() {
			
			@Override
			public void handleEvent(Event event) {
				StructuredSelection ss = (StructuredSelection) nodeTree.getSelection();
				if(ss!=null && !ss.isEmpty()){
					TreeNode node = (TreeNode) ss.getFirstElement();
					node.getTreeDB().getOrCreateTemplatesFolder();
					if(node.getTreeDB().startTransaction("Remove template "+node.getId())){
						node.getTreeDB().removeNode(node);
						for(String s : node.getTreeDB().getActiveTemplates().keySet())
							if(node.getTreeDB().getActiveTemplates().get(s).contains(node.getId())&&
									node.getTreeDB().getActiveTemplates().get(s).equals(node.getId()))
										node.getTreeDB().setActiveTemplate(node.getType(), null);
						node.getTreeDB().commit();
					}
					nodeTree.refresh(true);
					clearSelection();
				}
			}
		});
    	
    	gd = new GridData(SWT.LEFT,SWT.CENTER,true,false);
        gd.widthHint = 100;
        
    	setsel = new Button(buttons2,SWT.NONE);
    	setsel.setText("Set as active");
    	setsel.setLayoutData(gd);
    	setsel.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				TreeSelection ts = (TreeSelection)nodeTree.getSelection();
				if(ts!=null)
				{
					String id = null;
					Object o = ts.getFirstElement();
					removeB.setEnabled(false);
					if(o!=null){
						if(o instanceof TreeNode){
							removeB.setEnabled(true);
							id = ((TreeNode) o).getId();;
						}
						db.getOrCreateTemplatesFolder();
						if(db.startTransaction("Change '"+CoreUtils.splitWords(filteredType)+"' template 'to "+(id==null?"Default":id)+"'")){
							db.setActiveTemplate(filteredType, id);
							db.commit();
						}
					}
					nodeTree.refresh();
				}
				updateTemplatesList(db);
			}
		});
    	
//        nodeTree.addSelectionChangedListener(new NodeSelectionListener());

  //      nodeTree.getTree().addMouseListener(mouseList);
        //nodeTree.getTree().addKeyListener(keyList);
        
    	//TODO tree
    	
    	if(db!=null){
    		updateTemplatesList(db);
        	templlist.select(types.indexOf(filteredType));
    	}
    	clearSelection();
    	//if(seltp!=null)
    		//setSelectedType(seltp);
    	//viewp = new ElementViewPanel(details, 2, labels,node);
    	    	
    };
    
    protected void callEditTemplate() {
    	TreeSelection sel = (TreeSelection) nodeTree.getSelection();
		if(sel!=null&&!sel.isEmpty()&&sel.getFirstElement() instanceof TreeNode){
			TreeNode node = (TreeNode) sel.getFirstElement();
			if(node!=null)
				openEditor(node);
		}

	}

	protected void correctButtonsEnabledState() {
    	setsel.setEnabled(false);
		createt.setEnabled(true);
		String tpe = filteredType;
		StructuredSelection s = (StructuredSelection) nodeTree.getSelection();
		editt.setEnabled(false);
		if(s!=null && !s.isEmpty()&&s.getFirstElement() instanceof TreeNode){
			TreeNode selcd =(TreeNode) s.getFirstElement();
			if(selcd!=null && selcd.getType().equals(tpe))
				editt.setEnabled(true);
			
		}
		setsel.setEnabled(true);
				
	}

	protected boolean updateTemplListSel() {
    	TreeSelection ts = (TreeSelection) nodeTree.getSelection();
    	editt.setEnabled(false);
    	removeB.setEnabled(false);
		if(ts!=null&&!ts.isEmpty()&& ts.getFirstElement() instanceof TreeNode){
			TreeNode node = (TreeNode) ts.getFirstElement();
			if(node!=null){
				removeB.setEnabled(true);
				editt.setEnabled(true);
				return true;
				//java.util.List<String> vals = getTypesList(db);
				//for(int i = 0;i<vals.size();i++)
					//if(vals.get(i).equals(node.getType())){
						//templlist.select(i);
						//updateSelectedTemplateLabel(getSelectedTemplType());
						//return true;
					//}
			}
		}
		return false;
	}

    public void clearSelection(){
    	//clearsel.setEnabled(false);
    	setsel.setEnabled(false);
    	//templlist.select(-1);
    }
	protected String getSelectedTemplType() {
		TreeSelection ts = (TreeSelection) nodeTree.getSelection();
		if(ts!=null&&!ts.isEmpty()&&ts.getFirstElement() instanceof TreeNode){
			TreeNode node = (TreeNode) ts.getFirstElement();
			if(node!=null)
				return node.getType();
		}
		return null;
	}
    private java.util.List<String> getTypesList(TreeDB db2){
    	if(types== null){
    		Set<String> st = new TreeSet<String>(); 
    		for(String s:db2.getTypes().keySet())
    			if(db2.getTreeLogic().canBeTemplate(s))
	    			try {
	    				TreeNode node = ((NodeType)db2.getTypes().get(s)).getTypeClass().getConstructor( TreeDB.class,
	                            UUID.class).newInstance(db2,UUID.randomUUID());
	    				if(node!=null && !node.isRestricted() && !(node.getClass().equals(TreeNode.class))&&
	    						node.getTreeDB().getTreeLogic().canBeTemplate(s))
	    					st.add(s);
	    			} catch (InstantiationException e) {
						e.printStackTrace();
					} catch (IllegalAccessException e) {
						e.printStackTrace();
					} catch (IllegalArgumentException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (SecurityException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (InvocationTargetException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (NoSuchMethodException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
    		types = new ArrayList<String>();
    		types.addAll(st);
    	}
    	return types;
    }
	private void updateTemplatesList(TreeDB db2) {
		templlist.removeAll();
		SortedSet<String> ss = new TreeSet<String>();
		for(String s:getTypesList(db2))
			templlist.add(s);
		templlist.select(types.indexOf(filteredType));
		updateTemplListSel();
		
	}
	protected void openEditor(TreeNode node) {
    	if(node.getTreeDB().startTransaction("Change template '"+node.getId()+"'("+node.getType()+")"))
    	{
        	Map<String,String> labels = new HashMap<String,String>(ElementViewPanel.getDefaultLabels());
        	labels.put("idLabel","Template Id:");
    		EditElementDialog eed = new EditElementDialog(nodeTree.getTree().getShell(),
    				node, "Edit template '"+node.getId()+"'("+node.getType()+")", true, labels);
    		eed.setTransactionIgnore(true);
    		if(eed.open() == Dialog.CANCEL)
    			node.getTreeDB().rollback();
    		else{
    			node.getTreeDB().commit();
    			nodeTree.refresh(node);
    		}
    	}
		
	}
	protected ILabelProvider getLabelProvider() {
        return lprovider;
    }
	public void setTreeDB(TreeDB db2){
		if(db!=null)
			db.removeListener(this);
		db = db2;
		db.addListener(this);
		nodeTree.setInput(db);
		nodeTree.refresh(true);
		if(db!=null)
    		updateTemplatesList(db);
	}
	
    @Override
	public void moved(TreeNode node, TreeNode oldParent, boolean internal) {
	}
	
	@Override
	public void deleted(NodeDesc nodeDesc, boolean internal) {
		if(db!=null && nodeDesc.getParentUUId().equals(db.getOrCreateTemplatesFolder().getUUId())){
			nodeTree.refresh();
			clearSelection();
		}
	}
	
	@Override
	public void created(TreeNode node, boolean internal) {
		if(db!=null && node.getParentUUId().equals(db.getOrCreateTemplatesFolder().getUUId())){
			nodeTree.refresh();
			clearSelection();
		}
	}
		
	public void setSelectedType(String tp){
		filteredType = tp;
		for(int i = 0;i<types.size();i++)
			if(types.get(i).equals(filteredType))
			{
				templlist.select(i);
				nodeTree.refresh();
				break;
			}
	}

	public Composite getComposite() {
		return newPar;
	}

	@Override
	public void changed(TreeNode node, Set<String> params, boolean internal) {
		if(node.getUUId().equals(node.getTreeDB().getOrCreateTemplatesFolder().getUUId())){
			if(params.contains(TreeDB.ACTIVE_TEMPLATES_ATTRIBUTE)){
				setTreeDB(node.getTreeDB());
				//updateTemplatesList(node.getTreeDB());
				//nodeTree.refresh();
			}
		}
	}

	@Override
	public void changed(List<TreeNodeChangeEvent> changes) {
		//TODO remade
		for(TreeNodeChangeEvent ce:changes)
		{
			switch (ce.getType()) {
				case DELETE:
					deleted(ce.getNodeDescTarget(), (Boolean)ce.getArguments()[0]);
				break;
				case CREATE:
					created(ce.getTreeNodeTarget(),(Boolean)ce.getArguments()[0]);
				break;
				case MOVE:
					moved(ce.getTreeNodeTarget(),(TreeNode)ce.getArguments()[1],(Boolean)ce.getArguments()[0]);
				break;
				case CHANGE:
					changed(ce.getTreeNodeTarget(),(Set<String>)ce.getArguments()[1],(Boolean)ce.getArguments()[0]);
				break;
			default:
				break;
			}
		}
	}
}
