package com.unitesk.requality.eclipse.wizards;


import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.jface.dialogs.DialogSettings;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IImportWizard;
import org.eclipse.ui.IWorkbench;
import org.w3c.dom.Document;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.tools.ReqIFConverter;

public class ImportFromReqIFWizard extends Wizard implements IImportWizard {

	private IStructuredSelection selection;
	private TreeDB db;
	private WorkspaceFileSelectionPage fp;
	private ReqMappingPage map,map2;

	public ImportFromReqIFWizard() {
		super();
	}
	
	@Override
	public void addPages() {
    	TreeDB db = getSelectedTreeDB();
        StructuredSelection sel;
        if(db != null)
        	sel = new StructuredSelection(db);
        else
        	sel = new StructuredSelection();
        addPage(fp);
        addPage(map);
        addPage(map2);
	};
	private TreeDB getSelectedTreeDB() {
    	if(!(selection instanceof StructuredSelection))
    		return db;
    	TreeDB db = this.db;
    	Object o = ((StructuredSelection)selection).getFirstElement();
    	if(o instanceof IProject)
    		db = TreesTracker.getTracker().getTree((IProject)o);
    	if(o instanceof TreeNode)
    		db = ((TreeNode)o).getTreeDB();
    	if(o instanceof TreeDB)
    		db = (TreeDB)o;
    	this.db = db;
    	return db;
	}
	@Override
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		setWindowTitle("ReqIF to Requality import wizard");
        this.selection = selection;
        DialogSettings ds = new DialogSettings("importFromReqIf");
        fp = new WorkspaceFileSelectionPage(ds,new String[]{"reqif"});
        map = new ReqMappingPage(ds);
        
        map2 = new ReqMappingPage(ds){
        	@Override
        	public void updateSecondField() {
        		Map<String,String> mapp1 = map.getMapping();
        		if(mapp1==null || mapp1.size()<1)
        			return;
        		Set<String> ret = new HashSet<String>();
        		ret.add("_id");
        		ret.add("_name");
        		ret.add("_description");
        		map2.initSecondField(ret);
        	}
        	@Override
        	public void updateFirstField() {
        		Map<String,String> mapp1 = map.getMapping();
        		if(mapp1==null || mapp1.size()<1)
        			return;
        		Set<String> types = new HashSet<String>(mapp1.keySet());
        		Document doc = ReqIFConverter.loadReqIFModel(fp.getSelectedResource());
        		if(doc!=null){
        			Map<String,Set<String>> attrs = ReqIFConverter.loadReqIFAttributes(doc, types);
        			Set<String> ret = new HashSet<String>();
        			for(String s:attrs.keySet()){
        				for(String s2:attrs.get(s))
        					ret.add(s+"."+s2);
        				ret.add(s+".LongName");
        			}
        			map2.initFirstField(ret);
        		}
        	}
        	@Override
        	protected boolean isClose(String s, String k) {
        		return k.length()>0 && s.endsWith(k);
        	}
        };
        map2.setTitle("Attributes mapping");
		map2.setDescription("Map ReqIF attributes to Requality attributes.");
		
	}

	@Override
	public boolean canFinish() {
		return map.getMapping()!=null && !map.getMapping().isEmpty() && fp.getSelectedResource()!=null && fp.getSelectedResource().exists();
	};
	@Override
	public boolean performFinish() {
		Map<String,String>mapp1=map.getMapping();
		Map<String,String>mapp2=map2.getMapping();
		Document d = ReqIFConverter.loadReqIFModel(fp.getSelectedResource());
		ReqIFConverter.convertToCatalog(d,mapp1,mapp2);
		return true;
	}

	public IFile getSelectedFile() {
		return (IFile) fp.getSelectedResource();
	}

}
