package com.unitesk.requality.eclipse.wizards.reports;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DateTime;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.attribute.Attribute;
import com.unitesk.requality.core.attribute.AttributeType;
import com.unitesk.requality.nodetypes.ReportSettings;


public class ProgressReportSettingsWizard extends Dialog{
	String dateFrom,dateTo,step;
	DateTime dtfrom,dtto,dtfromtime,dttotime;
	Text stepText;
	Composite p;
	boolean changed = false;
	String def = "";
	@Override
    protected Control createDialogArea(Composite parent) {
		getShell().setText("Progress report settings");
		parent.setLayout(new GridLayout(1,true));
		parent.setLayoutData(new GridData(GridData.FILL_BOTH));
		p = new Composite(parent, SWT.NONE);
		GridData gd = new GridData(GridData.FILL_BOTH);
		p.setLayoutData(gd);
		p.setLayout(new GridLayout(2,true));
		(new Label(p,SWT.NONE)).setText("Date from:");
		(new Label(p,SWT.NONE)).setText("Date to:");
		dtfrom = new DateTime(p, SWT.CALENDAR);
		dtto = new DateTime(p, SWT.CALENDAR);
		dtfromtime = new DateTime(p, SWT.TIME);
		dttotime = new DateTime(p, SWT.TIME);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		gd.widthHint = 400;
		Text helpStep = new Text(p,SWT.MULTI|SWT.WRAP);
		helpStep.setEditable(false);
		helpStep.setLayoutData(gd);
		helpStep.setBackground(helpStep.getParent().getBackground());
		helpStep.setText("Step in format of pairs number - period. For example, '1 week 2 day'. Allowed "
						+ " periods are 'day', 'week', 'year', 'month'. Shorten names can be used instead"
						+ " ('d','w','y','m'). If not defifed, this parameter will be selected automatically.");
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = 2;
		stepText = new Text(p,SWT.NONE);
		stepText.setLayoutData(gd);
		DateFormat df = ReportSettings.dataFormat;
		Calendar c = Calendar.getInstance();
		if(dateFrom!=null){
			try {
				c.setTime(df.parse(dateFrom));
			} catch (ParseException e) {
				e.printStackTrace();
			}
			dtfrom.setDate(c.get(Calendar.YEAR), c.get(Calendar.MONTH), c.get(Calendar.DAY_OF_MONTH));
			dtfromtime.setTime(c.get(Calendar.HOUR), c.get(Calendar.MINUTE), c.get(Calendar.SECOND));
		}
		
		if(dateTo!=null){
			try {
				c.setTime(df.parse(dateTo));
			} catch (ParseException e) {
				e.printStackTrace();
			}
			dtto.setDate(c.get(Calendar.YEAR), c.get(Calendar.MONTH), c.get(Calendar.DAY_OF_MONTH));
			dttotime.setTime(c.get(Calendar.HOUR), c.get(Calendar.MINUTE), c.get(Calendar.SECOND));
		}
		if(step!=null){
			stepText.setText(step);
			changed = true;
		}else
			stepText.setText("1 week");
		//setControl(p);
		//shell.layout(true, true);
		final Point newSize = shell.computeSize(SWT.DEFAULT, SWT.DEFAULT, true);  
		//shell.setSize(parent.getSize());
		return p;
	}
	private Shell shell;
	TreeNode rep;
	public ProgressReportSettingsWizard(Shell shell, TreeNode rep){
		super(shell);
		this.shell = shell;
		this.rep = rep;
		Attribute df = rep.getAttribute("dateFrom");
		if(df!=null && df.getResultType().equals(AttributeType.STRING))
			dateFrom = df.getValue().toString();
		Attribute dt = rep.getAttribute("dateTo");
		if(dt!=null && dt.getResultType().equals(AttributeType.STRING))
			dateTo = dt.getValue().toString();
		Attribute st = rep.getAttribute("step");
		if(st!=null && st.getResultType().equals(AttributeType.STRING))
			step = st.getValue().toString();
		
	}

    @Override
    protected void okPressed() {
		try{
			if(!rep.getTreeDB().startTransaction("Change report settings"))
				return;
			Calendar c = Calendar.getInstance();
			c.set(dtfrom.getYear(),dtfrom.getMonth(), dtfrom.getDay(), dtfromtime.getHours(), dtfromtime.getMinutes(), dtfromtime.getSeconds());
			String dval1 = ReportSettings.dataFormat.format(c.getTime());
			if(dateFrom==null||!dateFrom.equals(dval1))
				rep.putAttribute(new Attribute(rep,AttributeType.STRING,"dateFrom",dval1));
			c.set(dtto.getYear(), dtto.getMonth(), dtto.getDay(), dttotime.getHours(), dttotime.getMinutes(), dttotime.getSeconds());
			String dval2 = ReportSettings.dataFormat.format(c.getTime());
			if(dateTo==null||!dateTo.equals(dval2))
				rep.putAttribute(new Attribute(rep,AttributeType.STRING,"dateTo",dval2));
			if(step==null || !stepText.getText().equals(step))
				rep.putAttribute(new Attribute(rep,AttributeType.STRING,"step",stepText.getText()));
			rep.saveAttributes();
			rep.getTreeDB().commit();
			super.okPressed();
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}

}
