/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.nodetypes;

import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.UUID;

import com.unitesk.requality.core.INodeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;

public class ReportSettings extends TreeNode {
	public static SimpleDateFormat dataFormat = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy", Locale.ENGLISH);
	protected String lastTemplate = null;
	private INodeChangeListener ncl = new INodeChangeListener() {
		@Override
		public void deleted(TreeNode source) {
		}
		
		@Override
		public void attributeChange(TreeNode source, String what) {
			String old = source.getStringAttributeValue(ATTR_TEMPLATE);
			if(source == ReportSettings.this
					&& (what.equals(ATTR_TEMPLATE) || what.equals(TreeNode.ATTR_UNDEFINED))
					&& old != null
					&& (lastTemplate == null || !lastTemplate.equals(old))){
				lastTemplate = old;
				//if(source.getTreeDB().getStorage() instanceof ResourceStorage)
					//GenerateReportHandler.generateReport(ReportSettings.this);
					
			}
		}

		@Override
		public void moved(TreeNode source, TreeNode oldParent,
				TreeNode newParent) {
			// TODO Auto-generated method stub
			
		}
	};
    public static final String TYPE_NAME = "ReportSettings";
    public static final String ATTR_ROOT_REQUIREMENT = "_root_requiremnt_qid";
    public static final String ATTR_DATE = "date";
    public static final String ATTR_TEMPLATE = "_template_id";
    public static final String ATTR_LAST_GENERATED = "_last_generated";
    public ReportSettings() {
        this(null, null);
    }

    public ReportSettings(TreeDB db, UUID uuid) {
        super(db, uuid);
        this.addChangeListener(ncl);
        lastTemplate = this.getStringAttributeValue(ATTR_TEMPLATE);
    }

    @Override
    public String getMultiTypeLabel(){
    	return getTypeLabel();
    }
	
    public String getStartRequirementQId() {
        return (String) getAttributeValue(ATTR_ROOT_REQUIREMENT, Requirement.getTypeRootQId());
    }

    public Map<String, Object> getData() {
        Map<String, Object> result = new HashMap<String, Object>();
        for (String key : getAttributeKeys()) {
            if (!key.startsWith("_")) {
                result.put(key, getAttributeValue(key));
            }
        }
        return result;
    }

    public String getSaveDir() {
        return getTreeDB().getStorage().getNodeResourcePath(getUUId());

    }

    public String getTemplateId() {
        return (String) getAttributeValue(ATTR_TEMPLATE);
    }
    @Override
    public boolean hasUserAttributes() {
    	return true;
    }
}
