/*
 * Copyright 2015 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.nodetypes;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.eclipse.core.resources.IProject;

import com.unitesk.requality.core.FSStorage;
import com.unitesk.requality.core.INodeChangeListener;
import com.unitesk.requality.core.ITreeChangeListener;
import com.unitesk.requality.core.ITreeStorage;
import com.unitesk.requality.core.NodeDesc;
import com.unitesk.requality.core.TreeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.TreeNodeChangeEvent;
import com.unitesk.requality.core.attribute.Attribute;
import com.unitesk.requality.core.attribute.AttributeGenerator;
import com.unitesk.requality.core.attribute.AttributeType;
import com.unitesk.requality.core.instancer.TreeInstancer;
import com.unitesk.requality.core.nodeiterators.RequalityChildIterator;
import com.unitesk.requality.core.nodeiterators.RequalityChildIterator.RequalityChildIteratorType;
import com.unitesk.requality.core.nodeiterators.RequalityIterator;
import com.unitesk.requality.core.nodeiterators.RequalityIteratorType;
import com.unitesk.requality.core.transaction.TransactionStorage;
import com.unitesk.requality.eclipse.core.TreesTracker;

public class VirtualNode extends TreeNode {
    public static final String TYPE_NAME = "VirtualNode";
    public static final String ATTR_CHILD_ITERATOR = "_childiterators";
	public static final String ATTR_TARGET = "_target";
	public static final String ATTR_TREECHANGES = "_changes";
	public static final String ATTR_HIDDEN_STATUS = "_hiddenstate";
	
    private TreeNode target;
    
    private static Map<TreeDB,TreeInstancer> instanceStorages = new HashMap<TreeDB,TreeInstancer>();
    private static HashMap<TreeDB,TreeDB> instanceTreeDBs = new HashMap<TreeDB, TreeDB>(),
    									  revInstanceTreeDBs = new HashMap<TreeDB, TreeDB>();
    
    private AttributeGenerator iditeration;
    private RequalityChildIterator childiterator;
    private boolean childrenLoaded = false;
	private INodeChangeListener targetChange = new INodeChangeListener() {
		@Override
		public void moved(TreeNode source, TreeNode oldParent, TreeNode newParent) {
		}
		
		@Override
		public void deleted(TreeNode source) {
			try {
				setTarget(null);
				saveAttributes();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		@Override
		public void attributeChange(TreeNode source, String what) {
		}
	};
	
    @Override
    public void setParent(TreeNode parent)  {
    	TreeNode targ = getTarget();
    	try {
			setTarget(null);
			super.setParent(parent);
			if(getChildIterator().checkTarget(targ)==null)
				setTarget(targ);
			saveAttributes();
		} catch (Exception e) {
			e.printStackTrace();
		}
    	
    };
    public RequalityChildIterator getChildIterator(){
    	if(childiterator == null){
    		Attribute a = getAttribute(ATTR_CHILD_ITERATOR);
    		if(a != null){
    			String val = (String)a.getValue();
    			if(val != null){
    				String[] s = val.split("\\|(?=(?:[^\"]*[\"][^\"]*[\"])*[^\"]*$)");
    				if(s.length>1)
    				{
    					childiterator = RequalityChildIterator.fromAttributeString(null, this, val);
    				}else
    					childiterator = (RequalityChildIterator) RequalityIterator.fromAttributeString(getUUId(), val);
    			}
    		}
    	}
    	//first initiation
    	if(childiterator == null){
    		childiterator = (RequalityChildIterator) RequalityChildIterator.getOne(null, getUUId(), RequalityIteratorType.CHILD);
    		if(childiterator.getMasterListener()!=null)
    			addChangeListener(childiterator.getMasterListener());
    	}
    	return childiterator;
    }
    @Override
    protected void removeChildUUID(UUID uid){
    	//getVirtualTreeDB().getNode(uid).setParent(this);
    }
    @Override
    protected void addChildUUID(UUID uid){
    	//getVirtualTreeDB().getNode(uid).setParent(this);
    }
    public void removeClones(){
    	List<UUID> clones = getClones(true);
    	childrenLoaded = false;
		if(clones!=null)
			for(UUID uid:clones){
				getVirtualTreeDB().getNode(uid).delete();
				//getVirtualTreeDB().getNode(uid).fireNodeDelete();
				//getVirtualStorage().remove(uid,true);
			}
    }
    public void setChildIterator(RequalityChildIterator rci) {
    	String childIteratorNow = getStringAttributeValue(ATTR_CHILD_ITERATOR);
    	String toSave = rci.toAttributeString();
    	if(childIteratorNow == null || !childIteratorNow.equals(toSave)){
    		putAttribute(new Attribute(this, AttributeType.STRING, ATTR_CHILD_ITERATOR, toSave));
    	}
    }
    @Override
    public void setChildrenUUIDs(java.util.List<UUID> children2) {
    	super.setChildrenUUIDs(children2);
    	if(childiterator!=null){
    		children = new ArrayList<UUID> (getClones(true));
    		outchildren = updateVisablility(getVirtualTreeDB(),children);
    		getTreeDB().updateNodeChildren(getParent(true));
    	}
    };
    public static TreeDB getRootTree(TreeDB db){
    	if(revInstanceTreeDBs.containsKey(db))
    		return getRootTree(revInstanceTreeDBs.get(db));
    	return db;
    }
    public static TreeDB getBaseTree(TreeDB fakedb){
    	return revInstanceTreeDBs.get(fakedb);
    }

//    private void reloadStorage(){
//    	initStorageOver(getTarget());
//    }
    @Override
	public void reactAfterAttributeChange(String s,Attribute oldAttribute, Attribute attribute) {
    	super.reactAfterAttributeChange(s,oldAttribute, attribute);
    	if(s.equals(ATTR_TARGET)){
    		String value = attribute.getValue().toString();
    		if(value.equals(""))
    			removeClones();
    		TreeNode oldt = target;
    		target = value.equals("")?null:getTreeDB().getNode(value);
    		childrenLoaded = false;
    		if(oldt==null && target!=null || oldt!=null && !oldt.equals(target)){
    			if(oldt!=null)
    				oldt.removeChangeListener(targetChange);
    			updateChildrenIfNeeded();
    			if(target!=null)
    				target.addChangeListener(targetChange);
    		}
    	}
    	if(s.equals(ATTR_CHILD_ITERATOR)){
    		String old = null;
    		if(childiterator!=null && childiterator.getMasterListener()!=null){
        		removeChangeListener(childiterator.getMasterListener());
        		old = childiterator.toAttributeString();
    		}
    		childiterator = null;
    		if(old != null&& !old.equals(attribute.getValue().toString()))
    			removeClones();
    		if(attribute!=null){
    			childiterator = RequalityChildIterator.fromAttributeString(null, this, attribute.getValue().toString());
    			childiterator.updateDependencies(getTreeDB());
    			updateChildrenIfNeeded();
    			if(childiterator!=null && childiterator.getMasterListener()!=null)
    				addChangeListener(childiterator.getMasterListener());
    		}
    		childrenLoaded = false;
    	}
    	if(s.equals(ATTR_HIDDEN_STATUS)){
    		try{
            	TreeNode par = getParent(true);
            	if(!par.isVirtual()){
            		TreeDB db = VirtualNode.getBaseTree(getTreeDB());
            		if(db == null)
            			db = getTreeDB();
            		db.updateNodeChildren(par);
            	}else
            		getTreeDB().updateNodeChildren(par);
            }
            catch(RuntimeException re){
            	re.printStackTrace();
            	//parent not found
            }	
    	}
    };
    public void setTarget(TreeNode node) throws Exception{
    	if(node != null && getQualifiedId().contains(node.getQualifiedId()+"/"))
    		throw new RuntimeException("Recursion on targeting mechanism is prohibited");
    	//if(target != null)
    		//target.removeChangeListener(targetChange);
    	boolean changes = true;
    	if(target!=null)
    		changes = target != node;
    	//if(target!=null)
    		//target.addChangeListener(targetChange);
    	if(changes){
    		String s = getStringAttributeValue(ATTR_TARGET);
        	if(node == null || !node.getQualifiedId().equals(s)){
        		putAttribute(new Attribute(this,AttributeType.STRING, ATTR_TARGET, node!=null?node.getQualifiedId():""));
        	//	reloadStorage();
        	}
    	}
    }
    
    public TreeNode getTarget(){
    	String tid = (String) getAttributeValue(ATTR_TARGET);
    	try{
    		if(target == null && !tid.equals("")){
    			target = getTreeDB().getNode(tid);
        		if(target!=null)
        			target.addChangeListener(targetChange);
    		}
        }catch(Exception e){
    	}
    	return target;
    }
    public int getCloneCount(){
    	if (getTarget() == null)
    		return 0;
     	return getClones().size();
    }
    private static TreesTracker getTracker(TreeDB db){
    	if(!(db.getStorage() instanceof FSStorage)){
    		ITreeStorage ts = db.getStorage();
    		while(ts instanceof TransactionStorage)
    			ts = ((TransactionStorage)ts).getStorage();
    		if(!(ts instanceof FSStorage))
    			return TreesTracker.getTracker();
    	}
    	return null;

    }
	public boolean hide;
	public TreeDB getVirtualTreeDB() {
 		TreeDB db = getTreeDB();
		if(db.isVirtual())
			return db;
		return getVirtualTreeDB(getTreeDB());
	}
	public static TreeDB getVirtualTreeDB(TreeDB db) {
		TreesTracker tt = getTracker(db);
		if(tt!=null){
			IProject p = tt.getProject(db);
			if(p!=null)
				db = tt.getTree(p);
		}

		if(!instanceTreeDBs.containsKey(db)){
			//ti.setAttribute(node.getUUId(), new Attribute(node, AttributeType.STRING,TreeNode.ATTR_TYPE,"none"));
			TreeDB instanceDB = TreeDB.load(getVirtualStorage(db));
			//instanceTreeDBs.put(instanceDB, getTreeDB());
			instanceTreeDBs.put(db, instanceDB);
			revInstanceTreeDBs.put(instanceDB, db);
			instanceDB.setVirtualState(true);
				//TODO look it
			//ti.cloneUUID(this.getUUId(), this.getUUId(), "root");
			db.addListener(getOldToNewDBListener(db));
		}
		return instanceTreeDBs.get(db);
	}
	private static Map<TreeDB, ITreeChangeListener> oldtonewlistenters = new HashMap<TreeDB, ITreeChangeListener>();
	
	private static ITreeChangeListener getOldToNewDBListener(final TreeDB db) {
		if(!oldtonewlistenters.containsKey(db))
			oldtonewlistenters.put(db, new TreeChangeListener(){
				@Override
				public void changed(TreeNode node, Set<String> what) {
				}

				@Override
				public void created(TreeNode node) {
					TreeDB db = node.getTreeDB(), basedb = db;
					// TODO Auto-generated method stub
					List<UUID>ids = getVirtualStorage(db).getClones(node.getParentUUId(true)), 
							  worked = new ArrayList<UUID>();
					basedb.startChangeRow();
					for(UUID uid:ids){
						if(worked.contains(uid))
							continue;

						worked.add(uid);
						//TODO maybe move to actual update
						if(VirtualNode.TYPE_NAME.equals(getVirtualStorage(db).getNodeType(uid)))
							getVirtualStorage(db).cloneChild(node.getUUId(), uid, node.getId());
						getVirtualTreeDB(db).updateNodeChildren(getVirtualTreeDB(db).getNode(uid));
						//getVirtualTreeDB(db).fireNodeChange(getVirtualTreeDB(db).getNode(uid), true);
						//getVirtualTreeDB(db).fireNodeGUIChange(getVirtualTreeDB(db).getNode(uid), true);
					}
					//if(ids.isEmpty()){
						boolean stop = false;
						TreeNode parent = node.getParent(true);
						if(parent != null &&
								parent.sizeRawChildren()>0){
							List<UUID> childs = parent.getRawChildrenUUIDs();
							for(int i = 0 ;i<childs.size()&&!stop;i++){
								UUID uids = childs.get(i);
								ids = getVirtualStorage(db).getClones(uids);
								for(int j =0 ;j<ids.size();j++){
									UUID uid = ids.get(j);
									db = getVirtualTreeDB(db);
									TreeNode unode = db.getNode(uid);
									UUID par = unode.getParentUUId(true);
									if(par != null)
									{
										while(db!=null && !db.getStorage().canLoad(par))
											db = getBaseTree(db);
						//			if(!VirtualNode.TYPE_NAME.equals(db.getStorage().getNodeType(par))){
										TreeNode parN = db.getNode(par);
										if(worked.contains(par)||VirtualNode.TYPE_NAME.equals(parN.getType())&&
												(!((VirtualNode)parN).getChildIterator().getType().equals(RequalityIteratorType.CHILD)
												||!((VirtualNode)parN).getChildIterator().getChildIteratorType().equals(RequalityChildIteratorType.BASEREQ)))
											continue;
										getVirtualStorage(db).cloneChild(node.getUUId(), par, node.getId());
										worked.add(par);
									
										//if(!VirtualNode.getVirtualStorage(db).hasCloneParent(par))
											//db = VirtualNode.getBaseTree(db);
										db.updateNodeChildren(parN);
										//db.fireNodeChange(parN, true);
										//db.fireNodeGUIChange(parN, true);
									}
							//		}
								}
								if(ids.size()>0)
									stop = true;
							}
						}
					//}
					basedb.stopChangeRow();
				}

				@Override
				public void deleted(NodeDesc node) {
					delete(node.getUUId(),node.getParentUUId());
				}
				private void delete(UUID node, UUID parent){
					TreeDB db2=db;
					if(parent == null || !getVirtualStorage(db).canLoad(parent)){
						return;
					}
					TreeInstancer virtST = getVirtualStorage(db2);
					if(virtST.getClones(node).size()==0)
						return;
					getVirtualTreeDB(db).startChangeRow();
					TreeDB db = getVirtualTreeDB(db2);
					for(UUID uid:virtST.getClones(node)){
						TreeNode nde = db.getNode(uid);
						TreeNode par = nde.getParent(true);
						db2 = db;
						if(!getVirtualStorage(db2).hasCloneParent(par.getUUId())&&db2.isVirtual()){
							db2 = getBaseTree(db2);
							if(db2!=null)
								par = db2.getNode(par.getUUId());
						}
						nde.delete();
						//TODO vnode check
						getVirtualStorage(db).remove(nde.getUUId(),true);
						if(getVirtualStorage(db2).hasCloneParent(par.getUUId()))
							db2 = db;
						db2.updateNodeChildren(par);
						//db2.fireNodeChange(par, true);
						//db2.fireNodeGUIChange(par, true);
					}
					getVirtualTreeDB(db).stopChangeRow();

				}
				@Override
				public void moved(TreeNode node, TreeNode oldParent) {
					TreeDB db2 = db.isVirtual()?getBaseTree(db):db;
					db.startChangeRow();
					delete(node.getUUId(), oldParent.getUUId());
					created(node);
					for(UUID uid : findParent(oldParent.getUUId())){
						TreeNode par = db2.getNode(uid);
						if(VirtualNode.TYPE_NAME.equals(par.getType())&&((VirtualNode)par).getChildIterator().getChildIteratorType().equals(RequalityChildIteratorType.BASEREQ))
							((VirtualNode)par).updateChildrenIfNeeded(true);
						//if(!getVirtualStorage(db2).hasCloneParent(par.getUUId()))
							//db2 = db;
						par.getTreeDB().updateNodeChildren(par);
						//par.getTreeDB().fireNodeChange(par, true);
						//par.getTreeDB().fireNodeGUIChange(par, true);
					}
					for(UUID uid : findParent(node.getParentUUId(true))){
						TreeNode par = db2.getNode(uid);
						if(VirtualNode.TYPE_NAME.equals(par.getType())&&((VirtualNode)par).getChildIterator().getChildIteratorType().equals(RequalityChildIteratorType.BASEREQ))
							((VirtualNode)par).updateChildrenIfNeeded(true);
						//if(!getVirtualStorage(db2).hasCloneParent(par.getUUId()))
							//db2 = db;
						par.getTreeDB().updateNodeChildren(par);
						//par.getTreeDB().fireNodeChange(par, true);
						//par.getTreeDB().fireNodeGUIChange(par, true);
					}
					db.stopChangeRow();
				}

				private List<UUID> findParent(UUID parent) {
					List<UUID>ids = getVirtualStorage(db).getClones(parent), 
							  worked = new ArrayList<UUID>();
					for(UUID uid:ids){
						if(worked.contains(uid))
							continue;
						worked.add(uid);
					}
					for(UUID chi: db.getNode(parent).getChildrenUUIds()){
						for(UUID clone:getVirtualStorage(db).getClones(chi)){
							TreeNode cl = db.getNode(clone);
							if(!worked.contains(cl.getParentUUId()))
								worked.add(cl.getParentUUId());
						}
					}
					return worked;
				}

				@Override
				public void changed(List<TreeNodeChangeEvent> changes) {
					//TODO remade
					for(TreeNodeChangeEvent ce:changes)
					{
						switch (ce.getType()) {
							case DELETE:
								deleted(ce.getNodeDescTarget(), (Boolean)ce.getArguments()[0]);
							break;
							case CREATE:
								created(ce.getTreeNodeTarget(),(Boolean)ce.getArguments()[0]);
							break;
							case MOVE:
								moved(ce.getTreeNodeTarget(),(TreeNode)ce.getArguments()[1],(Boolean)ce.getArguments()[0]);
							break;
							case CHANGE:
								changed(ce.getTreeNodeTarget(),(Set<String>)ce.getArguments()[1],(Boolean)ce.getArguments()[0]);
							break;
						default:
							break;
						}
					}
				}
				
			});
		return oldtonewlistenters.get(db);
	}

	public static HashMap<TreeDB, TreeDB> getRealToVirtualDBMap() {
		return instanceTreeDBs;
	}
    
    public VirtualNode() {
        this(null, null);
    }
    
    private void updateChildrenIfNeeded(){
    	updateChildrenIfNeeded(false);
    }
    public static List<UUID> updateVisablility(TreeDB db, List<UUID> nodes){
    	if(nodes == null || nodes.size() == 0)
    		return nodes;
    	Set<TreeNode>toupdate = new HashSet<TreeNode>();
    	List<UUID> rnodes = new ArrayList<UUID>();
    	db.startChangeRow();
		if(nodes!=null)
			for(UUID uid:nodes){
				getVirtualStorage(db).setHidden(uid, false);
				TreeNode node = null;
				try{
					node = db.getNode(uid);
				}catch(Exception e){
				}
				if(node == null)
					continue;
				String s = "1";
				try{
					s = getVirtualStorage(db).checkPredicate(node.getUUId(), getVirtualStorage(db).getPredicate(uid));
				}catch(Exception e){
					e.printStackTrace();
				}
				if((s.startsWith("0")||s.equals("false"))&&!node.getPredicate().equals("false")){ 
					boolean changes = !getVirtualStorage(db).isHidden(uid);
					getVirtualStorage(db).setHidden(uid, true);
					if(changes)
						toupdate.add(node.getParent(true));
				}else{
					boolean changes = getVirtualStorage(db).isHidden(uid);
					getVirtualStorage(db).setHidden(uid, false);
					if(changes)
						toupdate.add(node.getParent(true));
				}
				if(!getVirtualStorage(db).isHidden(uid))
					rnodes.add(uid);
			}
		//TODO look into next two lines
		//for(TreeNode nde:toupdate)
			//db.updateNodeChildren(nde);
		//if(changes)
			//getTreeDB().updateNodeChildren(this);
		db.stopChangeRow();
		return rnodes;
    }
    @Override
    public TreeNode findChild(String id) {
    	for(UUID clone:getClones(true)){
    		TreeNode node = getVirtualTreeDB().getNode(clone);
    		if(node.getStringAttributeValue("__cloneid").equals(id))
    			return node;
    	}
    	return null;
    }
    public void updateChildrenIfNeeded(boolean force){
    	if(!childrenLoaded || force){
    		childiterator = getChildIterator();
    		childrenLoaded = true;
        	if(childiterator != null){
        		//getTreeDB().startChangeRow();
        		getVirtualTreeDB().startChangeRow();
    			getChildIterator().updateChildren(getTreeDB());
    			List<UUID> clones = getClones(true);
    			if(clones!=null)
    				for(UUID uid:clones){
    					TreeNode node = getVirtualTreeDB().getNode(uid);
    					node.virtualReplaceNames();
    					node.saveAttributes();
    				}
    			getTreeDB().updateNodeChildren(this);
    			getVirtualTreeDB().stopChangeRow();
        	}
        	//getTreeDB().fireNodeChange(this, true);
        	//getTreeDB().fireNodeGUIChange(this, true);
        	//fireGUINodeAttributeChange(TreeNode.ATTR_UNDEFINED);
        	//getTreeDB().stopChangeRow();
    	}
    }

    public void delete(){
    	try {
			setTarget(null);
			saveAttributes();
		} catch (Exception e) {
			e.printStackTrace();
		}
    	super.delete();
    }
    @Override 
    public TreeNode getChild(int i) {
		updateChildrenIfNeeded();
		//int cnt = getRawChildrenUUID().size();
    	UUID uid = getCloneUUID(i);
    	if(uid == null)
    		return null;
		return getVirtualTreeDB().getNode(uid);
    };
    @Override
    public List<UUID> getRawChildrenUUIDs() {
    	updateChildrenIfNeeded();
    	List<UUID> clones = getClones(true);
		if(clones!=null)
			children = new ArrayList<UUID>(clones);
		return super.getRawChildrenUUIDs();
    }
    @Override 
    public java.util.List<UUID> getChildrenUUIds() {
		List<UUID> clones = getRawChildrenUUIDs();
		//outchildren = updateVisablility(getVirtualTreeDB(), children);
		outchildren = updateVisablility(getVirtualTreeDB(), children);
		return outchildren;
    };
    @Override 
    public int sizeChildren() {
		updateChildrenIfNeeded();
    	return getClones().size();
    };
    public static TreeInstancer getVirtualStorage(TreeDB db){
    	return getVirtualStorage(db,false);
    }
    public static TreeInstancer getVirtualStorage(TreeDB db,boolean forceNew){
    	return getVirtualStorage(db,forceNew,Collections.EMPTY_MAP);
    }
    public static TreeInstancer getVirtualStorage(TreeDB db,boolean forceNew,Map<String,Object>initpar){
    	return getVirtualStorage(db, forceNew, initpar, null);
    }

    public static TreeInstancer getVirtualStorage(TreeDB db,boolean forceNew,Map<String,Object>initpar,TreeInstancer oldti){
    	while(db.isVirtual())
    		db = getBaseTree(db);
    	TreeInstancer ti = null;
    	TreesTracker tt = getTracker(db);
    	if(tt!=null){
    		IProject p = tt.getProject(db);
    		if(p!=null)
    			db = tt.getTree(p);
    	}
    	if(!instanceStorages.containsKey(db)||forceNew){
    		if(oldti!=null)
    			ti = new TreeInstancer(db,initpar,oldti);
    		else
    			ti = new TreeInstancer(db,initpar);
    		if(forceNew)
    			return ti;
    		instanceStorages.put(db, ti);
    	}
    	return instanceStorages.get(db);
    }
    public TreeInstancer getVirtualStorage(){
    	return getVirtualStorage(getTreeDB(),false);
    }
    
    public UUID getCloneUUID(int i){
    	//updateChildrenIfNeeded();
    	List<UUID> clones = getClones();
    	//in a case when child iterator does not iterate
    	if(i<clones.size())
    		return clones.get(i);
    	return null;
    }
    
    public VirtualNode(TreeDB db, UUID uuid) {
        super(db, uuid);
    }
	public void saveChildIterator()  {
		setChildIterator(getChildIterator());
	}
	public void setHiddenState(boolean state){
		putAttribute(new Attribute(this, AttributeType.BOOL, ATTR_HIDDEN_STATUS, state));
		saveAttributes();
	}
	public boolean isHidden() {
		boolean hiddenLayer;
		Object value = getAttributeValue(ATTR_HIDDEN_STATUS);
		if(value instanceof Boolean)
			hiddenLayer = (Boolean)value;
		else if(value instanceof String)
			hiddenLayer = ((String)value).toLowerCase().equals("true");
		else
			hiddenLayer = false;
		return hiddenLayer;
	}
	public List<UUID> getClones() {
		return getClones(false);
	}
	public List<UUID> getClones(boolean raw) {
		List<UUID> childs =  getVirtualStorage().getRepChildren(getUUId()),
				   tosee = new ArrayList<UUID>(childs);
		TreeDB db = getTreeDB();
		if(!db.isVirtual())
			db = VirtualNode.getVirtualTreeDB(db);
		if(!raw)
			for(UUID uid:tosee){
				TreeNode node = db.getNode(uid);
				String s = node.evaluatePredicateString(node,getVirtualStorage().getPredicate(uid));
				if(s.equals(""))
					continue;
				if(s.startsWith("0")||s.equals("false"))
					childs.remove(uid);
			}
    	return childs;
	}
	public static void updateClonesAttributes(TreeNode treeNode,
			Map<String, Attribute> adi,boolean notchange) {
		Map<String, Attribute> adiff = new HashMap<String, Attribute> (adi);
		TreeDB db = treeNode.getTreeDB();
		if(!db.isVirtual())
			db = getVirtualTreeDB(db);
		TreeInstancer ti = getVirtualStorage(db);
		List<UUID> list = ti.getClones(treeNode.getUUId());
		if(list.size()==0)
			return;
		List<String> toup = new ArrayList<String>();
		List<Attribute> toput = new ArrayList<Attribute>();
		List<String> torem = new ArrayList<String>();
		Map<String, Attribute> baseAttr = treeNode.getAttributes();
		db.startChangeRow();
		for (UUID id : list){
			toup.clear();
			toput.clear();
			torem.clear();
			if(!db.getStorage().canLoad(id))
				continue;
			TreeNode node = db.getNode(id);
			if(node.isDisposed()){
				node.delete();
				continue;
			}
			//node.updateGeneratedVars(true);
			Map<String,Attribute> ci = ti.getChangedAttributes(id);
			Map<String,Attribute> diff = new HashMap<String,Attribute>();
			boolean changed = false;
			for(String k:adiff.keySet()){
				Attribute a = Attribute.deepCopy(adiff.get(k));
				if(ci!=null && ci.containsKey(k) && ci.get(k).getOrigin().equals(id)){
					//stay unchanged
					diff.put(k, ci.get(k));
					adiff.put(k, ci.get(k));
					/*Attribute b = ci.get(a.getKey());
					if(a.isMoreVisible(db, b)){
						if(a.isGenerated())
							a = node.getGeneratedAttribute(a.getKey());
						toput.add(a);
						toup.add(a.getKey());
						changed = true;
					}*/
				}else{
					if(a==null){
						Attribute temp = node.getAttribute(k);
						if(temp!=null){
							changed = true;
							torem.add(k);
						}
					} else{
						if(a.isGenerated())
							if(a.getGenerator()!=null)
								a.getGenerator().updateValue(node);
							else{
								a = null;
								torem.add(k);
							}
						if(a!=null){
							a = node.getUpdatedAttribute(a);
							toput.add(a);
							toup.add(k);
							changed = true;
						}
					}				
				}
			}
			if(changed){
				for(Attribute a:toput){
					diff.put(a.getKey(), node.putAttribute(Attribute.deepCopy(a),notchange,true));
					//because we change attribute on master node -> in this case
					//attributes on storage will be taken from master and
					//be equal to setted value. So, change needs to be called
					//manually
					//node.fireNodeAttributeChange(a.getKey());
				}
				for(String a:torem){
					diff.put(a,node.removeAttribute(node.getAttribute(a).getOrigin(),a, notchange, true));
					//AttributeGenerator ag = node.getGenerator(a);
					//if(ag!=null)
						//node.putAttribute(ag.getAttribute(node));
						//because we change attribute on master node -> in this case
						//attributes on storage will be taken from master and
						//be equal to setted value. So, change needs to be called
						//manually
				}
				/*for(String a:torem)
					node.fireGUINodeAttributeChange(a);
				for(Attribute a:toput)
					node.fireGUINodeAttributeChange(a.getKey());*/
				//db.fireNodeChange(node, true);
				//db.fireNodeGUIChange(node, true);
			}

 			for(String a:adiff.keySet())
				node.reactAfterAttributeChange(a,diff.get(a),adiff.get(a));
			node.saveAttributes();
		}
		db.stopChangeRow();
	}
	public static void markInstanceDB(TreeDB instTree, TreeDB db,TreeInstancer ti) {
    	TreesTracker tt = getTracker(db);
    	if(tt!=null){
    		IProject p = tt.getProject(db);
    		if(p!=null)
    			db = tt.getTree(p);
    	}
   		instanceTreeDBs.put(db, instTree);
   		revInstanceTreeDBs.put(instTree, db);
   		instanceStorages.put(db, ti);
	}

	public static void updateClonesChildren(TreeNode treeNode) {
		TreeDB db = treeNode.getTreeDB();
		if(!db.isVirtual())
			db = getVirtualTreeDB(db);
		TreeInstancer ti = getVirtualStorage(db);
		List<UUID> clones = ti.getClones(treeNode.getUUId());
		//TODO needed?
		if(!treeNode.isVirtual())
			db.getNode(treeNode.getUUId()).setChildrenUUIDs(treeNode.getRawChildrenUUIDs());
		for(UUID uid:clones)
			db.updateNodeChildren(db.getNode(uid));
	}
	public static void clearInstanceInfo(TreeDB instTree, TreeDB db,
			TreeInstancer ti) {
		TreesTracker tt = getTracker(db);
    	if(tt!=null){
    		IProject p = tt.getProject(db);
    		if(p!=null)
    			db = tt.getTree(p);
    	}
		instanceTreeDBs.remove(db);
   		revInstanceTreeDBs.remove(instTree);
   		instanceStorages.remove(db);
	}
    @Override
    public boolean hasUserAttributes() {
    	return true;
    }
}

